@@ -31,7 +31,6 @@
 
 #include "libbalsa.h"
 #include "misc.h"
-#include "mime-stream-shared.h"
 #include "i18n.h"
 
 LibBalsaMessageBody *
@@ -273,7 +272,7 @@ libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
    allocates a temporary file name and saves the body there.
 */
 gboolean
-libbalsa_message_body_save_temporary(LibBalsaMessageBody * body)
+libbalsa_message_body_save_temporary(LibBalsaMessageBody * body, GError **err)
 {
     if (body->temp_filename == NULL) {
 	gint count = 100; /* Magic number, same as in g_mkstemp. */
@@ -286,17 +285,17 @@ libbalsa_message_body_save_temporary(LibBalsaMessageBody * body)
 	    gint fd;
 	    gchar *tmp_file_name;
 	    gchar *dotpos = NULL;
-	    GError *err = NULL;
 
 	    fd = g_file_open_tmp("balsa-body-XXXXXX", &tmp_file_name,
-				 &err);
-	    if (err) {
-		printf("libbalsa_message_body_save_temporary:\n %s\n",
-		       err->message);
-		g_error_free(err);
-	    }
+				 err);
 	    if (fd < 0)
 		return FALSE;
+	    if (err && *err  &&  /* We should have returned by now! */
+                (*err)->message) {
+		printf("libbalsa_message_body_save_temporary:\n %s\n",
+		       (*err)->message);
+		g_clear_error(err);
+	    }
 	    close(fd);
 	    unlink(tmp_file_name);
 
@@ -311,9 +310,10 @@ libbalsa_message_body_save_temporary(LibBalsaMessageBody * body)
 		g_free(tmp_file_name);
 	    } else
 		body->temp_filename = tmp_file_name;
-	    fd = open(body->temp_filename, O_WRONLY | O_EXCL | O_CREAT, 0600);
+	    fd = open(body->temp_filename, O_WRONLY | O_EXCL | O_CREAT,
+                      LIBBALSA_MESSAGE_BODY_SAFE);
 	    if (fd >= 0)
-		return libbalsa_message_body_save_fd(body, fd, FALSE);
+		return libbalsa_message_body_save_fd(body, fd, FALSE, err);
 	} while (errno == EEXIST && --count > 0);
 
 	/* Either we hit a real error, or we used up 100 attempts. */
@@ -329,7 +329,8 @@ libbalsa_message_body_save_temporary(LibBalsaMessageBody * body)
 	    return TRUE;
 	else
 	    return libbalsa_message_body_save(body, body->temp_filename,
-                                              FALSE);
+                                              LIBBALSA_MESSAGE_BODY_SAFE,
+                                              FALSE, err);
     }
 }
 
@@ -339,7 +340,9 @@ libbalsa_message_body_save_temporary(LibBalsaMessageBody * body)
 */
 gboolean
 libbalsa_message_body_save(LibBalsaMessageBody * body,
-			   const gchar * filename, gboolean filter_crlf)
+			   const gchar * filename, mode_t mode,
+                           gboolean filter_crlf,
+                           GError **err)
 {
     int fd;
     int flags = O_CREAT | O_EXCL | O_WRONLY;
@@ -348,9 +351,9 @@ libbalsa_message_body_save(LibBalsaMessageBody * body,
     flags |= O_NOFOLLOW;
 #endif
 
-    if ((fd=libbalsa_safe_open(filename, flags)) < 0)
+    if ((fd=libbalsa_safe_open(filename, flags, mode)) < 0)
 	return FALSE;
-    return libbalsa_message_body_save_fd(body, fd, filter_crlf);
+    return libbalsa_message_body_save_fd(body, fd, filter_crlf, err);
 }
 
 static GMimeStream *
@@ -371,7 +374,7 @@ libbalsa_message_body_stream_add_filter(GMimeStream * stream,
 }
 
 GMimeStream *
-libbalsa_message_body_get_stream(LibBalsaMessageBody * body)
+libbalsa_message_body_get_stream(LibBalsaMessageBody * body, GError **err)
 {
     GMimeStream *stream;
     GMimeFilter *filter;
@@ -381,11 +384,13 @@ libbalsa_message_body_get_stream(LibBalsaMessageBody * body)
     g_return_val_if_fail(body != NULL, NULL);
     g_return_val_if_fail(body->message != NULL, NULL);
 
-    if (body->message->mailbox
-        && libbalsa_mailbox_get_message_part(body->message, body)) {
+    if (body->message->mailbox) {
         GMimeDataWrapper *wrapper;
         GMimePartEncodingType encoding;
 
+        if(!libbalsa_mailbox_get_message_part(body->message, body, err))
+            return NULL;
+
         wrapper =
             g_mime_part_get_content_object(GMIME_PART(body->mime_part));
         stream = g_mime_data_wrapper_get_stream(wrapper);
@@ -418,21 +423,21 @@ libbalsa_message_body_get_stream(LibBalsaMessageBody * body)
     } else if (body->mime_part) {
         /* Not a GMimePart... */
         GMimeObject *object = body->mime_part;
-        GMimeStream *tmp = NULL;
         if (GMIME_IS_MESSAGE_PART(object))
             object = GMIME_OBJECT(g_mime_message_part_get_message
                                   ((GMimeMessagePart *) object));
         else
             g_object_ref(object);
         stream = g_mime_stream_mem_new();
-        tmp = libbalsa_message_stream(body->message);
-        libbalsa_mime_stream_shared_lock(tmp);
+        libbalsa_mailbox_lock_store(body->message->mailbox);
         g_mime_object_write_to_stream(object, stream);
-        libbalsa_mime_stream_shared_unlock(tmp);
-        g_object_unref(tmp);
+        libbalsa_mailbox_unlock_store(body->message->mailbox);
         g_object_unref(object);
-    } else
+    } else {
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_ACCESS_ERROR,
+                    "Internal error in get_stream");
         return NULL;
+    }
 
     /* convert text bodies but HTML - gtkhtml does conversion on its own. */
     if (libbalsa_message_body_type(body) == LIBBALSA_MESSAGE_BODY_TYPE_TEXT
@@ -452,10 +457,10 @@ libbalsa_message_body_get_stream(LibBalsaMessageBody * body)
         g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream_filter),
                                  filter_windows);
 
-        libbalsa_mime_stream_shared_lock(stream);
+        libbalsa_mailbox_lock_store(body->message->mailbox);
         g_mime_stream_reset(stream);
         g_mime_stream_write_to_stream(stream, stream_filter);
-        libbalsa_mime_stream_shared_unlock(stream);
+        libbalsa_mailbox_unlock_store(body->message->mailbox);
         g_object_unref(stream_filter);
 
         charset = g_mime_filter_windows_real_charset(GMIME_FILTER_WINDOWS
@@ -475,7 +480,8 @@ libbalsa_message_body_get_stream(LibBalsaMessageBody * body)
 }
 
 gssize
-libbalsa_message_body_get_content(LibBalsaMessageBody * body, gchar ** buf)
+libbalsa_message_body_get_content(LibBalsaMessageBody * body, gchar ** buf,
+                                  GError **err)
 {
     GMimeStream *stream, *stream_mem;
     GByteArray *array;
@@ -486,7 +492,7 @@ libbalsa_message_body_get_content(LibBalsaMessageBody * body, gchar ** buf)
     g_return_val_if_fail(buf != NULL, -1);
 
     *buf = NULL;
-    stream = libbalsa_message_body_get_stream(body);
+    stream = libbalsa_message_body_get_stream(body, err);
     if (!stream)
         return -1;
 
@@ -494,10 +500,10 @@ libbalsa_message_body_get_content(LibBalsaMessageBody * body, gchar ** buf)
     stream_mem = g_mime_stream_mem_new_with_byte_array(array);
     g_mime_stream_mem_set_owner(GMIME_STREAM_MEM(stream_mem), FALSE);
 
-    libbalsa_mime_stream_shared_lock(stream);
+    libbalsa_mailbox_lock_store(body->message->mailbox);
     g_mime_stream_reset(stream);
     len = g_mime_stream_write_to_stream(stream, stream_mem);
-    libbalsa_mime_stream_shared_unlock(stream);
+    libbalsa_mailbox_unlock_store(body->message->mailbox);
     g_object_unref(stream);
     g_object_unref(stream_mem);
 
@@ -523,10 +529,10 @@ libbalsa_message_body_get_pixbuf(LibBalsaMessageBody * body, GError ** err)
     gchar buf[4096];
     GdkPixbuf *pixbuf = NULL;
 
-    stream = libbalsa_message_body_get_stream(body);
+    stream = libbalsa_message_body_get_stream(body, err);
     if (!stream)
         return pixbuf;
-    libbalsa_mime_stream_shared_lock(stream);
+    libbalsa_mailbox_lock_store(body->message->mailbox);
     g_mime_stream_reset(stream);
 
     loader = gdk_pixbuf_loader_new();
@@ -536,7 +542,7 @@ libbalsa_message_body_get_pixbuf(LibBalsaMessageBody * body, GError ** err)
             break;
         }
     }
-    libbalsa_mime_stream_shared_unlock(stream);
+    libbalsa_mailbox_unlock_store(body->message->mailbox);
     g_object_unref(stream);
 
     if (!gdk_pixbuf_loader_close(loader, ok ? err : NULL))
@@ -553,38 +559,56 @@ libbalsa_message_body_get_pixbuf(LibBalsaMessageBody * body, GError ** err)
 
 gboolean
 libbalsa_message_body_save_fd(LibBalsaMessageBody * body, int fd,
-                              gboolean filter_crlf)
+                              gboolean filter_crlf, GError ** err)
 {
     GMimeStream *stream, *stream_fs;
-    gboolean retval = TRUE;
+    ssize_t len;
+
+    stream = libbalsa_message_body_get_stream(body, err);
+    if (!body->mime_part)
+        return FALSE;
 
-    stream = libbalsa_message_body_get_stream(body);
-    libbalsa_mime_stream_shared_lock(stream);
-    g_mime_stream_reset(stream);
     stream_fs = g_mime_stream_fs_new(fd);
+    libbalsa_mailbox_lock_store(body->message->mailbox);
 
-    if (filter_crlf) {
-        GMimeFilter *filter;
+    if (stream) {
+        g_mime_stream_reset(stream);
 
-        if (!GMIME_IS_STREAM_FILTER(stream)) {
-            GMimeStream *stream_filter =
-                g_mime_stream_filter_new_with_stream(stream);
-            g_object_unref(stream);
-            stream = stream_filter;
+        if (filter_crlf) {
+            GMimeFilter *filter;
+
+            if (!GMIME_IS_STREAM_FILTER(stream)) {
+                GMimeStream *stream_filter =
+                    g_mime_stream_filter_new_with_stream(stream);
+                g_object_unref(stream);
+                stream = stream_filter;
+            }
+
+            filter = g_mime_filter_crlf_new(GMIME_FILTER_CRLF_DECODE,
+                                            GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
+            g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream), filter);
+            g_object_unref(filter);
         }
-        filter = g_mime_filter_crlf_new(GMIME_FILTER_CRLF_DECODE,
-                                        GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
-        g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream), filter);
-        g_object_unref(filter);
+
+        len = g_mime_stream_write_to_stream(stream, stream_fs);
+        g_object_unref(stream);
+    } else {
+        /* body->mime_part is not a GMimePart. */
+        GMimeObject *mime_part = body->mime_part;
+
+        if (GMIME_IS_MESSAGE_PART(mime_part))
+            /* The user probably wants the message without the part's
+             * mime headers. */
+            mime_part =
+                GMIME_OBJECT(GMIME_MESSAGE_PART(mime_part)->message);
+
+        len = g_mime_object_write_to_stream(mime_part, stream_fs);
     }
 
-    if (g_mime_stream_write_to_stream(stream, stream_fs) < 0)
-	retval = FALSE;
-    libbalsa_mime_stream_shared_unlock(stream);
-    g_object_unref(stream);
+    libbalsa_mailbox_unlock_store(body->message->mailbox);
     g_object_unref(stream_fs);
 
-    return retval;
+    return len >= 0;
 }
 
 gchar *
@@ -710,3 +734,28 @@ libbalsa_message_body_get_by_id(LibBalsaMessageBody * body,
 
     return libbalsa_message_body_get_by_id(body->next, id);
 }
+
+
+#ifdef HAVE_GPGME
+LibBalsaMsgProtectState
+libbalsa_message_body_protect_state(LibBalsaMessageBody *body)
+{
+    if (!body || !body->sig_info ||
+	body->sig_info->status == GPG_ERR_NOT_SIGNED ||
+	body->sig_info->status == GPG_ERR_CANCELED)
+	return LIBBALSA_MSG_PROTECT_NONE;
+
+    if (body->sig_info->status == GPG_ERR_NO_ERROR) {
+	/* good signature, check if the validity and trust (OpenPGP only) are
+	   at least marginal */
+	if (body->sig_info->validity >= GPGME_VALIDITY_MARGINAL &&
+	    (body->sig_info->protocol == GPGME_PROTOCOL_CMS ||
+	     body->sig_info->trust >= GPGME_VALIDITY_MARGINAL))
+	    return LIBBALSA_MSG_PROTECT_SIGN_GOOD;
+	else
+	    return LIBBALSA_MSG_PROTECT_SIGN_NOTRUST;
+    }
+    
+    return LIBBALSA_MSG_PROTECT_SIGN_BAD;
+}
+#endif
