@@ -282,8 +282,8 @@ gboolean libbalsa_condition_is_flag_only(LibBalsaCondition * cond,
                                          guint msgno, gboolean * match);
 
 /* Compatibility */
-LibBalsaCondition *libbalsa_condition_new_2_0(gchar * prefix,
-					      gchar * filter_section_name,
-					      ConditionMatchType cmt);
+LibBalsaCondition *libbalsa_condition_new_2_0(const gchar *
+                                              filter_section_name,
+                                              ConditionMatchType cmt);
 
 #endif				/* __FILTER_H__ */
