@@ -35,7 +35,6 @@ static gchar*
 coalesce_seq_range(int lo, int hi, CoalesceFunc incl, void *data)
 {
   GString * res = g_string_sized_new(16);
-  char buf[10], *str;
   enum { BEGIN, LASTOUT, LASTIN, RANGE } mode = BEGIN;
   int seq;
   unsigned prev =0, num = 0;
@@ -44,11 +43,11 @@ coalesce_seq_range(int lo, int hi, CoalesceFunc incl, void *data)
     if(seq<=hi && (num=incl(seq, data)) != 0) {
       switch(mode) {
       case BEGIN: 
-        sprintf(buf, "%u", num); g_string_append(res, buf); 
+        g_string_append_printf(res, "%u", num);
         mode = LASTIN; break;
       case RANGE:
         if(num!=prev+1) {
-          sprintf(buf, ":%u,%u", prev, num); g_string_append(res, buf); 
+          g_string_append_printf(res, ":%u,%u", prev, num);
           mode = LASTIN;
         }
         break;
@@ -58,7 +57,7 @@ coalesce_seq_range(int lo, int hi, CoalesceFunc incl, void *data)
           break;
         } /* else fall through */
       case LASTOUT: 
-        sprintf(buf, ",%u", num); g_string_append(res, buf); 
+        g_string_append_printf(res, ",%u", num);
         mode = LASTIN; break;
       }
     } else {
@@ -67,21 +66,14 @@ coalesce_seq_range(int lo, int hi, CoalesceFunc incl, void *data)
       case LASTOUT: break;
       case LASTIN: mode = LASTOUT; break;
       case RANGE: 
-        sprintf(buf, ":%u", prev); g_string_append(res, buf); 
+        g_string_append_printf(res, ":%u", prev);
         mode = LASTOUT;
         break;
       }
     }
     prev = num;
   }
-  if(mode == BEGIN) {
-    str = NULL;
-    g_string_free(res, TRUE);
-  } else {
-    str = res->str;
-    g_string_free(res, FALSE);
-  }
-  return str;
+  return g_string_free(res, mode == BEGIN);
 }
 
 static unsigned
@@ -901,7 +893,8 @@ imap_mbox_handle_fetch_set(ImapMboxHandle* handle,
 static void
 write_nstring(unsigned seqno, const char *str, int len, void *fl)
 {
-  fwrite(str, 1, len, (FILE*)fl);
+  if (fwrite(str, 1, len, (FILE*)fl) != (size_t) len)
+    perror("write_nstring");
 }
 
 ImapResponse
@@ -1241,7 +1234,7 @@ imap_mbox_sort_filter(ImapMboxHandle *handle, ImapSortKey key, int ascending,
       handle->mbox_view.entries = 0; /* FIXME: I do not like this! 
                                       * we should not be doing such 
                                       * low level manipulations here */
-      rc = imap_search_exec(handle, filter, append_no, NULL);
+      rc = imap_search_exec(handle, FALSE, filter, append_no, NULL);
     } else {
       if(handle->thread_root)
         g_node_destroy(handle->thread_root);
@@ -1309,8 +1302,8 @@ ImapResponse
 imap_mbox_filter_msgnos(ImapMboxHandle *handle, ImapSearchKey *filter,
 			GHashTable *msgnos)
 {
-  return imap_search_exec(handle, filter, (ImapSearchCb)make_msgno_table,
-                          msgnos);
+  return imap_search_exec(handle, FALSE, filter,
+                          (ImapSearchCb)make_msgno_table, msgnos);
 }
 
 /* imap_mbox_complete_msgids:
