@@ -148,6 +148,8 @@ imap_mbox_handle_init(ImapMboxHandle *handle)
 
   handle->info_cb  = NULL;
   handle->info_arg = NULL;
+  handle->enable_anonymous = 0;
+  handle->enable_binary    = 0;
   mbox_view_init(&handle->mbox_view);
 }
 
@@ -210,6 +212,16 @@ imap_mbox_handle_new(void)
   return handle;
 }
 
+void
+imap_handle_set_option(ImapMboxHandle *h, ImapOption opt, gboolean state)
+{
+  switch(opt) {
+  case IMAP_OPT_ANONYMOUS: h->enable_anonymous = !!state; break;
+  case IMAP_OPT_BINARY:    h->enable_binary    = !!state; break;
+  default: g_warning("imap_set_option: invalid option\n");
+  }
+}
+
 void
 imap_handle_set_infocb(ImapMboxHandle* h, ImapInfoCb cb, void *arg)
 {
@@ -552,8 +564,9 @@ imap_mbox_connect(ImapMboxHandle* handle)
     sio_set_monitorcb(handle->sio, handle->monitor_cb, handle->monitor_arg);
 
   handle->state = IMHS_CONNECTED;
-  if (imap_cmd_step(handle, 0) != IMR_UNTAGGED) {
-    g_message("imap_mbox_connect:unexpected initial response\n");
+  if ( (resp=imap_cmd_step(handle, 0)) != IMR_UNTAGGED) {
+    g_message("imap_mbox_connect:unexpected initial response(%d):\n%s\n",
+	      resp, handle->last_msg);
     imap_handle_disconnect(handle);
     return IMAP_PROTOCOL_ERROR;
   }
@@ -1328,7 +1341,7 @@ imap_cmd_step(ImapMboxHandle* handle, unsigned lastcmd)
   }
 #endif
   if( imap_cmd_get_tag(handle->sio, tag, sizeof(tag))<0) {
-    printf("connection severed.\n");
+    printf("imap connection severed.\n");
     imap_handle_disconnect(handle);
     return IMR_SEVERED;
   }
@@ -1367,9 +1380,10 @@ imap_cmd_step(ImapMboxHandle* handle, unsigned lastcmd)
   if(cmdno == lastcmd)
     return rc;
   else {
-    g_hash_table_insert(handle->cmd_queue,
-                        GUINT_TO_POINTER(cmdno),
-                        GINT_TO_POINTER(rc));
+    if(cmdno)
+      g_hash_table_insert(handle->cmd_queue,
+                          GUINT_TO_POINTER(cmdno),
+                          GINT_TO_POINTER(rc));
     return IMR_UNTAGGED;
   }
 }
@@ -1563,10 +1577,11 @@ ir_capability_data(ImapMboxHandle *handle)
 {
   /* ordered identically as ImapCapability constants */
   static const char* capabilities[] = {
-    "IMAP4", "IMAP4rev1", "STATUS", "ACL", "NAMESPACE", "AUTH=CRAM-MD5", 
-    "AUTH=GSSAPI", "AUTH=ANONYMOUS", "STARTTLS", "LOGINDISABLED", "SORT",
-    "THREAD=ORDEREDSUBJECT", "THREAD=REFERENCES", "UNSELECT", "SCAN",
-    "CHILDREN", "LITERAL+", "IDLE", "SASL-IR"
+    "IMAP4", "IMAP4rev1", "STATUS", "ACL", "NAMESPACE",
+    "AUTH=ANONYMOUS", "AUTH=CRAM-MD5", "AUTH=GSSAPI", "AUTH=PLAIN",
+    "STARTTLS", "LOGINDISABLED", "SORT",
+    "THREAD=ORDEREDSUBJECT", "THREAD=REFERENCES",
+    "UNSELECT", "SCAN", "CHILDREN", "LITERAL+", "IDLE", "SASL-IR"
   };
   unsigned x;
   int c;
@@ -1804,13 +1819,13 @@ ir_status(ImapMboxHandle *h)
 
   name = imap_get_astring(h->sio, &c);
   resp = g_hash_table_lookup(h->status_resps, name);
-  if(c                != ' ') return IMR_PROTOCOL;
-  if(sio_getc(h->sio) != '(') return IMR_PROTOCOL;
+  if(c                != ' ') {g_free(name); return IMR_PROTOCOL;}
+  if(sio_getc(h->sio) != '(') {g_free(name); return IMR_PROTOCOL;}
   do {
     char item[13], count[13]; /* longest than UIDVALIDITY */
     c = imap_get_atom(h->sio, item, sizeof(item));
     if(c == ')') break;
-    if(c != ' ') return IMR_PROTOCOL;
+    if(c != ' ') {g_free(name); return IMR_PROTOCOL;}
     c = imap_get_atom(h->sio, count, sizeof(count));
     /* FIXME: process the response */
     if(resp) {
@@ -1820,7 +1835,10 @@ ir_status(ImapMboxHandle *h)
           break;
       for(i= 0; resp[i].item != IMSTAT_NONE; i++) {
         if(resp[i].item == idx) {
-          sscanf(count, "%u", &resp[i].result);
+          if (sscanf(count, "%u", &resp[i].result) != 1) {
+            g_free(name);
+            return IMR_PROTOCOL;
+          }
           break;
         }
       }
@@ -2185,7 +2203,7 @@ static ImapResponse
 ir_msg_att_rfc822(ImapMboxHandle *h, int c, unsigned seqno)
 {
   gchar *str = imap_get_nstring(h->sio);
-  if(h->body_cb)
+  if(str && h->body_cb)
     h->body_cb(seqno, str, strlen(str), h->body_arg);
   g_free(str);
   return IMR_OK;
@@ -2495,7 +2513,7 @@ ir_body_extension (struct siobuf *sio, ImapBody * body)
     }
   else
     /* nstring */
-    imap_get_nstring (sio);
+    g_free(imap_get_nstring(sio));
 
   return IMR_OK;
 }
@@ -2835,9 +2853,11 @@ ir_body_section(struct siobuf *sio, unsigned seqno,
   if(c != ']') { puts("] expected"); return IMR_PROTOCOL; }
   if(sio_getc(sio) != ' ') { puts("space expected"); return IMR_PROTOCOL;}
   str = imap_get_nstring(sio);
-  if(body_cb)
-    body_cb(seqno, str, strlen(str), arg);
-  g_free(str);
+  if(str) {
+    if(body_cb)
+      body_cb(seqno, str, strlen(str), arg);
+    g_free(str);
+  }
   return IMR_OK;
 }
 
@@ -2862,7 +2882,7 @@ ir_body_header_fields(ImapMboxHandle *h, unsigned seqno)
 
   tmp = imap_get_nstring(h->sio);
   if(h->body_cb) {
-    h->body_cb(seqno, tmp, strlen(tmp), h->body_arg);
+    if(tmp) h->body_cb(seqno, tmp, strlen(tmp), h->body_arg);
     g_free(tmp);
   } else {
     CREATE_IMSG_IF_NEEDED(h, seqno);
