@@ -37,7 +37,6 @@
 #include <ldap.h>
 #endif
 
-#include "address-book-gpe.h"
 #include "libbalsa.h"
 #include "misc.h"
 #include "i18n.h"
@@ -149,15 +148,14 @@ libbalsa_guess_email_address(void)
     /* Q: Find this location with configure? or at run-time? */
     static const gchar* MAILNAME_FILE = "/etc/mailname";
     char hostbuf[512];
-    FILE *mailname_in;
+    FILE *mailname_in = NULL;
 
     gchar* preset, *domain;
     if(g_getenv("EMAIL") != NULL){                  /* 1. */
         preset = g_strdup(g_getenv("EMAIL"));
-    } else if( (mailname_in = fopen(MAILNAME_FILE, "r")) != NULL){ /* 2. */
-        fgets(hostbuf, sizeof(hostbuf)-1, mailname_in);
+    } else if( (mailname_in = fopen(MAILNAME_FILE, "r")) != NULL
+              && fgets(hostbuf, sizeof(hostbuf)-1, mailname_in)){ /* 2. */
         hostbuf[sizeof(hostbuf)-1] = '\0';
-        fclose(mailname_in);
         preset = g_strconcat(g_get_user_name(), "@", hostbuf, NULL);
         
     }else if((domain = libbalsa_get_domainname())){ /* 3. */
@@ -167,6 +165,8 @@ libbalsa_guess_email_address(void)
         gethostname(hostbuf, 511);
         preset = g_strconcat(g_get_user_name(), "@", hostbuf, NULL);
     }
+    if (mailname_in)
+        fclose(mailname_in);
     return preset;
 }
 
@@ -343,7 +343,7 @@ libbalsa_assure_balsa_dir(void)
 }
 
 gchar*
-libbalsa_date_to_gchar(const time_t *date, const gchar *date_string)
+libbalsa_date_to_utf8(const time_t *date, const gchar *date_string)
 {
     struct tm footime;
     gchar rettime[128];
@@ -355,7 +355,7 @@ libbalsa_date_to_gchar(const time_t *date, const gchar *date_string)
 
     strftime(rettime, sizeof(rettime), date_string, &footime);
 
-    return g_strdup(rettime);
+    return g_locale_to_utf8(rettime, -1, NULL, NULL, NULL);
 }
 
 LibBalsaMessageStatus
