@@ -87,7 +87,7 @@ typedef enum {
 typedef struct _SortTuple SortTuple;
 /* Sorting */
 struct _SortTuple {
-    gint offset;
+    guint offset;
     GNode *node;
 };
 
@@ -230,6 +230,9 @@ struct _LibBalsaMailbox {
 
     LibBalsaMailboxView *view;
     LibBalsaMailboxState state;
+
+    /* Whether to reassemble a message from its parts. */
+    gboolean no_reassemble;
 };
 
 /* Search iter */
@@ -253,7 +256,8 @@ struct _LibBalsaMailboxClass {
     gboolean (*open_mailbox) (LibBalsaMailbox * mailbox, GError **err);
     void (*close_mailbox) (LibBalsaMailbox * mailbox, gboolean expunge);
     LibBalsaMessage *(*get_message) (LibBalsaMailbox * mailbox, guint msgno);
-    void (*prepare_threading)(LibBalsaMailbox *mailbox, guint lo, guint hi);
+    void (*prepare_threading)(LibBalsaMailbox *mailbox, guint * msgnos,
+                              guint len);
     gboolean (*fetch_message_structure)(LibBalsaMailbox *mailbox,
                                         LibBalsaMessage * message,
                                         LibBalsaFetchFlag flags);
@@ -262,7 +266,8 @@ struct _LibBalsaMailboxClass {
     void (*release_message) (LibBalsaMailbox * mailbox,
 			     LibBalsaMessage * message);
     gboolean (*get_message_part) (LibBalsaMessage *message,
-				  LibBalsaMessageBody *part);
+				  LibBalsaMessageBody *part,
+                                  GError **err);
     GMimeStream *(*get_message_stream) (LibBalsaMailbox * mailbox,
                                         guint msgno);
 
@@ -277,8 +282,9 @@ struct _LibBalsaMailboxClass {
     void (*save_config) (LibBalsaMailbox * mailbox, const gchar * prefix);
     void (*load_config) (LibBalsaMailbox * mailbox, const gchar * prefix);
     gboolean (*sync) (LibBalsaMailbox * mailbox, gboolean expunge);
-    int (*add_message) (LibBalsaMailbox * mailbox, LibBalsaMessage * message,
-                        GError **err);
+    gboolean (*add_message) (LibBalsaMailbox * mailbox,
+                             GMimeStream * stream,
+                             LibBalsaMessageFlag flags, GError ** err);
     gboolean (*messages_change_flags) (LibBalsaMailbox * mailbox,
 				       GArray *msgnos,
 				       LibBalsaMessageFlag set,
@@ -300,6 +306,9 @@ struct _LibBalsaMailboxClass {
     gboolean (*close_backend)(LibBalsaMailbox * mailbox);
     guint (*total_messages)(LibBalsaMailbox * mailbox);
     GArray *(*duplicate_msgnos) (LibBalsaMailbox * mailbox);
+#if BALSA_USE_THREADS
+    void (*lock_store) (LibBalsaMailbox * mailbox, gboolean lock);
+#endif                          /* BALSA_USE_THREADS */
 };
 
 GType libbalsa_mailbox_get_type(void);
@@ -336,11 +345,11 @@ LibBalsaMessage *libbalsa_mailbox_get_message(LibBalsaMailbox * mailbox,
 					      guint msgno);
 
 /** libbalsa_mailbox_prepare_threading() requests prefetching of information
-    needed for client-size message threading.
-    lo and hi are related to currently set view.
+    needed for client-side message threading.
+    msgnos are related to currently set view.
 */
 void libbalsa_mailbox_prepare_threading(LibBalsaMailbox *mailbox,
-					guint lo, guint hi);
+					guint * msgnos, guint len);
 
 /** libbalsa_mailbox_fetch_message_structure() fetches detailed
     message structure for given message. It can also fetch all RFC822
@@ -367,7 +376,8 @@ void libbalsa_mailbox_set_msg_headers(LibBalsaMailbox * mailbox,
     part of the message is loaded.
 */
 gboolean libbalsa_mailbox_get_message_part(LibBalsaMessage    *message,
-					   LibBalsaMessageBody *part);
+					   LibBalsaMessageBody *part,
+                                           GError **err);
 
 /** libbalsa_mailbox_get_message_stream() returns a message stream associated
     with full RFC822 text of the message.
@@ -418,8 +428,10 @@ void libbalsa_mailbox_run_filters_on_reception(LibBalsaMailbox * mailbox);
 void libbalsa_mailbox_save_config(LibBalsaMailbox * mailbox,
 				  const gchar * prefix);
 
-int libbalsa_mailbox_copy_message(LibBalsaMessage *message,
-				  LibBalsaMailbox *dest, GError **err);
+gboolean libbalsa_mailbox_add_message(LibBalsaMailbox * mailbox,
+                                      GMimeStream * stream,
+                                      LibBalsaMessageFlag flags,
+                                      GError ** err);
 gboolean libbalsa_mailbox_close_backend(LibBalsaMailbox * mailbox);
 
 /* Message number-list methods */
@@ -530,7 +542,9 @@ time_t libbalsa_mailbox_get_mtime(LibBalsaMailbox * mailbox);
 
 /** force update of given msgno */
 void libbalsa_mailbox_msgno_changed(LibBalsaMailbox  *mailbox, guint seqno);
-void libbalsa_mailbox_msgno_inserted(LibBalsaMailbox *mailbox, guint seqno);
+void libbalsa_mailbox_msgno_inserted(LibBalsaMailbox * mailbox,
+                                     guint seqno, GNode * parent,
+                                     GNode ** sibling);
 void libbalsa_mailbox_msgno_removed(LibBalsaMailbox  *mailbox, guint seqno);
 void libbalsa_mailbox_msgno_filt_in(LibBalsaMailbox * mailbox, guint seqno);
 void libbalsa_mailbox_msgno_filt_out(LibBalsaMailbox * mailbox, guint seqno);
@@ -584,6 +598,22 @@ const gchar *libbalsa_mailbox_msgno_get_subject(LibBalsaMailbox * mailbox,
 void libbalsa_mailbox_msgno_update_attach(LibBalsaMailbox * mailbox,
 					  guint msgno,
 					  LibBalsaMessage * message);
+void libbalsa_mailbox_cache_message(LibBalsaMailbox * mailbox, guint msgno,
+                                    LibBalsaMessage * message);
+
+#if BALSA_USE_THREADS
+
+/* Lock and unlock the mail store--currently, a no-op except for mbox.
+ */
+void libbalsa_mailbox_lock_store  (LibBalsaMailbox * mailbox);
+void libbalsa_mailbox_unlock_store(LibBalsaMailbox * mailbox);
+
+#else                           /* BALSA_USE_THREADS */
+
+#define libbalsa_mailbox_lock_store(mailbox)
+#define libbalsa_mailbox_unlock_store(mailbox)
+
+#endif                          /* BALSA_USE_THREADS */
 
 /* columns ids */
 typedef enum {
