@@ -46,6 +46,11 @@ struct message_info {
     LibBalsaMessageFlag flags;  	/* May have pseudo-flags */
     LibBalsaMessageFlag orig_flags;     /* Has only real flags */
     LibBalsaMessage *message;
+
+    /* The message's order when parsing; needed for saving the message
+     * tree in a form that will match the msgnos when the mailbox is
+     * reopened. */
+    guint fileno;
 };
 #define REAL_FLAGS(flags) (flags & LIBBALSA_MESSAGE_FLAGS_REAL)
 #define FLAGS_REALLY_DIFFER(orig_flags, flags) \
@@ -58,14 +63,17 @@ static LibBalsaMailboxLocalClass *parent_class = NULL;
 
 static void libbalsa_mailbox_maildir_class_init(LibBalsaMailboxMaildirClass *klass);
 static void libbalsa_mailbox_maildir_init(LibBalsaMailboxMaildir * mailbox);
+
+/* Object class method */
 static void libbalsa_mailbox_maildir_finalize(GObject * object);
+
+/* Mailbox class methods */
 static void libbalsa_mailbox_maildir_load_config(LibBalsaMailbox * mailbox,
 						 const gchar * prefix);
 
 static GMimeStream *libbalsa_mailbox_maildir_get_message_stream(LibBalsaMailbox *
 							   mailbox,
 							   guint msgno);
-static void libbalsa_mailbox_maildir_remove_files(LibBalsaMailboxLocal *mailbox);
 
 static gboolean libbalsa_mailbox_maildir_open(LibBalsaMailbox * mailbox,
 					      GError **err);
@@ -75,18 +83,17 @@ static void libbalsa_mailbox_maildir_close_mailbox(LibBalsaMailbox *
 static void libbalsa_mailbox_maildir_check(LibBalsaMailbox * mailbox);
 static gboolean libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox,
                                               gboolean expunge);
-static struct message_info *message_info_from_msgno(
-						  LibBalsaMailbox * mailbox,
-						  guint msgno);
-static LibBalsaMessage *libbalsa_mailbox_maildir_get_message(LibBalsaMailbox * mailbox,
+static LibBalsaMessage *lbm_maildir_get_message(LibBalsaMailbox * mailbox,
 						     guint msgno);
 static gboolean
 libbalsa_mailbox_maildir_fetch_message_structure(LibBalsaMailbox * mailbox,
 						 LibBalsaMessage * message,
 						 LibBalsaFetchFlag flags);
-static int libbalsa_mailbox_maildir_add_message(LibBalsaMailbox * mailbox,
-						LibBalsaMessage * message,
-                                                GError **err);
+static gboolean libbalsa_mailbox_maildir_add_message(LibBalsaMailbox *
+                                                     mailbox,
+                                                     GMimeStream * stream,
+                                                     LibBalsaMessageFlag
+                                                     flags, GError ** err);
 static gboolean
 libbalsa_mailbox_maildir_messages_change_flags(LibBalsaMailbox * mailbox,
                                                GArray * msgnos,
@@ -100,10 +107,21 @@ libbalsa_mailbox_maildir_msgno_has_flags(LibBalsaMailbox * mailbox,
 static guint
 libbalsa_mailbox_maildir_total_messages(LibBalsaMailbox * mailbox);
 
+/* MailboxLocal class methods */
+static gint lbm_maildir_check_files(const gchar * path, gboolean create);
+static void lbm_maildir_set_path(LibBalsaMailboxLocal * mailbox,
+                                 const gchar * path);
+static void lbm_maildir_remove_files(LibBalsaMailboxLocal *mailbox);
+static LibBalsaMessageFlag lbm_maildir_load_message(LibBalsaMailbox * mailbox,
+						     guint msgno, 
+                                                     LibBalsaMessage **msg);
+static guint lbm_maildir_fileno(LibBalsaMailboxLocal * local, guint msgno);
+
 /* util functions */
+static struct message_info *message_info_from_msgno(LibBalsaMailboxMaildir
+                                                    * mdir, guint msgno);
 static LibBalsaMessageFlag parse_filename(const gchar *subdir,
 					  const gchar *filename);
-static void parse_mailbox(LibBalsaMailbox * mailbox, const gchar *subdir);
 static void free_message_info(struct message_info *msg_info);
 static int libbalsa_mailbox_maildir_open_temp (const gchar *dest_path,
 					  char **name_used);
@@ -161,7 +179,7 @@ libbalsa_mailbox_maildir_class_init(LibBalsaMailboxMaildirClass * klass)
     libbalsa_mailbox_class->sync = libbalsa_mailbox_maildir_sync;
     libbalsa_mailbox_class->close_mailbox =
 	libbalsa_mailbox_maildir_close_mailbox;
-    libbalsa_mailbox_class->get_message = libbalsa_mailbox_maildir_get_message;
+    libbalsa_mailbox_class->get_message = lbm_maildir_get_message;
     libbalsa_mailbox_class->fetch_message_structure =
 	libbalsa_mailbox_maildir_fetch_message_structure;
     libbalsa_mailbox_class->add_message = libbalsa_mailbox_maildir_add_message;
@@ -172,110 +190,110 @@ libbalsa_mailbox_maildir_class_init(LibBalsaMailboxMaildirClass * klass)
     libbalsa_mailbox_class->total_messages =
 	libbalsa_mailbox_maildir_total_messages;
 
-    libbalsa_mailbox_local_class->remove_files = 
-	libbalsa_mailbox_maildir_remove_files;
-    libbalsa_mailbox_local_class->load_message =
-        libbalsa_mailbox_maildir_get_message;
+    libbalsa_mailbox_local_class->check_files  = lbm_maildir_check_files;
+    libbalsa_mailbox_local_class->set_path     = lbm_maildir_set_path;
+    libbalsa_mailbox_local_class->remove_files = lbm_maildir_remove_files;
+    libbalsa_mailbox_local_class->load_message = lbm_maildir_load_message;
+    libbalsa_mailbox_local_class->fileno       = lbm_maildir_fileno;
 }
 
 static void
-libbalsa_mailbox_maildir_init(LibBalsaMailboxMaildir * mailbox)
+libbalsa_mailbox_maildir_init(LibBalsaMailboxMaildir * mdir)
 {
 }
 
-static void lbm_maildir_set_subdirs(LibBalsaMailboxMaildir * mdir,
-                                    const gchar * path);
-
-gint
-libbalsa_mailbox_maildir_create(const gchar * path, gboolean create,
-                                LibBalsaMailboxMaildir * mdir)
+static gint
+lbm_maildir_check_files(const gchar * path, gboolean create)
 {
-    gint exists;
-    GType magic_type;
+    g_return_val_if_fail(path != NULL, -1);
 
-    g_return_val_if_fail( path != NULL, -1);
+    if (access(path, F_OK) == 0) {
+        /* File exists. Check if it is a maildir... */
+        if (libbalsa_mailbox_type_from_path(path) !=
+            LIBBALSA_TYPE_MAILBOX_MAILDIR) {
+            libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                                 _("Mailbox %s does not appear to be a Maildir mailbox."),
+                                 path);
+            return -1;
+        }
+    } else if (create) {
+        char tmp[_POSIX_PATH_MAX];
 
-    exists = access(path, F_OK);
-    if ( exists == 0 ) {
-	/* File exists. Check if it is a maildir... */
-	
-	magic_type = libbalsa_mailbox_type_from_path(path);
-	if ( magic_type != LIBBALSA_TYPE_MAILBOX_MAILDIR ) {
-	    libbalsa_information(LIBBALSA_INFORMATION_WARNING, 
-				 _("Mailbox %s does not appear to be a Maildir mailbox."), path);
-	    return(-1);
-	}
-    } else {
-	if(create) {    
-	    char tmp[_POSIX_PATH_MAX];
-	    
-	    if (mkdir (path, S_IRWXU)) {
-		libbalsa_information(LIBBALSA_INFORMATION_WARNING, 
-				     _("Could not create a MailDir directory at %s (%s)"), path, strerror(errno) );
-		return (-1);
-	    }
-	    
-	    snprintf (tmp, sizeof (tmp), "%s/cur", path);
-	    if (mkdir (tmp, S_IRWXU)) {
-		libbalsa_information(LIBBALSA_INFORMATION_WARNING, 
-				     _("Could not create a MailDir at %s (%s)"), path, strerror(errno) );
-		rmdir (path);
-		return (-1);
-	    }
-	    
-	    snprintf (tmp, sizeof (tmp), "%s/new", path);
-	    if (mkdir (tmp, S_IRWXU)) {
-		libbalsa_information(LIBBALSA_INFORMATION_WARNING, 
-				     _("Could not create a MailDir at %s (%s)"), path, strerror(errno) );
-		snprintf (tmp, sizeof (tmp), "%s/cur", path);
-		rmdir (tmp);
-		rmdir (path);
-		return (-1);
-	    }
-	    
-	    snprintf (tmp, sizeof (tmp), "%s/tmp", path);
-	    if (mkdir (tmp, S_IRWXU)) {
-		libbalsa_information(LIBBALSA_INFORMATION_WARNING, 
-				     _("Could not create a MailDir at %s (%s)"), path, strerror(errno) );
-		snprintf (tmp, sizeof (tmp), "%s/cur", path);
-		rmdir (tmp);
-		snprintf (tmp, sizeof (tmp), "%s/new", path);
-		rmdir (tmp);
-		rmdir (path);
-		return (-1);
-	    }
-	} else 
-	    return(-1);
-    }
-    lbm_maildir_set_subdirs(mdir, path);
-    return(0);
+        if (mkdir(path, S_IRWXU)) {
+            libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                                 _("Could not create a MailDir directory at %s (%s)"),
+                                 path, strerror(errno));
+            return -1;
+        }
+
+        snprintf(tmp, sizeof(tmp), "%s/cur", path);
+        if (mkdir(tmp, S_IRWXU)) {
+            libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                                 _("Could not create a MailDir at %s (%s)"),
+                                 path, strerror(errno));
+            rmdir(path);
+            return -1;
+        }
+
+        snprintf(tmp, sizeof(tmp), "%s/new", path);
+        if (mkdir(tmp, S_IRWXU)) {
+            libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                                 _("Could not create a MailDir at %s (%s)"),
+                                 path, strerror(errno));
+            snprintf(tmp, sizeof(tmp), "%s/cur", path);
+            rmdir(tmp);
+            rmdir(path);
+            return -1;
+        }
+
+        snprintf(tmp, sizeof(tmp), "%s/tmp", path);
+        if (mkdir(tmp, S_IRWXU)) {
+            libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                                 _("Could not create a MailDir at %s (%s)"),
+                                 path, strerror(errno));
+            snprintf(tmp, sizeof(tmp), "%s/cur", path);
+            rmdir(tmp);
+            snprintf(tmp, sizeof(tmp), "%s/new", path);
+            rmdir(tmp);
+            rmdir(path);
+            return -1;
+        }
+    } else
+        return -1;
+
+    return 0;
 }
 
 static void
 lbm_maildir_set_subdirs(LibBalsaMailboxMaildir * mdir, const gchar * path)
 {
-    mdir->curdir = g_strdup_printf("%s/cur", path);
-    mdir->newdir = g_strdup_printf("%s/new", path);
-    mdir->tmpdir = g_strdup_printf("%s/tmp", path);
+    g_free(mdir->curdir);
+    mdir->curdir = g_build_filename(path, "cur", NULL);
+    g_free(mdir->newdir);
+    mdir->newdir = g_build_filename(path, "new", NULL);
+    g_free(mdir->tmpdir);
+    mdir->tmpdir = g_build_filename(path, "tmp", NULL);
+}
+
+static void
+lbm_maildir_set_path(LibBalsaMailboxLocal * local, const gchar * path)
+{
+    lbm_maildir_set_subdirs(LIBBALSA_MAILBOX_MAILDIR(local), path);
 }
 
 GObject *
 libbalsa_mailbox_maildir_new(const gchar * path, gboolean create)
 {
     LibBalsaMailbox *mailbox;
-    LibBalsaMailboxMaildir *mdir;
 
     mailbox = g_object_new(LIBBALSA_TYPE_MAILBOX_MAILDIR, NULL);
 
     mailbox->is_directory = TRUE;
 
-    LIBBALSA_MAILBOX(mailbox)->url = g_strconcat("file://", path, NULL);
-
-
-    mdir = LIBBALSA_MAILBOX_MAILDIR(mailbox);
-    if(libbalsa_mailbox_maildir_create(path, create, mdir) < 0) {
-	g_object_unref(G_OBJECT(mailbox));
-	return NULL;
+    if (libbalsa_mailbox_local_set_path(LIBBALSA_MAILBOX_LOCAL(mailbox),
+                                        path, create) != 0) {
+        g_object_unref(mailbox);
+        return NULL;
     }
 
     return G_OBJECT(mailbox);
@@ -318,7 +336,8 @@ libbalsa_mailbox_maildir_get_message_stream(LibBalsaMailbox * mailbox,
 
     g_return_val_if_fail(MAILBOX_OPEN(mailbox), NULL);
 
-    msg_info = message_info_from_msgno(mailbox, msgno);
+    msg_info =
+        message_info_from_msgno((LibBalsaMailboxMaildir *) mailbox, msgno);
     if (!msg_info)
 	return NULL;
 
@@ -328,7 +347,7 @@ libbalsa_mailbox_maildir_get_message_stream(LibBalsaMailbox * mailbox,
 }
 
 static void
-libbalsa_mailbox_maildir_remove_files(LibBalsaMailboxLocal *mailbox)
+lbm_maildir_remove_files(LibBalsaMailboxLocal *mailbox)
 {
     const gchar* path;
     g_return_if_fail(LIBBALSA_IS_MAILBOX_MAILDIR(mailbox));
@@ -346,6 +365,7 @@ libbalsa_mailbox_maildir_remove_files(LibBalsaMailboxLocal *mailbox)
 			     _("Could not remove %s:\n%s"),
 			     path, strerror(errno));
     }
+    LIBBALSA_MAILBOX_LOCAL_CLASS(parent_class)->remove_files(mailbox);
 }
 
 static LibBalsaMessageFlag parse_filename(const gchar *subdir,
@@ -387,7 +407,8 @@ static LibBalsaMessageFlag parse_filename(const gchar *subdir,
     return flags;
 }
 
-static void parse_mailbox(LibBalsaMailbox * mailbox, const gchar *subdir)
+static void lbm_maildir_parse(LibBalsaMailboxMaildir * mdir,
+                              const gchar *subdir, guint * fileno)
 {
     gchar *path;
     GDir *dir;
@@ -397,10 +418,10 @@ static void parse_mailbox(LibBalsaMailbox * mailbox, const gchar *subdir)
     gchar *key;
     gchar *p;
     LibBalsaMessageFlag flags;
-    LibBalsaMailboxMaildir *mdir = LIBBALSA_MAILBOX_MAILDIR(mailbox);
+    LibBalsaMailbox *mailbox = (LibBalsaMailbox *) mdir;
 
-    path = g_strdup_printf("%s/%s", libbalsa_mailbox_local_get_path(mailbox),
-			   subdir);
+    path = g_build_filename(libbalsa_mailbox_local_get_path(mailbox),
+			    subdir, NULL);
     dir = g_dir_open(path, 0, NULL);
     g_free(path);
     if (dir == NULL)
@@ -442,17 +463,29 @@ static void parse_mailbox(LibBalsaMailbox * mailbox, const gchar *subdir)
 	    msg_info->key=key;
 	    msg_info->filename=g_strdup(filename);
 	    msg_info->flags = msg_info->orig_flags = flags;
+	    msg_info->fileno = 0;
 	}
 	msg_info->subdir = subdir;
+        if (!msg_info->fileno)
+            /* First time we saw this key. */
+	    msg_info->fileno = ++*fileno;
     }
     g_dir_close(dir);
 }
 
 static void
-parse_mailbox_subdirs(LibBalsaMailbox * mailbox)
+lbm_maildir_parse_subdirs(LibBalsaMailboxMaildir * mdir)
 {
-    parse_mailbox(mailbox, "cur");
-    parse_mailbox(mailbox, "new");
+    guint msgno, fileno = 0;
+
+    for (msgno = mdir->msgno_2_msg_info->len; msgno > 0; --msgno) {
+        struct message_info *msg_info =
+            message_info_from_msgno(mdir, msgno);
+        msg_info->fileno = 0;
+    }
+
+    lbm_maildir_parse(mdir, "cur", &fileno);
+    lbm_maildir_parse(mdir, "new", &fileno);
 }
 
 static gboolean
@@ -486,7 +519,7 @@ libbalsa_mailbox_maildir_open(LibBalsaMailbox * mailbox, GError **err)
           access(mdir->tmpdir, W_OK) == 0);
 
     mailbox->unread_messages = 0;
-    parse_mailbox_subdirs(mailbox);
+    lbm_maildir_parse_subdirs(mdir);
     LIBBALSA_MAILBOX_LOCAL(mailbox)->sync_time = 0;
     LIBBALSA_MAILBOX_LOCAL(mailbox)->sync_cnt  = 1;
 #ifdef DEBUG
@@ -565,7 +598,7 @@ libbalsa_mailbox_maildir_check(LibBalsaMailbox * mailbox)
     for (msgno = 1; msgno <= mdir->msgno_2_msg_info->len; ) {
 	gchar *filename;
 
-        msg_info = message_info_from_msgno(mailbox, msgno);
+        msg_info = message_info_from_msgno(mdir, msgno);
 	filename = g_build_filename(path, msg_info->subdir,
 				    msg_info->filename, NULL);
 	if (access(filename, F_OK) == 0)
@@ -581,13 +614,13 @@ libbalsa_mailbox_maildir_check(LibBalsaMailbox * mailbox)
 	g_free(filename);
     }
     for (msgno = renumber; msgno <= mdir->msgno_2_msg_info->len; msgno++) {
-	msg_info = message_info_from_msgno(mailbox, msgno);
+	msg_info = message_info_from_msgno(mdir, msgno);
 	if (msg_info->message)
 	    msg_info->message->msgno = msgno;
     }
 
     msgno = mdir->msgno_2_msg_info->len;
-    parse_mailbox_subdirs(mailbox);
+    lbm_maildir_parse_subdirs(mdir);
     libbalsa_mailbox_local_load_messages(mailbox, msgno);
 }
 
@@ -612,25 +645,26 @@ libbalsa_mailbox_maildir_close_mailbox(LibBalsaMailbox * mailbox,
                                        gboolean expunge)
 {
     LibBalsaMailboxMaildir *mdir = LIBBALSA_MAILBOX_MAILDIR(mailbox);
-    guint len = 0;
+    guint len;
 
-    if (mdir->msgno_2_msg_info)
-	len = mdir->msgno_2_msg_info->len;
+    g_assert(mdir->msgno_2_msg_info != NULL);
+    g_assert(mdir->messages_info != NULL);
+
+    len = mdir->msgno_2_msg_info->len;
     libbalsa_mailbox_maildir_sync(mailbox, expunge);
+    if (mdir->msgno_2_msg_info->len != len)
+        libbalsa_mailbox_changed(mailbox);
 
-    if (mdir->messages_info) {
-	g_hash_table_destroy(mdir->messages_info);
-	mdir->messages_info = NULL;
-    }
-    if (mdir->msgno_2_msg_info) {
-	if (mdir->msgno_2_msg_info->len != len)
-	    libbalsa_mailbox_changed(mailbox);
-	g_ptr_array_free(mdir->msgno_2_msg_info, TRUE);
-	mdir->msgno_2_msg_info = NULL;
-    }
     if (LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox)
         LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox(mailbox,
                                                             expunge);
+
+    /* Now it's safe to free the message info. */
+    g_hash_table_destroy(mdir->messages_info);
+    mdir->messages_info = NULL;
+
+    g_ptr_array_free(mdir->msgno_2_msg_info, TRUE);
+    mdir->msgno_2_msg_info = NULL;
 }
 
 static gchar *
@@ -749,13 +783,12 @@ libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     guint changes = 0;
 
     for (msgno = 1; msgno <= mdir->msgno_2_msg_info->len; msgno++) {
-	msg_info = message_info_from_msgno(mailbox, msgno);
+	msg_info = message_info_from_msgno(mdir, msgno);
 
 	if (expunge && (msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
 	    /* skip this block if the message should only be renamed */
-	    gchar *orig =
-		g_strdup_printf("%s/%s/%s", path, msg_info->subdir, 
-                                msg_info->filename);
+	    gchar *orig = g_build_filename(path, msg_info->subdir,
+                                           msg_info->filename, NULL);
 	    unlink (orig);
 	    g_free(orig);
 	    removed_list =
@@ -787,7 +820,7 @@ libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	msgno = GPOINTER_TO_UINT(l->data);
 	if (renumber > msgno)
 	    renumber = msgno;
-	msg_info = message_info_from_msgno(mailbox, msgno);
+	msg_info = message_info_from_msgno(mdir, msgno);
 
 	g_ptr_array_remove(mdir->msgno_2_msg_info, msg_info);
 	libbalsa_mailbox_local_msgno_removed(mailbox, msgno);
@@ -796,7 +829,7 @@ libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     }
     g_slist_free(removed_list);
     for (msgno = renumber; msgno <= mdir->msgno_2_msg_info->len; msgno++) {
-	msg_info = message_info_from_msgno(mailbox, msgno);
+	msg_info = message_info_from_msgno(mdir, msgno);
 	if (msg_info->message)
 	    msg_info->message->msgno = msgno;
     }
@@ -804,6 +837,9 @@ libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     if (changes) {              /* Record mtime of dir. */
         struct stat st;
 
+        /* Reparse, to get the fileno entries right. */
+        lbm_maildir_parse_subdirs(mdir);
+
         if (stat(mdir->tmpdir, &st) == 0)
             libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
     }
@@ -811,15 +847,14 @@ libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     return TRUE;
 }
 
-static struct message_info *message_info_from_msgno(
-						  LibBalsaMailbox * mailbox,
-						  guint msgno)
+static struct message_info *
+message_info_from_msgno(LibBalsaMailboxMaildir * mdir, guint msgno)
 {
-    struct message_info *msg_info = NULL;
+    g_assert(msgno > 0 && msgno <= mdir->msgno_2_msg_info->len);
 
-    msg_info = g_ptr_array_index(LIBBALSA_MAILBOX_MAILDIR(mailbox)->msgno_2_msg_info,
-			      msgno - 1);
-    return msg_info;
+    return (struct message_info *) g_ptr_array_index(mdir->
+                                                     msgno_2_msg_info,
+                                                     msgno - 1);
 }
 
 
@@ -830,7 +865,8 @@ libbalsa_mailbox_maildir_fetch_message_structure(LibBalsaMailbox * mailbox,
 {
     if (!message->mime_msg) {
 	struct message_info *msg_info =
-	    message_info_from_msgno(mailbox, message->msgno);
+            message_info_from_msgno((LibBalsaMailboxMaildir *) mailbox,
+                                    message->msgno);
 	message->mime_msg =
 	    libbalsa_mailbox_local_get_mime_message(mailbox,
 						    msg_info->subdir,
@@ -842,40 +878,69 @@ libbalsa_mailbox_maildir_fetch_message_structure(LibBalsaMailbox * mailbox,
 }
 
 static LibBalsaMessage *
-libbalsa_mailbox_maildir_get_message(LibBalsaMailbox * mailbox,
-                                     guint msgno)
+lbm_maildir_get_message(LibBalsaMailbox * mailbox, guint msgno)
 {
     struct message_info *msg_info =
-	message_info_from_msgno(mailbox, msgno);
+        message_info_from_msgno((LibBalsaMailboxMaildir *) mailbox, msgno);
+    LibBalsaMessage *message;
+
+    message = msg_info->message;
+    if (message)
+        return g_object_ref(message);
+
+    msg_info->message = message = libbalsa_message_new();
+    g_object_add_weak_pointer(G_OBJECT(message),
+                              (gpointer) & msg_info->message);
+
+    message->flags = REAL_FLAGS(msg_info->flags);
+    message->mailbox = mailbox;
+    message->msgno = msgno;
+    libbalsa_message_load_envelope(message);
+
+    libbalsa_mailbox_local_cache_message(LIBBALSA_MAILBOX_LOCAL(mailbox),
+                                         msgno, message);
+
+    return message;
+}
+
+static LibBalsaMessageFlag
+lbm_maildir_load_message(LibBalsaMailbox * mailbox, guint msgno,
+                         LibBalsaMessage ** msg)
+{
+    struct message_info *msg_info =
+        message_info_from_msgno((LibBalsaMailboxMaildir *) mailbox, msgno);
 
     if (msg_info->message)
-	g_object_ref(msg_info->message);
-    else {
-	LibBalsaMessage *message;
+        *msg = g_object_ref(msg_info->message);
 
-	msg_info->message = message = libbalsa_message_new();
-	g_object_add_weak_pointer(G_OBJECT(message),
-				  (gpointer) & msg_info->message);
+    return REAL_FLAGS(msg_info->flags);
+}
 
-	message->flags = REAL_FLAGS(msg_info->flags);
-	message->mailbox = mailbox;
-	message->msgno = msgno;
-	libbalsa_message_load_envelope(message);
-    }
+static guint
+lbm_maildir_fileno(LibBalsaMailboxLocal * local, guint msgno)
+{
+    struct message_info *msg_info;
 
-    return msg_info->message;
+    if (!msgno)
+        return 0;
+
+    msg_info =
+        message_info_from_msgno((LibBalsaMailboxMaildir *) local, msgno);
+
+    return msg_info->fileno;
 }
 
 /* Called with mailbox locked. */
-static int
+static gboolean
 libbalsa_mailbox_maildir_add_message(LibBalsaMailbox * mailbox,
-                                     LibBalsaMessage * message, GError **err)
+                                     GMimeStream * stream,
+                                     LibBalsaMessageFlag flags,
+                                     GError **err)
 {
     const char *path;
     char *tmp;
     int fd;
     GMimeStream *out_stream;
-    GMimeStream *tmp_stream;
     GMimeStream *in_stream;
     GMimeFilter *crlffilter;
     char *new_filename;
@@ -888,44 +953,30 @@ libbalsa_mailbox_maildir_add_message(LibBalsaMailbox * mailbox,
     path = libbalsa_mailbox_local_get_path(mailbox);
     fd = libbalsa_mailbox_maildir_open_temp(path, &tmp);
     if (fd == -1)
-	return -1;
+	return FALSE;
     out_stream = g_mime_stream_fs_new(fd);
 
-    tmp_stream = libbalsa_message_stream(message);
-    if (!tmp_stream) {
-        g_object_unref(out_stream);
-        unlink(tmp);
-        g_free(tmp);
-        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
-                    LIBBALSA_MAILBOX_COPY_ERROR,
-                    _("Cannot read message"));
-        return -1;
-    }
-    in_stream = g_mime_stream_filter_new_with_stream(tmp_stream);
-    g_object_unref(tmp_stream);
-
+    in_stream = g_mime_stream_filter_new_with_stream(stream);
     crlffilter =
         g_mime_filter_crlf_new(GMIME_FILTER_CRLF_DECODE,
                                GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
     g_mime_stream_filter_add(GMIME_STREAM_FILTER(in_stream), crlffilter);
     g_object_unref(crlffilter);
  
-    libbalsa_mime_stream_shared_lock(in_stream);
-    if (g_mime_stream_write_to_stream( in_stream, out_stream) == -1)
-    {
-        libbalsa_mime_stream_shared_unlock(in_stream);
-	g_object_unref(in_stream);
-	g_object_unref(out_stream);
+    libbalsa_mime_stream_shared_lock(stream);
+    retval = g_mime_stream_write_to_stream(in_stream, out_stream);
+    libbalsa_mime_stream_shared_unlock(stream);
+    g_object_unref(in_stream);
+    g_object_unref(out_stream);
+
+    if (retval < 0) {
 	unlink (tmp);
 	g_free(tmp);
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_COPY_ERROR,
                     _("Data copy error"));
-	return -1;
+	return FALSE;
     }
-    libbalsa_mime_stream_shared_unlock(in_stream);
-    g_object_unref(in_stream);
-    g_object_unref(out_stream);
 
     new_filename = strrchr(tmp, '/');
     if (new_filename)
@@ -936,8 +987,8 @@ libbalsa_mailbox_maildir_add_message(LibBalsaMailbox * mailbox,
     msg_info->subdir = "tmp";
     msg_info->key = g_strdup(new_filename);
     msg_info->filename = g_strdup(new_filename);
-    msg_info->flags = message->flags | LIBBALSA_MESSAGE_FLAG_RECENT;
-    retval = maildir_sync_add(msg_info, path) ? 1 : -1;
+    msg_info->flags = flags | LIBBALSA_MESSAGE_FLAG_RECENT;
+    retval = maildir_sync_add(msg_info, path);
     free_message_info(msg_info);
     g_free(tmp);
 
@@ -963,7 +1014,8 @@ libbalsa_mailbox_maildir_messages_change_flags(LibBalsaMailbox * mailbox,
     for (i = 0; i < msgnos->len; i++) {
         guint msgno = g_array_index(msgnos, guint, i);
         struct message_info *msg_info =
-            message_info_from_msgno(mailbox, msgno);
+            message_info_from_msgno((LibBalsaMailboxMaildir *) mailbox,
+                                    msgno);
         LibBalsaMessageFlag old_flags = msg_info->flags;
         gboolean was_unread_undeleted, is_unread_undeleted;
 
@@ -978,7 +1030,6 @@ libbalsa_mailbox_maildir_messages_change_flags(LibBalsaMailbox * mailbox,
             msg_info->message->flags = msg_info->flags;
 
         libbalsa_mailbox_index_set_flags(mailbox, msgno, msg_info->flags);
-        libbalsa_mailbox_msgno_changed(mailbox, msgno);
 
         was_unread_undeleted = (old_flags & LIBBALSA_MESSAGE_FLAG_NEW)
             && !(old_flags & LIBBALSA_MESSAGE_FLAG_DELETED);
@@ -1005,7 +1056,7 @@ libbalsa_mailbox_maildir_msgno_has_flags(LibBalsaMailbox * mailbox,
                                          LibBalsaMessageFlag unset)
 {
     struct message_info *msg_info =
-        message_info_from_msgno(mailbox, msgno);
+        message_info_from_msgno((LibBalsaMailboxMaildir *) mailbox, msgno);
 
     return (msg_info->flags & set) == set
         && (msg_info->flags & unset) == 0;
