@@ -68,6 +68,7 @@ static void libbalsa_mailbox_mbox_init(LibBalsaMailboxMbox * mailbox);
 static GMimeStream *libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox *
 							     mailbox,
 							     guint msgno);
+static gint lbm_mbox_check_files(const gchar * path, gboolean create);
 static void libbalsa_mailbox_mbox_remove_files(LibBalsaMailboxLocal *mailbox);
 
 static gboolean libbalsa_mailbox_mbox_open(LibBalsaMailbox * mailbox,
@@ -77,16 +78,20 @@ static void libbalsa_mailbox_mbox_close_mailbox(LibBalsaMailbox * mailbox,
 static void libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox);
 static gboolean libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox,
                                            gboolean expunge);
-static LibBalsaMessage *libbalsa_mailbox_mbox_get_message(
-						  LibBalsaMailbox * mailbox,
-						  guint msgno);
+static LibBalsaMessage *lbm_mbox_get_message(LibBalsaMailbox * mailbox,
+                                             guint msgno);
+static LibBalsaMessageFlag lbm_mbox_load_message(LibBalsaMailbox * mailbox,
+                                                 guint msgno,
+                                                 LibBalsaMessage **msg);
 static gboolean
 libbalsa_mailbox_mbox_fetch_message_structure(LibBalsaMailbox * mailbox,
                                               LibBalsaMessage * message,
                                               LibBalsaFetchFlag flags);
-static int libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
-                                             LibBalsaMessage *message,
-                                             GError **err);
+static gboolean libbalsa_mailbox_mbox_add_message(LibBalsaMailbox *
+                                                  mailbox,
+                                                  GMimeStream * stream,
+                                                  LibBalsaMessageFlag
+                                                  flags, GError ** err);
 static gboolean
 libbalsa_mailbox_mbox_messages_change_flags(LibBalsaMailbox * mailbox,
                                             GArray * msgnos,
@@ -99,6 +104,10 @@ libbalsa_mailbox_mbox_msgno_has_flags(LibBalsaMailbox * mailbox,
                                       LibBalsaMessageFlag unset);
 static guint
 libbalsa_mailbox_mbox_total_messages(LibBalsaMailbox * mailbox);
+#if BALSA_USE_THREADS
+static void libbalsa_mailbox_mbox_lock_store(LibBalsaMailbox * mailbox,
+                                             gboolean lock);
+#endif                          /* BALSA_USE_THREADS */
 
 struct _LibBalsaMailboxMboxClass {
     LibBalsaMailboxLocalClass klass;
@@ -160,7 +169,7 @@ libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
     libbalsa_mailbox_class->sync = libbalsa_mailbox_mbox_sync;
     libbalsa_mailbox_class->close_mailbox =
 	libbalsa_mailbox_mbox_close_mailbox;
-    libbalsa_mailbox_class->get_message = libbalsa_mailbox_mbox_get_message;
+    libbalsa_mailbox_class->get_message = lbm_mbox_get_message;
     libbalsa_mailbox_class->fetch_message_structure =
 	libbalsa_mailbox_mbox_fetch_message_structure;
     libbalsa_mailbox_class->add_message = libbalsa_mailbox_mbox_add_message;
@@ -170,12 +179,15 @@ libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
 	libbalsa_mailbox_mbox_msgno_has_flags;
     libbalsa_mailbox_class->total_messages =
 	libbalsa_mailbox_mbox_total_messages;
+#if BALSA_USE_THREADS
+    libbalsa_mailbox_class->lock_store = libbalsa_mailbox_mbox_lock_store;
+#endif                          /* BALSA_USE_THREADS */
 
+    libbalsa_mailbox_local_class->check_files  = lbm_mbox_check_files;
     libbalsa_mailbox_local_class->remove_files = 
 	libbalsa_mailbox_mbox_remove_files;
 
-    libbalsa_mailbox_local_class->load_message =
-        libbalsa_mailbox_mbox_get_message;
+    libbalsa_mailbox_local_class->load_message = lbm_mbox_load_message;
 }
 
 
@@ -185,39 +197,34 @@ libbalsa_mailbox_mbox_init(LibBalsaMailboxMbox * mbox)
 {
 }
 
-gint
-libbalsa_mailbox_mbox_create(const gchar * path, gboolean create)
+static gint
+lbm_mbox_check_files(const gchar * path, gboolean create)
 {
-    gint exists; 
-    GType magic_type;
-    gint fd;
-    
-    g_return_val_if_fail( path != NULL, -1);
+    g_return_val_if_fail(path != NULL, -1);
 
-    exists = access(path, F_OK);
-    if ( exists == 0 ) {
-	/* File exists. Check if it is an mbox... */
-	
-	magic_type = libbalsa_mailbox_type_from_path(path);
-	if ( magic_type != LIBBALSA_TYPE_MAILBOX_MBOX ) {
-	    libbalsa_information(LIBBALSA_INFORMATION_WARNING, 
-				 _("Mailbox %s does not appear to be an Mbox mailbox."), path);
-	    return(-1);
-	}
-    } else {
-	if(!create)
-	    return(-1);
+    if (access(path, F_OK) == 0) {
+        /* File exists. Check if it is an mbox... */
+        if (libbalsa_mailbox_type_from_path(path) !=
+            LIBBALSA_TYPE_MAILBOX_MBOX) {
+            libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                                 _("Mailbox %s does not appear to be an Mbox mailbox."),
+                                 path);
+            return -1;
+        }
+    } else if (create) {
+        gint fd;
 
-	if ((fd = creat(path, S_IRUSR | S_IWUSR)) == -1) {
-	    g_warning("An error:\n%s\n occured while trying to "
-		      "create the mailbox \"%s\"\n",
-		      strerror(errno), path);
-	    return -1;
-	} else {
-	    close(fd);
-	}
-    }
-    return(0);
+        if ((fd = creat(path, S_IRUSR | S_IWUSR)) == -1) {
+            g_warning("An error:\n%s\n occurred while trying to "
+                      "create the mailbox \"%s\"\n",
+                      strerror(errno), path);
+            return -1;
+        } else
+            close(fd);
+    } else
+        return -1;
+
+    return 0;
 }
 
 GObject *
@@ -226,11 +233,12 @@ libbalsa_mailbox_mbox_new(const gchar * path, gboolean create)
     LibBalsaMailbox *mailbox;
 
     mailbox = g_object_new(LIBBALSA_TYPE_MAILBOX_MBOX, NULL);
+
     mailbox->is_directory = FALSE;
 	
-    mailbox->url = g_strconcat("file://", path, NULL);
-    if( libbalsa_mailbox_mbox_create(path, create) < 0 ) {
-	g_object_unref(G_OBJECT(mailbox));
+    if (libbalsa_mailbox_local_set_path(LIBBALSA_MAILBOX_LOCAL(mailbox),
+                                        path, create) != 0) {
+	g_object_unref(mailbox);
 	return NULL;
     }
     
@@ -300,6 +308,7 @@ libbalsa_mailbox_mbox_remove_files(LibBalsaMailboxLocal *mailbox)
 			     _("Could not remove %s:\n%s"), 
 			     libbalsa_mailbox_local_get_path(mailbox), 
 			     strerror(errno));
+    LIBBALSA_MAILBOX_LOCAL_CLASS(parent_class)->remove_files(mailbox);
 }
 
 static int mbox_lock(LibBalsaMailbox * mailbox, GMimeStream *stream)
@@ -351,6 +360,7 @@ extern int message_cnt;
 static void
 parse_mailbox(LibBalsaMailboxMbox * mbox)
 {
+    LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mbox);
     GMimeParser *gmime_parser;
     struct message_info msg_info;
     struct message_info * msg_info_p = &msg_info;
@@ -364,6 +374,7 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
 				   (GMimeParserHeaderRegexFunc)
 				   lbm_mbox_header_cb, &msg_info_p);
 
+    libbalsa_mailbox_local_set_threading_info(local);
     msg_info.message = NULL;
     while (!g_mime_parser_eos(gmime_parser)) {
 	GMimeMessage *mime_message;
@@ -406,10 +417,13 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
         msg->msgno = ++msgno;
         g_ptr_array_add(LIBBALSA_MAILBOX(mbox)->mindex, 
                         libbalsa_mailbox_index_entry_new_from_msg(msg));
+
+        libbalsa_mailbox_local_cache_message(local, msgno, msg);
+
         g_object_unref(msg);
     }
 
-    g_object_unref(G_OBJECT(gmime_parser));
+    g_object_unref(gmime_parser);
     printf("done, msgcnt=%d\n", message_cnt);
 }
 
@@ -614,6 +628,7 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
     GMimeStream *mbox_stream;
     guint msgno;
     time_t mtime;
+    off_t start;
 
     g_assert(LIBBALSA_IS_MAILBOX_MBOX(mailbox));
 
@@ -665,18 +680,27 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
      * parse_mailbox assumes that the stream is positioned at the first
      * message to be parsed.
      */
+
     libbalsa_mime_stream_shared_lock(mbox_stream);
+
+    /* If Balsa appended a message, it was prefixed with "\nFrom ", so
+     * we first check one byte beyond the end of the last message: */
+    start = mbox->size + 1;
+
     while ((msgno = mbox->messages_info->len) > 0) {
 	off_t offset;
-        struct message_info *msg_info =
-            &g_array_index(mbox->messages_info, struct message_info,
-                           msgno - 1);
-        if (lbm_mbox_stream_seek_to_message(mbox_stream, msg_info->end))
-	    /* A message begins at the end of this one, so it must(?) be
-	     * in its original position--start parsing here. */
+        struct message_info *msg_info;
+
+        if (lbm_mbox_stream_seek_to_message(mbox_stream, start))
+	    /* A message begins here, so it must(?) be
+	     * the first new message--start parsing here. */
             break;
 
 	/* Back up over this message and try again. */
+        msg_info = &g_array_index(mbox->messages_info, struct message_info,
+                                  msgno - 1);
+        start = msg_info->start;
+
         if ((msg_info->flags & LIBBALSA_MESSAGE_FLAG_NEW)
             && !(msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED))
             --mailbox->unread_messages;
@@ -1191,9 +1215,9 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     } else if (!lbm_mbox_stream_seek_to_message(mbox_stream, offset))
         g_warning("mbox_sync: message not in expected position.\n");
     else if (g_mime_stream_write_to_stream(temp_stream, mbox_stream) != -1) {
-        save_failed = FALSE;
         mbox->size = g_mime_stream_tell(mbox_stream);
-        ftruncate(GMIME_STREAM_FS(mbox_stream)->fd, mbox->size);
+        if (ftruncate(GMIME_STREAM_FS(mbox_stream)->fd, mbox->size) == 0)
+            save_failed = FALSE;
     }
     g_object_unref(temp_stream);
     mbox_unlock(mailbox, mbox_stream);
@@ -1202,7 +1226,7 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     libbalsa_mime_stream_shared_unlock(mbox_stream);
     if (save_failed) {
 	/*
-	 * error occured while writing the mailbox back,
+	 * error occurred while writing the mailbox back,
 	 * so keep the temp copy around
 	 */
 	char *savefile;
@@ -1237,11 +1261,9 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 				      struct message_info, j);
 	    if (expunge &&
 		(msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
+	        libbalsa_mailbox_local_msgno_removed(mailbox, j + 1);
 		free_message_info(msg_info);
 		g_array_remove_index(mbox->messages_info, j);
-                libbalsa_mailbox_index_entry_free(g_ptr_array_index
-                                                  (mailbox->mindex, j));
-		g_ptr_array_remove_index(mailbox->mindex, j);
 	    } else
 		j++;
 	}
@@ -1287,9 +1309,9 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	    msg_info->message->msgno = j + 1;
 	j++;
 
-	msg_info->start = g_mime_parser_tell(gmime_parser);
 	msg_info->status = msg_info->x_status = msg_info->mime_version = -1;
 	mime_msg = g_mime_parser_construct_message(gmime_parser);
+        msg_info->start = g_mime_parser_get_from_offset(gmime_parser);
 
 	/* Make sure we don't have offsets for any encapsulated headers. */
 	if (!g_mime_object_get_header(GMIME_OBJECT(mime_msg), "Status"))
@@ -1319,30 +1341,51 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     }
     libbalsa_mime_stream_shared_unlock(mbox_stream);
     mbox->messages_info->len = j;
-    g_object_unref(G_OBJECT(gmime_parser));
+    g_object_unref(gmime_parser);
 
     return TRUE;
 }
 
-static LibBalsaMessage*
-libbalsa_mailbox_mbox_get_message(LibBalsaMailbox * mailbox, guint msgno)
+static LibBalsaMessage *
+lbm_mbox_get_message(LibBalsaMailbox * mailbox, guint msgno)
 {
     LibBalsaMailboxMbox *mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
     struct message_info *msg_info;
+    LibBalsaMessage *message;
 
-    msg_info = &g_array_index(mbox->messages_info,
-			      struct message_info, msgno-1);
-    if(!msg_info->message) {
-	msg_info->message = libbalsa_message_new();
-	msg_info->message->mailbox = mailbox;
-	msg_info->message->msgno   = msgno;
-	msg_info->message->flags   =
-	    msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
-        libbalsa_message_load_envelope(msg_info->message);
-	g_object_weak_ref(G_OBJECT(msg_info->message), mbox_msg_unref,
-			  mbox);
-    } else g_object_ref(msg_info->message);
-    return msg_info->message;
+    msg_info = &g_array_index(mbox->messages_info, struct message_info,
+                              msgno - 1);
+
+    message = msg_info->message;
+    if (message)
+        return g_object_ref(message);
+
+    msg_info->message = message = libbalsa_message_new();
+    g_object_weak_ref(G_OBJECT(message), mbox_msg_unref, mbox);
+
+    message->flags = msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
+    message->mailbox = mailbox;
+    message->msgno = msgno;
+    libbalsa_message_load_envelope(message);
+
+    libbalsa_mailbox_local_cache_message(LIBBALSA_MAILBOX_LOCAL(mailbox),
+                                         msgno, message);
+
+    return message;
+}
+
+static LibBalsaMessageFlag
+lbm_mbox_load_message(LibBalsaMailbox * mailbox, guint msgno,
+                      LibBalsaMessage ** msg)
+{
+    LibBalsaMailboxMbox *mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
+    struct message_info *msg_info =
+        &g_array_index(mbox->messages_info, struct message_info, msgno - 1);
+
+    if (msg_info->message)
+        *msg = g_object_ref(msg_info->message);
+
+    return msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
 }
 
 static gboolean
@@ -1452,7 +1495,7 @@ lbm_mbox_prepare_object(GMimeObject * mime_part)
                                  message));
     else if (GMIME_IS_MESSAGE(mime_part))
         lbm_mbox_prepare_object(((GMimeMessage *) mime_part)->mime_part);
-    else {
+    else if (!GMIME_IS_MESSAGE_PARTIAL(mime_part)) {
         GMimePartEncodingType encoding;
         const GMimeContentType *mime_type;
 
@@ -1509,10 +1552,13 @@ lbm_mbox_armored_stream(GMimeStream * stream)
 }
 
 /* Called with mailbox locked. */
-static int
+static gboolean
 libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
-                                  LibBalsaMessage *message, GError **err)
+                                  GMimeStream * stream,
+                                  LibBalsaMessageFlag flags,
+                                  GError ** err)
 {
+    LibBalsaMessage *message;
     gchar date_string[27];
     gchar *sender;
     gchar *address;
@@ -1520,18 +1566,23 @@ libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
     gchar *from = NULL;
     const char *path;
     int fd;
-    GMimeStream *orig;
     GMimeObject *armored_object;
     GMimeStream *armored_dest;
     GMimeStream *dest;
-    gint retval = 1;
+    gint retval;
     off_t orig_length;
 
+    message = libbalsa_message_new();
+    libbalsa_message_load_envelope_from_stream(message, stream);
+
     ctime_r(&(message->headers->date), date_string);
 
     sender = message->headers->from ?
 	internet_address_list_to_string(message->headers->from, FALSE) :
 	g_strdup("none");
+
+    g_object_unref(message);
+
     if ( (brack = strrchr( sender, '<' )) ) {
         gchar * a = strrchr ( brack , '>' );
         if (a)
@@ -1552,55 +1603,46 @@ libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_APPEND_ERROR,
                     _("%s: could not open %s."), "MBOX", path);
-        return -1;
+        return FALSE;
     }
     
     orig_length = lseek (fd, 0, SEEK_END);
     lseek (fd, 0, SEEK_SET);
     dest = g_mime_stream_fs_new (fd);
     if (!dest) {
+	g_free(from);
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_APPEND_ERROR,
                     _("%s: could not get new mime stream."),
                     "MBOX");
-	g_free(from);
-	return -1;
+	return FALSE;
     }
     if (orig_length > 0 && !lbm_mbox_stream_seek_to_message(dest, 0)) {
+	g_object_unref(dest);
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_APPEND_ERROR,
                     _("%s: %s is not in mbox format."),
                     "MBOX", path);
-	g_object_unref(dest);
 	g_free(from);
-	return -1;
+	return FALSE;
     }
     mbox_lock ( mailbox, dest );
 
-    orig = libbalsa_message_stream(message);
-    if (!orig) {
-        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
-                    LIBBALSA_MAILBOX_APPEND_ERROR,
-                    _("%s: could not get message stream."),
-                    "MBOX");
-	mbox_unlock(mailbox, dest);
-	g_object_unref(dest);
-	g_free(from);
-	return -1;
-    }
-
     /* From_ armor */
-    libbalsa_mime_stream_shared_lock(orig);
-    armored_object = lbm_mbox_armored_object(orig);
+    libbalsa_mime_stream_shared_lock(stream);
+    g_mime_stream_reset(stream);
+    armored_object = lbm_mbox_armored_object(stream);
     /* Make sure we have "Status" and "X-Status" headers, so we can
      * update them in place later, if necessary. */
     update_message_status_headers(GMIME_MESSAGE(armored_object),
-                                  message->flags |
-                                  LIBBALSA_MESSAGE_FLAG_RECENT);
+                                  flags | LIBBALSA_MESSAGE_FLAG_RECENT);
     armored_dest = lbm_mbox_armored_stream(dest);
 
-    g_mime_stream_seek(dest, 0, GMIME_STREAM_SEEK_END);
-    if (g_mime_stream_write_string(dest, from) < (gint) strlen(from)
+    retval = g_mime_stream_seek(dest, 0, GMIME_STREAM_SEEK_END);
+    if (retval > 0)
+        retval = lbm_mbox_newline(dest);
+    if (retval < 0
+        || g_mime_stream_write_string(dest, from) < (gint) strlen(from)
 	|| g_mime_object_write_to_stream(armored_object, armored_dest) < 0) {
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_APPEND_ERROR, _("Data copy error"));
@@ -1608,19 +1650,15 @@ libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
     }
     g_free(from);
     g_object_unref(armored_object);
-    libbalsa_mime_stream_shared_unlock(orig);
-    g_object_unref(orig);
+    libbalsa_mime_stream_shared_unlock(stream);
     g_object_unref(armored_dest);
 
-    if (retval > 0)
-	retval = lbm_mbox_newline(dest);
-
-    if(retval<0)
-        truncate(path, orig_length);
+    if (retval < 0 && truncate(path, orig_length) < 0)
+        retval = -2;
     mbox_unlock (mailbox, dest);
     g_object_unref(dest);
 
-    return retval;
+    return retval >= 0;
 }
 
 static gboolean
@@ -1652,7 +1690,6 @@ libbalsa_mailbox_mbox_messages_change_flags(LibBalsaMailbox * mailbox,
 		msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
 
         libbalsa_mailbox_index_set_flags(mailbox, msgno, msg_info->flags);
-        libbalsa_mailbox_msgno_changed(mailbox, msgno);
 
         was_unread_undeleted = (old_flags & LIBBALSA_MESSAGE_FLAG_NEW)
             && !(old_flags & LIBBALSA_MESSAGE_FLAG_DELETED);
@@ -1692,3 +1729,17 @@ libbalsa_mailbox_mbox_total_messages(LibBalsaMailbox * mailbox)
 
     return mbox->messages_info ? mbox->messages_info->len : 0;
 }
+
+#if BALSA_USE_THREADS
+static void
+libbalsa_mailbox_mbox_lock_store(LibBalsaMailbox * mailbox, gboolean lock)
+{
+    LibBalsaMailboxMbox *mbox = (LibBalsaMailboxMbox *) mailbox;
+    GMimeStream *stream = mbox->gmime_stream;
+
+    if (lock)
+        libbalsa_mime_stream_shared_lock(stream);
+    else
+        libbalsa_mime_stream_shared_unlock(stream);
+}
+#endif                          /* BALSA_USE_THREADS */
