@@ -38,6 +38,7 @@
 #include "libbalsa.h"
 #include "misc.h"
 #include "libbalsa_private.h"
+#include "libbalsa-conf.h"
 #include "mime-stream-shared.h"
 #include "i18n.h"
 
@@ -53,10 +54,15 @@ static LibBalsaMailboxLocalClass *parent_class = NULL;
 static void libbalsa_mailbox_mh_class_init(LibBalsaMailboxMhClass *klass);
 static void libbalsa_mailbox_mh_init(LibBalsaMailboxMh * mailbox);
 static void libbalsa_mailbox_mh_finalize(GObject * object);
+static void libbalsa_mailbox_mh_load_config(LibBalsaMailbox * mailbox,
+                                            const gchar * prefix);
 
 static GMimeStream *libbalsa_mailbox_mh_get_message_stream(LibBalsaMailbox *
 							   mailbox,
 							   guint msgno);
+static gint lbm_mh_check_files(const gchar * path, gboolean create);
+static void lbm_mh_set_path(LibBalsaMailboxLocal * mailbox,
+                            const gchar * path);
 static void libbalsa_mailbox_mh_remove_files(LibBalsaMailboxLocal *mailbox);
 
 static gboolean libbalsa_mailbox_mh_open(LibBalsaMailbox * mailbox,
@@ -69,17 +75,21 @@ static gboolean libbalsa_mailbox_mh_sync(LibBalsaMailbox * mailbox,
 static struct message_info *lbm_mh_message_info_from_msgno(
 						  LibBalsaMailboxMh * mailbox,
 						  guint msgno);
-static LibBalsaMessage *libbalsa_mailbox_mh_get_message(LibBalsaMailbox * mailbox,
-						     guint msgno);
+static LibBalsaMessage *lbm_mh_get_message(LibBalsaMailbox * mailbox,
+                                           guint msgno);
+static LibBalsaMessageFlag lbm_mh_load_message(LibBalsaMailbox * mailbox,
+                                               guint msgno,
+                                               LibBalsaMessage **msg);
 static gboolean libbalsa_mailbox_mh_fetch_message_structure(LibBalsaMailbox
                                                             * mailbox,
                                                             LibBalsaMessage
                                                             * message,
                                                             LibBalsaFetchFlag
                                                             flags);
-static int libbalsa_mailbox_mh_add_message(LibBalsaMailbox * mailbox,
-					   LibBalsaMessage * message,
-                                           GError **err);
+static gboolean libbalsa_mailbox_mh_add_message(LibBalsaMailbox * mailbox,
+                                                GMimeStream * stream,
+                                                LibBalsaMessageFlag flags,
+                                                GError ** err);
 static gboolean
 libbalsa_mailbox_mh_messages_change_flags(LibBalsaMailbox * mailbox,
                                           GArray * msgnos,
@@ -135,6 +145,7 @@ libbalsa_mailbox_mh_class_init(LibBalsaMailboxMhClass * klass)
 
     object_class->finalize = libbalsa_mailbox_mh_finalize;
 
+    libbalsa_mailbox_class->load_config = libbalsa_mailbox_mh_load_config;
     libbalsa_mailbox_class->get_message_stream =
 	libbalsa_mailbox_mh_get_message_stream;
 
@@ -144,7 +155,7 @@ libbalsa_mailbox_mh_class_init(LibBalsaMailboxMhClass * klass)
     libbalsa_mailbox_class->sync = libbalsa_mailbox_mh_sync;
     libbalsa_mailbox_class->close_mailbox =
 	libbalsa_mailbox_mh_close_mailbox;
-    libbalsa_mailbox_class->get_message = libbalsa_mailbox_mh_get_message;
+    libbalsa_mailbox_class->get_message = lbm_mh_get_message;
     libbalsa_mailbox_class->fetch_message_structure =
 	libbalsa_mailbox_mh_fetch_message_structure;
     libbalsa_mailbox_class->add_message = libbalsa_mailbox_mh_add_message;
@@ -155,8 +166,9 @@ libbalsa_mailbox_mh_class_init(LibBalsaMailboxMhClass * klass)
     libbalsa_mailbox_class->total_messages =
 	libbalsa_mailbox_mh_total_messages;
 
-    libbalsa_mailbox_local_class->load_message =
-        libbalsa_mailbox_mh_get_message;
+    libbalsa_mailbox_local_class->load_message = lbm_mh_load_message;
+    libbalsa_mailbox_local_class->check_files  = lbm_mh_check_files;
+    libbalsa_mailbox_local_class->set_path     = lbm_mh_set_path;
     libbalsa_mailbox_local_class->remove_files = 
 	libbalsa_mailbox_mh_remove_files;
 }
@@ -164,72 +176,80 @@ libbalsa_mailbox_mh_class_init(LibBalsaMailboxMhClass * klass)
 static void
 libbalsa_mailbox_mh_init(LibBalsaMailboxMh * mailbox)
 {
+    /* mh->sequences_file = NULL; */
 }
 
-gint
-libbalsa_mailbox_mh_create(const gchar * path, gboolean create) 
+static gint
+lbm_mh_check_files(const gchar * path, gboolean create)
 {
-    GType magic_type;
-    gint exists;
+    g_return_val_if_fail(path != NULL, -1);
 
-    g_return_val_if_fail( path != NULL, -1);
-	
-    exists = access(path, F_OK);
-    if ( exists == 0 ) {
-	/* File exists. Check if it is a mh... */
-	
-	magic_type = libbalsa_mailbox_type_from_path(path);
-	if ( magic_type != LIBBALSA_TYPE_MAILBOX_MH ) {
-	    libbalsa_information(LIBBALSA_INFORMATION_WARNING, 
-				 _("Mailbox %s does not appear to be a Mh mailbox."), path);
-	    return(-1);
-	}
-    } else {
-	if(create) {
-	    char tmp[_POSIX_PATH_MAX];
-	    int i;
+    if (access(path, F_OK) == 0) {
+        /* File exists. Check if it is a mh... */
+        if (libbalsa_mailbox_type_from_path(path) !=
+            LIBBALSA_TYPE_MAILBOX_MH) {
+            libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                                 _("Mailbox %s does not appear to be a Mh mailbox."),
+                                 path);
+            return -1;
+        }
+    } else if (create) {
+        gint fd;
+        gchar *sequences_filename;
 
-	    if (mkdir (path, S_IRWXU)) {
-		libbalsa_information(LIBBALSA_INFORMATION_WARNING, 
-				     _("Could not create MH directory at %s (%s)"), path, strerror(errno) );
-		return (-1);
-	    } 
-	    snprintf (tmp, sizeof (tmp), "%s/.mh_sequences", path);
-	    if ((i = creat (tmp, S_IRWXU)) == -1) {
-		libbalsa_information
-		    (LIBBALSA_INFORMATION_WARNING, 
-		     _("Could not create MH structure at %s (%s)"),
-		     path, strerror(errno));
-		rmdir (path);
-		return (-1);
-	    } else close(i);   	    
-	} else 
-	    return(-1);
-    }
-    return(0);
+        if (mkdir(path, S_IRWXU)) {
+            libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                                 _("Could not create MH directory at %s (%s)"),
+                                 path, strerror(errno));
+            return -1;
+        }
+
+        sequences_filename = g_build_filename(path, ".mh_sequences", NULL);
+        fd = creat(sequences_filename, S_IRWXU);
+        g_free(sequences_filename);
+
+        if (fd == -1) {
+            libbalsa_information
+                (LIBBALSA_INFORMATION_WARNING,
+                 _("Could not create MH structure at %s (%s)"),
+                 path, strerror(errno));
+            rmdir(path);
+            return -1;
+        } else
+            close(fd);
+    } else
+        return -1;
+
+    return 0;
+}
+
+static void
+lbm_mh_set_sequences_filename(LibBalsaMailboxMh * mh, const gchar * path)
+{
+    g_free(mh->sequences_filename);
+    mh->sequences_filename = g_build_filename(path, ".mh_sequences", NULL);
+}
+
+static void
+lbm_mh_set_path(LibBalsaMailboxLocal * local, const gchar * path)
+{
+    lbm_mh_set_sequences_filename(LIBBALSA_MAILBOX_MH(local), path);
 }
-    
 
 GObject *
 libbalsa_mailbox_mh_new(const gchar * path, gboolean create)
 {
     LibBalsaMailbox *mailbox;
-    LibBalsaMailboxMh *mh;
 
-    
     mailbox = g_object_new(LIBBALSA_TYPE_MAILBOX_MH, NULL);
-    
+
     mailbox->is_directory = TRUE;
-    
-    mailbox->url = g_strconcat("file://", path, NULL);
-    
-    if(libbalsa_mailbox_mh_create(path, create) < 0) {
-	g_object_unref(G_OBJECT(mailbox));
+
+    if (libbalsa_mailbox_local_set_path(LIBBALSA_MAILBOX_LOCAL(mailbox),
+                                        path, create) != 0) {
+	g_object_unref(mailbox);
 	return NULL;
     }
-    
-    mh = LIBBALSA_MAILBOX_MH(mailbox);
-    mh->sequences_filename = g_strdup_printf("%s/.mh_sequences", path);
 
     return G_OBJECT(mailbox);
 }
@@ -242,6 +262,20 @@ libbalsa_mailbox_mh_finalize(GObject * object)
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
+static void
+libbalsa_mailbox_mh_load_config(LibBalsaMailbox * mailbox,
+                                const gchar * prefix)
+{
+    LibBalsaMailboxMh *mh = LIBBALSA_MAILBOX_MH(mailbox);
+    gchar *path;
+
+    path = libbalsa_conf_get_string("Path");
+    lbm_mh_set_sequences_filename(mh, path);
+    g_free(path);
+
+    LIBBALSA_MAILBOX_CLASS(parent_class)->load_config(mailbox, prefix);
+}
+
 #define MH_BASENAME(msgno) \
     g_strdup_printf((msgno->orig_flags & LIBBALSA_MESSAGE_FLAG_DELETED) ? \
 		    ",%d" : "%d", msg_info->fileno)
@@ -283,6 +317,7 @@ libbalsa_mailbox_mh_remove_files(LibBalsaMailboxLocal *mailbox)
 			     _("Could not remove %s:\n%s"),
 			     path, strerror(errno));
     }
+    LIBBALSA_MAILBOX_LOCAL_CLASS(parent_class)->remove_files(mailbox);
 }
 
 /* Ignore the garbage files.  A valid MH message consists of only
@@ -309,7 +344,7 @@ lbm_mh_compare_fileno(const struct message_info ** a,
 #define INVALID_FLAG ((unsigned) -1)
 
 static void
-lbm_mh_parse_mailbox(LibBalsaMailboxMh * mh)
+lbm_mh_parse_mailbox(LibBalsaMailboxMh * mh, gboolean add_msg_info)
 {
     const gchar *path;
     GDir *dir;
@@ -331,11 +366,12 @@ lbm_mh_parse_mailbox(LibBalsaMailboxMh * mh)
 	if (lbm_mh_check_filename(filename) == FALSE)
 	    continue;
 
-	sscanf(filename, "%d", &fileno);
+	if (sscanf(filename, "%d", &fileno) != 1)
+            break;     /* FIXME report error? */
 	if (fileno > mh->last_fileno)
 	    mh->last_fileno = fileno;
 
-	if (mh->messages_info) {
+	if (add_msg_info && mh->messages_info) {
 	    struct message_info *msg_info =
 		g_hash_table_lookup(mh->messages_info,
 				    GINT_TO_POINTER(fileno));
@@ -412,9 +448,11 @@ lbm_mh_handle_seq_line(LibBalsaMailboxMh * mh, gchar * line)
 	line = strchr(*seq, '-');
 	if (line) {
 	    *line++ = '\0';
-	    sscanf(line, "%d", &end);
+	    if (sscanf(line, "%d", &end) != 1)
+                break; /* FIXME report error? */
 	}
-	sscanf(*seq, "%d", &fileno);
+	if (sscanf(*seq, "%d", &fileno) != 1)
+            break;     /* FIXME report error? */
 	do
 	    lbm_mh_set_flag(mh, fileno, flag);
 	while (++fileno <= end);
@@ -427,7 +465,7 @@ static void
 lbm_mh_parse_sequences(LibBalsaMailboxMh * mailbox)
 {
     struct stat st;
-    gchar* sequences_filename;
+    const gchar* sequences_filename;
     int fd;
     GMimeStream *gmime_stream;
     GMimeStream *gmime_stream_buffer;
@@ -463,7 +501,7 @@ lbm_mh_parse_sequences(LibBalsaMailboxMh * mailbox)
 static void
 lbm_mh_parse_both(LibBalsaMailboxMh * mh)
 {
-    lbm_mh_parse_mailbox(mh);
+    lbm_mh_parse_mailbox(mh, TRUE);
     if (mh->msgno_2_msg_info)
 	lbm_mh_parse_sequences(mh);
 }
@@ -557,9 +595,11 @@ lbm_mh_check(LibBalsaMailboxMh * mh, const gchar * path)
 		p = strchr(*seq, '-');
 		if (p) {
 		    *p++ = '\0';
-		    sscanf(p, "%d", &end);
+		    if (sscanf(p, "%d", &end) != 1)
+                        break; /* FIXME report error? */
 		}
-		sscanf(*seq, "%d", &fileno);
+		if (sscanf(*seq, "%d", &fileno) != 1)
+                    break; /* FIXME report error? */
 		do {
 		    p = g_strdup_printf("%s/%d", path, fileno);
 		    if (access(p, F_OK) == 0)
@@ -700,26 +740,10 @@ libbalsa_mailbox_mh_close_mailbox(LibBalsaMailbox * mailbox,
 }
 
 static int
-libbalsa_mailbox_mh_open_temp (const gchar *dest_path,
-					  char **name_used)
+libbalsa_mailbox_mh_open_temp (const gchar *dest_path, char **name_used)
 {
-    int fd;
-    static int counter;
-    gchar *filename;
-
-    do {
-	filename = g_strdup_printf("%s/%s-%d-%d", dest_path,
-	   		      g_get_user_name(), getpid (), counter++);
-	fd = open (filename, O_WRONLY | O_EXCL | O_CREAT, 0600);
-	if (fd == -1)
-	{
-	    g_free(filename);
-	    if (errno != EEXIST)
-		return -1;
-	}
-    } while (fd == -1);
-    *name_used = filename;
-    return fd;
+    *name_used = g_build_filename(dest_path, "mh-XXXXXX", NULL);
+    return g_mkstemp(*name_used);
 }
 
 struct line_info {
@@ -784,21 +808,20 @@ libbalsa_mailbox_mh_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     int fd;
     int sequences_fd;
     GMimeStream *temp_stream;
-    gchar* sequences_filename;
+    const gchar* sequences_filename;
     GMimeStream *gmime_stream;
     GMimeStream *gmime_stream_buffer;
     GByteArray *line;
-    gboolean retval;
+    gboolean retval = FALSE;
 
     g_return_val_if_fail(LIBBALSA_IS_MAILBOX_MH(mailbox), FALSE);
-    mh = LIBBALSA_MAILBOX_MH(mailbox);
+    mh = (LibBalsaMailboxMh *) mailbox;
     g_return_val_if_fail( mh->messages_info != NULL, FALSE );
 
     sequences_filename = mh->sequences_filename;
     sequences_fd = open(sequences_filename, O_RDONLY);
-    if (sequences_fd == -1)
-	return FALSE;
-    libbalsa_lock_file(sequences_filename, sequences_fd, FALSE, TRUE, 1);
+    if (sequences_fd >= 0)
+        libbalsa_lock_file(sequences_filename, sequences_fd, FALSE, TRUE, 1);
 
     /* Check for new mail before flushing any changes out to disk. */
     libbalsa_mailbox_mh_check(mailbox);
@@ -888,54 +911,65 @@ libbalsa_mailbox_mh_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     fd = libbalsa_mailbox_mh_open_temp(path, &tmp);
     if (fd == -1)
     {
+        g_free(tmp);
 	g_object_unref(unseen.line);
 	g_object_unref(flagged.line);
 	g_object_unref(replied.line);
 	g_object_unref(recent.line);
-	libbalsa_unlock_file(sequences_filename, sequences_fd, 1);
-	return FALSE;
+        if (sequences_fd >= 0)
+            libbalsa_unlock_file(sequences_filename, sequences_fd, 1);
+#ifdef DEBUG
+        g_print("MH sync \"%s\": cannot open temp file.\n", path);
+#endif
+	return retval;
     }
     temp_stream = g_mime_stream_fs_new(fd);
 
-    /* copy unknown sequences */
-    gmime_stream = g_mime_stream_fs_new(sequences_fd);
-    gmime_stream_buffer = g_mime_stream_buffer_new(gmime_stream,
-					GMIME_STREAM_BUFFER_BLOCK_READ);
-    g_object_unref(gmime_stream);
-    line = g_byte_array_new();
-    do {
-	gchar *tmp;
+    if (sequences_fd >= 0) {
+        /* copy unknown sequences */
+        gmime_stream = g_mime_stream_fs_new(sequences_fd);
+        gmime_stream_buffer =
+            g_mime_stream_buffer_new(gmime_stream,
+                                     GMIME_STREAM_BUFFER_BLOCK_READ);
+        g_object_unref(gmime_stream);
+        line = g_byte_array_new();
+        do {
+            gchar *tmp;
 
-	line->len = 0;
-	g_mime_stream_buffer_readln(gmime_stream_buffer, line);
-	tmp = (gchar *) line->data;
-	if (tmp &&
-	    !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhUnseen) &&
-	    !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhFlagged) &&
-	    !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhReplied) &&
-	    !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhRecent))
-	{
-	    /* unknown sequence */
-	    g_mime_stream_write(temp_stream, tmp, line->len);
-	}
-    } while (!g_mime_stream_eos(gmime_stream_buffer));
-    g_object_unref(gmime_stream_buffer);
-    g_byte_array_free(line, TRUE);
+            line->len = 0;
+            g_mime_stream_buffer_readln(gmime_stream_buffer, line);
+            tmp = (gchar *) line->data;
+            if (tmp &&
+                !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhUnseen) &&
+                !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhFlagged) &&
+                !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhReplied) &&
+                !libbalsa_str_has_prefix(tmp, LibBalsaMailboxMhRecent)) {
+                /* unknown sequence */
+                g_mime_stream_write(temp_stream, tmp, line->len);
+            }
+        } while (!g_mime_stream_eos(gmime_stream_buffer));
+        g_object_unref(gmime_stream_buffer);
+        g_byte_array_free(line, TRUE);
+    }
 
     /* write sequences */
     if (!lbm_mh_finish_line(&unseen, temp_stream, LibBalsaMailboxMhUnseen) ||
 	!lbm_mh_finish_line(&flagged, temp_stream, LibBalsaMailboxMhFlagged) ||
 	!lbm_mh_finish_line(&replied, temp_stream, LibBalsaMailboxMhReplied) ||
 	!lbm_mh_finish_line(&recent, temp_stream, LibBalsaMailboxMhRecent)) {
+	g_object_unref(temp_stream);
 	unlink(tmp);
 	g_free(tmp);
-	g_object_unref(temp_stream);
 	g_object_unref(unseen.line);
 	g_object_unref(flagged.line);
 	g_object_unref(replied.line);
 	g_object_unref(recent.line);
-	libbalsa_unlock_file(sequences_filename, sequences_fd, 1);
-	return FALSE;
+        if (sequences_fd >= 0)
+            libbalsa_unlock_file(sequences_filename, sequences_fd, 1);
+#ifdef DEBUG
+        g_print("MH sync \"%s\": error finishing sequences line.\n", path);
+#endif
+	return retval;
     }
 
     /* close tempfile */
@@ -946,21 +980,25 @@ libbalsa_mailbox_mh_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 
     /* rename tempfile to '.mh_sequences' */
     retval = (libbalsa_safe_rename(tmp, sequences_filename) != -1);
+#ifdef DEBUG
+    if (!retval)
+        g_print("MH sync \"%s\": error renaming sequences file.\n", path);
+#endif
     if (!retval)
 	unlink (tmp);
 
-    g_free(tmp);
-    g_object_unref(unseen.line);
-    g_object_unref(flagged.line);
-    g_object_unref(replied.line);
-    g_object_unref(recent.line);
-
     /* Record the mtimes; we'll just use the current time--someone else
      * might have changed something since we did, despite the file
      * locking, but we'll find out eventually. */
     libbalsa_mailbox_set_mtime(mailbox, mh->mtime_sequences = time(NULL));
 
-    libbalsa_unlock_file(sequences_filename, sequences_fd, 1);
+    g_free(tmp);
+    g_object_unref(unseen.line);
+    g_object_unref(flagged.line);
+    g_object_unref(replied.line);
+    g_object_unref(recent.line);
+    if (sequences_fd >= 0)
+        libbalsa_unlock_file(sequences_filename, sequences_fd, 1);
     return retval;
 }
 
@@ -976,32 +1014,48 @@ lbm_mh_message_info_from_msgno(LibBalsaMailboxMh * mh, guint msgno)
 }
 
 static LibBalsaMessage *
-libbalsa_mailbox_mh_get_message(LibBalsaMailbox * mailbox, guint msgno)
+lbm_mh_get_message(LibBalsaMailbox * mailbox, guint msgno)
 {
     struct message_info *msg_info;
+    LibBalsaMessage *message;
 
-    msg_info =
-	lbm_mh_message_info_from_msgno(LIBBALSA_MAILBOX_MH(mailbox),
-				       msgno);
+    msg_info = lbm_mh_message_info_from_msgno(LIBBALSA_MAILBOX_MH(mailbox),
+                                              msgno);
+
+    message = msg_info->message;
+    if (message)
+        return g_object_ref(message);
+
+    msg_info->message = message = libbalsa_message_new();
+    g_object_add_weak_pointer(G_OBJECT(message),
+                              (gpointer) & msg_info->message);
+
+    if (msg_info->flags == INVALID_FLAG)
+        msg_info->flags = msg_info->orig_flags;
+    message->flags = msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
+    message->mailbox = mailbox;
+    message->msgno = msgno;
+    libbalsa_message_load_envelope(message);
+
+    libbalsa_mailbox_local_cache_message(LIBBALSA_MAILBOX_LOCAL(mailbox),
+                                         msgno, message);
+
+    return message;
+}
+
+static LibBalsaMessageFlag
+lbm_mh_load_message(LibBalsaMailbox * mailbox, guint msgno,
+                    LibBalsaMessage ** msg)
+{
+    struct message_info *msg_info =
+        lbm_mh_message_info_from_msgno(LIBBALSA_MAILBOX_MH(mailbox), msgno);
 
     if (msg_info->message)
-	g_object_ref(msg_info->message);
-    else {
-	LibBalsaMessage *message;
+        *msg = g_object_ref(msg_info->message);
 
-	msg_info->message = message = libbalsa_message_new();
-	g_object_add_weak_pointer(G_OBJECT(message),
-				  (gpointer) & msg_info->message);
-
-	if (msg_info->flags == INVALID_FLAG)
-	    msg_info->flags = msg_info->orig_flags;
-	message->flags = msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
-	message->mailbox = mailbox;
-	message->msgno = msgno;
-	libbalsa_message_load_envelope(message);
-    }
-
-    return msg_info->message;
+    return (msg_info->flags == INVALID_FLAG ?
+            msg_info->orig_flags : msg_info->flags)
+        & LIBBALSA_MESSAGE_FLAGS_REAL;
 }
 
 static gboolean
@@ -1060,22 +1114,27 @@ lbm_mh_update_sequences(LibBalsaMailboxMh * mh, gint fileno,
 }
 
 /* Called with mailbox locked. */
-static int
+static gboolean
 libbalsa_mailbox_mh_add_message(LibBalsaMailbox * mailbox,
-                                LibBalsaMessage * message, GError **err)
+                                GMimeStream * stream,
+                                LibBalsaMessageFlag flags,
+                                GError ** err)
 {
     LibBalsaMailboxMh *mh;
     const char *path;
     char *tmp;
     int fd;
     GMimeStream *out_stream;
+    GMimeFilter *crlffilter;
     int fileno;
     int retries;
     GMimeStream *in_stream;
 
     mh = LIBBALSA_MAILBOX_MH(mailbox);
 
-    lbm_mh_parse_both(mh);
+    /* Make sure we know the highest message number: */
+    lbm_mh_parse_mailbox(mh, FALSE);
+
     /* open tempfile */
     path = libbalsa_mailbox_local_get_path(mailbox);
     fd = libbalsa_mailbox_mh_open_temp(path, &tmp);
@@ -1083,34 +1142,31 @@ libbalsa_mailbox_mh_add_message(LibBalsaMailbox * mailbox,
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_APPEND_ERROR,
                     _("Cannot create message"));
-	return -1;
+	g_free(tmp);
+	return FALSE;
     }
     out_stream = g_mime_stream_fs_new(fd);
-    {
-	GMimeStream *tmp = libbalsa_message_stream(message);
-	GMimeFilter *crlffilter = 
-	    g_mime_filter_crlf_new (  GMIME_FILTER_CRLF_DECODE,
-				      GMIME_FILTER_CRLF_MODE_CRLF_ONLY );
-	in_stream = g_mime_stream_filter_new_with_stream (tmp);
-	g_mime_stream_filter_add ( GMIME_STREAM_FILTER (in_stream), crlffilter );
-	g_object_unref (crlffilter);
-	g_object_unref (tmp);
-    }    
-    libbalsa_mime_stream_shared_lock(in_stream);
-    if (g_mime_stream_write_to_stream( in_stream, out_stream) == -1)
-    {
-        libbalsa_mime_stream_shared_unlock(in_stream);
+
+    crlffilter = g_mime_filter_crlf_new(GMIME_FILTER_CRLF_DECODE,
+                                        GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
+    in_stream = g_mime_stream_filter_new_with_stream(stream);
+    g_mime_stream_filter_add(GMIME_STREAM_FILTER(in_stream), crlffilter);
+    g_object_unref(crlffilter);
+
+    libbalsa_mime_stream_shared_lock(stream);
+    if (g_mime_stream_write_to_stream(in_stream, out_stream) == -1) {
+        libbalsa_mime_stream_shared_unlock(stream);
         g_object_unref(in_stream);
 	g_object_unref(out_stream);
-	unlink (tmp);
-	g_free(tmp);
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_APPEND_ERROR,
                     _("Data copy error"));
-	return -1;
+	unlink(tmp);
+	g_free(tmp);
+	return FALSE;
     }
     g_object_unref(out_stream);
-    libbalsa_mime_stream_shared_unlock(in_stream);
+    libbalsa_mime_stream_shared_unlock(stream);
     g_object_unref(in_stream);
 
     fileno = mh->last_fileno; 
@@ -1126,14 +1182,13 @@ libbalsa_mailbox_mh_add_message(LibBalsaMailbox * mailbox,
 	if (rename_status != -1)
 	    break;
 	
-	if (errno != EEXIST)
-	{
-	    unlink (tmp);
-	    g_free(tmp);
+	if (errno != EEXIST) {
             g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                         LIBBALSA_MAILBOX_APPEND_ERROR,
                         _("Message rename error"));
-	    return -1;
+	    unlink (tmp);
+	    g_free(tmp);
+	    return FALSE;
 	}
     } while (--retries > 0);
     g_free(tmp);
@@ -1142,14 +1197,14 @@ libbalsa_mailbox_mh_add_message(LibBalsaMailbox * mailbox,
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_APPEND_ERROR,
                     "Too high activity?");
-	return -1;
+	return FALSE;
     }
     mh->last_fileno = fileno;
 
     lbm_mh_update_sequences(mh, fileno,
-			    message->flags | LIBBALSA_MESSAGE_FLAG_RECENT);
+                            flags | LIBBALSA_MESSAGE_FLAG_RECENT);
 
-    return 1;
+    return TRUE;
 }
 
 static gboolean
@@ -1185,7 +1240,6 @@ libbalsa_mailbox_mh_messages_change_flags(LibBalsaMailbox * mailbox,
 		msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
 
         libbalsa_mailbox_index_set_flags(mailbox, msgno, msg_info->flags);
-        libbalsa_mailbox_msgno_changed(mailbox, msgno);
 
         was_unread_undeleted = (old_flags & LIBBALSA_MESSAGE_FLAG_NEW)
             && !(old_flags & LIBBALSA_MESSAGE_FLAG_DELETED);
