@@ -44,11 +44,9 @@
 int PopDebug = 0;
 
 enum {
-    CONFIG_CHANGED,
     LAST_SIGNAL
 };
 static LibBalsaMailboxClass *parent_class = NULL;
-static guint libbalsa_mailbox_pop3_signals[LAST_SIGNAL];
 
 static void libbalsa_mailbox_pop3_finalize(GObject * object);
 static void libbalsa_mailbox_pop3_class_init(LibBalsaMailboxPop3Class *
@@ -101,14 +99,6 @@ libbalsa_mailbox_pop3_class_init(LibBalsaMailboxPop3Class * klass)
     libbalsa_mailbox_class = LIBBALSA_MAILBOX_CLASS(klass);
 
     parent_class = g_type_class_peek_parent(klass);
-    libbalsa_mailbox_pop3_signals[CONFIG_CHANGED] =
-	g_signal_new("config-changed",
-                     G_TYPE_FROM_CLASS(object_class),
-                     G_SIGNAL_RUN_LAST | G_SIGNAL_NO_HOOKS,
-                     G_STRUCT_OFFSET(LibBalsaMailboxPop3Class,
-                                     config_changed),
-                     NULL, NULL,
-                     g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     object_class->finalize = libbalsa_mailbox_pop3_finalize;
 
@@ -159,16 +149,6 @@ libbalsa_mailbox_pop3_new(void)
 }
 
 
-static void
-libbalsa_mailbox_pop3_config_changed(LibBalsaMailboxPop3* mailbox)
-{
-    g_return_if_fail(mailbox != NULL);
-    g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox));
-
-    g_signal_emit(G_OBJECT(mailbox), 
-                  libbalsa_mailbox_pop3_signals[CONFIG_CHANGED], 0);
-}
-
 static gboolean
 libbalsa_mailbox_pop3_open(LibBalsaMailbox * mailbox, GError **err)
 {
@@ -182,40 +162,35 @@ libbalsa_mailbox_pop3_open(LibBalsaMailbox * mailbox, GError **err)
 }
 
 static gboolean
-move_from_msgdrop(LibBalsaMailbox *tmp_mailbox, LibBalsaMailbox *dest,
+move_from_msgdrop(const gchar *tmp_path, LibBalsaMailbox *dest,
                   gchar *pop_name, unsigned msgno)
 {
-    gboolean success = TRUE;
+    gint fd;
+    GMimeStream *stream;
+    gboolean success;
     GError *err = NULL;
-    if (libbalsa_mailbox_open(tmp_mailbox, NULL)) {
-	guint msgno, total = libbalsa_mailbox_total_messages(tmp_mailbox);
-        if(total>0) { /* message might been already filtered away by
-                         procmail */
-            GArray *messages = g_array_new(FALSE, FALSE, sizeof(guint));
-            
-            for (msgno = 1; msgno <= total; msgno++)
-                g_array_append_val(messages, msgno);
-            
-            if (!libbalsa_mailbox_messages_change_flags
-                (tmp_mailbox, messages,
-                 LIBBALSA_MESSAGE_FLAG_NEW | LIBBALSA_MESSAGE_FLAG_RECENT, 0) ||
-                !libbalsa_mailbox_sync_storage(tmp_mailbox, FALSE))
-                libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                                     _("Error setting flags on messages "
-                                       "in mailbox %s"), tmp_mailbox->name);
-            
-            success = libbalsa_mailbox_messages_move(tmp_mailbox, messages,
-                                                     dest, &err);
-            g_array_free(messages, TRUE);
-        }
-        libbalsa_mailbox_close(tmp_mailbox, TRUE);
-    }
+    struct stat buf;
+
+    if(stat(tmp_path, &buf) != 0)
+        return FALSE;
+    if(buf.st_size == 0) /* filtered out? Nothing to move in any case */
+        return TRUE;
+    fd = open(tmp_path, O_RDWR);
+    stream = g_mime_stream_fs_new(fd);
+    success =
+        libbalsa_mailbox_add_message(dest, stream,
+                                     LIBBALSA_MESSAGE_FLAG_NEW |
+                                     LIBBALSA_MESSAGE_FLAG_RECENT, &err)
+        && ftruncate(fd, 0) == 0;
+    g_object_unref(stream);
+
     if(!success)
         libbalsa_information(LIBBALSA_INFORMATION_WARNING,
                              _("Error appending message %d from %s to %s: %s"),
                              msgno, pop_name, dest->name,
                              err ? err->message : "?");
     g_clear_error(&err);
+
     return success;
 }
 
@@ -224,7 +199,8 @@ static GHashTable*
 mp_load_uids(void)
 {
     char line[1024]; /* arbitrary limit of uid len */
-    GHashTable *res = g_hash_table_new(g_str_hash, g_str_equal);
+    GHashTable *res = g_hash_table_new_full(g_str_hash, g_str_equal,
+                                            g_free, NULL);
     gchar *fname = g_strconcat(g_get_home_dir(), "/.balsa/pop-uids", NULL);
     FILE *f = fopen(fname, "r");
     g_free(fname);
@@ -297,23 +273,12 @@ dump_cb(unsigned len, char *buf, void *arg)
 /* ===================================================================
    Functions for direct or filtered saving to the output mailbox.
 */
-static gchar*
-pop_direct_get_path(const char *pattern, unsigned msgno)
-{
-    return g_strdup_printf("%s/%u", pattern, msgno);
-}
-
 static FILE*
 pop_direct_open(const char *path)
 {
     return fopen(path, "w");
 }
 
-static gchar*
-pop_filter_get_path(const char *pattern, unsigned msgno)
-{
-    return g_strdup(pattern);
-}
 static FILE*
 pop_filter_open(const char *command)
 {
@@ -321,15 +286,12 @@ pop_filter_open(const char *command)
 }
 
 struct PopDownloadMode {
-    gchar* (*get_path)(const char *pattern, unsigned msgno);
     FILE* (*open)(const char *path);
     int (*close)(FILE *arg);
 } pop_direct = {
-    pop_direct_get_path,
     pop_direct_open,
     fclose
 }, pop_filter = {
-    pop_filter_get_path,
     pop_filter_open,
     pclose
 };
@@ -339,10 +301,10 @@ struct PopDownloadMode {
 */
 struct fetch_data {
     LibBalsaMailbox *mailbox;
-    LibBalsaMailbox *tmp; /* temporary drop mailbox */
     PopHandle *pop;
     GError   **err;
-    gchar     *msg_name;
+    const gchar     *tmp_path;
+    const gchar     *dest_path;
     const struct PopDownloadMode *dm;
     unsigned   msgno, total_messages;
     unsigned long *current_pos;
@@ -369,10 +331,16 @@ fetch_async_cb(PopReqCode prc, void *arg, ...)
     switch(prc) {
     case POP_REQ_OK: 
 	{/* GError **err = va_arg(alist, GError**); */
-        fd->f  = fd->dm->open(fd->msg_name);
-        if(fd->f == NULL)
-            printf("fopen failed for %s\n", fd->msg_name);
-        else {
+        fd->f  = fd->dm->open(fd->dest_path);
+        if(fd->f == NULL) {
+            if(!*fd->err) {
+                g_set_error(fd->err, IMAP_ERROR, IMAP_POP_SAVE_ERROR,
+                            _("Saving POP message to %s failed"),
+                            fd->dest_path);
+            }
+            fd->error_occured = 1;
+            printf("fopen failed for %s %p\n", fd->dest_path, fd->err);
+        } else {
             snprintf(threadbuf, sizeof(threadbuf),
                      _("Retrieving Message %d of %d"),
                     fd->msgno, fd->total_messages);
@@ -412,7 +380,7 @@ fetch_async_cb(PopReqCode prc, void *arg, ...)
                 if(!*fd->err)
                     g_set_error(fd->err, IMAP_ERROR, IMAP_POP_SAVE_ERROR,
                                 _("Saving POP message to %s failed."),
-                                fd->msg_name);
+                                fd->dest_path);
                 fd->error_occured = 1;
             }
 	}
@@ -422,10 +390,11 @@ fetch_async_cb(PopReqCode prc, void *arg, ...)
                 if(!*fd->err)
                     g_set_error(fd->err, IMAP_ERROR, IMAP_POP_SAVE_ERROR,
                                 _("Transfering POP message to %s failed."),
-                                fd->msg_name);
+                                fd->dest_path);
             } else if(!fd->error_occured &&
                       move_from_msgdrop
-                      (fd->tmp, LIBBALSA_MAILBOX_POP3(fd->mailbox)->inbox,
+                      (fd->tmp_path,
+                       LIBBALSA_MAILBOX_POP3(fd->mailbox)->inbox,
                        fd->mailbox->name, fd->msgno) &&
                       fd->delete_on_server)
                 pop_delete_message(fd->pop, fd->msgno, NULL, NULL, NULL);
@@ -438,7 +407,6 @@ fetch_async_cb(PopReqCode prc, void *arg, ...)
 static void
 async_notify(struct fetch_data *fd)
 {
-    g_free(fd->msg_name);
     g_free(fd);
 }
 
@@ -469,10 +437,9 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
 {
     gchar *tmp_path, *dest_path, *uid_prefix = NULL;
     gint tmp_file;
-    LibBalsaMailbox *tmp_mailbox;
     LibBalsaMailboxPop3 *m = LIBBALSA_MAILBOX_POP3(mailbox);
     LibBalsaServer *server;
-    gchar *msgbuf, *mhs;
+    gchar *msgbuf;
     GError *err = NULL;
     unsigned msgcnt, i;
     GHashTable *uids = NULL, *current_uids = NULL;
@@ -480,7 +447,7 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
     unsigned long current_pos = 0, total_size;
     ImapTlsMode tls_mode;
     PopHandle *pop;
- 
+
     if (!m->check || !m->inbox) return;
 
     server = LIBBALSA_MAILBOX_REMOTE_SERVER(m);
@@ -495,44 +462,16 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
     libbalsa_mailbox_progress_notify(mailbox, LIBBALSA_NTFY_SOURCE,0,0,msgbuf);
     g_free(msgbuf);
 
-    do {
-	tmp_path = g_strdup("/tmp/pop-XXXXXX");
-	tmp_file = g_mkstemp(tmp_path);
-    } while ((tmp_file < 0) && (errno == EEXIST));
-
-    if(tmp_file < 0) {
+    tmp_file = g_file_open_tmp("pop-XXXXXX", &tmp_path, &err);
+    if (tmp_file < 0) {
 	libbalsa_information(LIBBALSA_INFORMATION_WARNING,
 			     _("POP3 mailbox %s temp file error:\n%s"), 
-			     mailbox->name,
-			     g_strerror(errno));
-	g_free(tmp_path);
+			     mailbox->name, err->message);
+	g_error_free(err);
 	return;
     }
     close(tmp_file);
-    unlink(tmp_path);
 
-    if( mkdir(tmp_path, 0700) < 0 ) {
-	libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-			     _("POP3 mailbox %s temp file error:\n%s"), 
-			     mailbox->name,
-			     g_strerror(errno));
-	g_free(tmp_path);
-	return;	
-    }
-    
-    mhs = g_strdup_printf ( "%s/.mh_sequences", tmp_path );
-    if( (tmp_file=creat( mhs, 0600)) != -1) close(tmp_file);
-    /* we fake a real mh box - it's good enough */
-    tmp_mailbox = (LibBalsaMailbox*)
-        libbalsa_mailbox_mh_new(tmp_path, FALSE);
-    if(!tmp_mailbox)  {
-	libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-			     _("POP3 mailbox %s temp mailbox error:\n"), 
-			     mailbox->name);
-	g_free(tmp_path); g_free(mhs);
-	return;
-    }
-    
     pop = pop_new();
     pop_set_option(pop, IMAP_POP_OPT_FILTER_CR, TRUE);
     pop_set_option(pop, IMAP_POP_OPT_OVER_SSL, server->use_ssl);
@@ -554,7 +493,8 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
 			     _("POP3 mailbox %s error: %s\n"), 
 			     mailbox->name, err->message);
         g_error_free(err);
-	g_free(tmp_path); g_free(mhs);
+        unlink(tmp_path); g_free(tmp_path);
+        pop_destroy(pop, NULL);
 	return;
     }
 
@@ -565,11 +505,12 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
     total_size = pop_get_total_size(pop);
     if(!m->delete_from_server) {
         uids = mp_load_uids();
-        current_uids = g_hash_table_new(g_str_hash, g_str_equal);
+        current_uids = g_hash_table_new_full(g_str_hash, g_str_equal,
+                                             g_free, NULL);
         uid_prefix = g_strconcat(server->user, "@", server->host, NULL);
     }
+
     for(i=1; i<=msgcnt; i++) {
-        char *msg_path = mode->get_path(dest_path, i);
         unsigned msg_size = pop_get_msg_size(pop, i);
 #if POP_SYNC
         FILE *f;
@@ -578,25 +519,28 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
             const char *uid = pop_get_uid(pop, i, NULL);
             char *full_uid = g_strconcat(uid_prefix, " ", uid, NULL);
             g_hash_table_insert(current_uids, full_uid, GINT_TO_POINTER(1));
-            if(g_hash_table_lookup(uids, full_uid))
+            if(g_hash_table_lookup(uids, full_uid)) {
+                total_size -= msg_size;
                 continue;
+            }
         }
         if(m->msg_size_limit>0 && msg_size >= (unsigned)m->msg_size_limit) {
             libbalsa_information
                 (LIBBALSA_INFORMATION_WARNING,
                  _("POP3 message %d oversized: %d kB - skipped."),
                  i, msg_size);
+            total_size -= msg_size;
             continue;
         }
 #if POP_SYNC
         libbalsa_mailbox_progress_notify(mailbox,
                                          LIBBALSA_NTFY_PROGRESS, i, msgcnt,
                                          "next message");
-        f = mode->open(msg_path);
+        f = mode->open(dest_path);
         if(!f) {
             libbalsa_information(LIBBALSA_INFORMATION_ERROR,
 			     _("POP3 error: cannot open %s for writing."), 
-			     msg_path);
+			     dest_path);
             break;
         }
         if(!pop_fetch_message_s(pop, i, dump_cb, f, &err)) 
@@ -604,21 +548,20 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
         if(mode->close(f) != 0) {
             libbalsa_information(LIBBALSA_INFORMATION_ERROR,
 			     _("POP3 error: cannot close %s."), 
-			     msg_path);
+			     dest_path);
             break;
         }
-        if(move_from_msgdrop(tmp_mailbox, m->inbox,
+        if(move_from_msgdrop(tmp_path, m->inbox,
                              mailbox->name, i) &&
            m->delete_from_server)
             pop_delete_message_s(pop, i, NULL);
-        g_free(msg_path);
 #else 
 	{struct fetch_data *fd = g_new0(struct fetch_data, 1);
         fd->mailbox  = mailbox;
-        fd->tmp      = tmp_mailbox;
+        fd->tmp_path = tmp_path;
 	fd->pop      = pop; 
 	fd->err      = &err;
-	fd->msg_name = msg_path;
+	fd->dest_path= dest_path;
 	fd->dm       = mode;
 	fd->msgno    = i;
         fd->total_messages = msgcnt;
@@ -630,14 +573,17 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
 	}
 #endif
     }
+    pop_destroy(pop, NULL);
     if(err) {
-        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+        /* 
+        libbalsa_mailbox_progress_notify(mailbox,
+                                         LIBBALSA_NTFY_ERROR, 0,
+                                         1, err->message);*/
+        libbalsa_information(LIBBALSA_INFORMATION_ERROR,
 			     _("POP3 error: %s."), 
 			     err->message);
-        g_clear_error(&err);
+        g_error_free(err);
     }
-
-    pop_destroy(pop, NULL);
     if(!m->delete_from_server) {
         mp_save_uids(uids, current_uids, uid_prefix);
         g_hash_table_destroy(uids);
@@ -652,12 +598,8 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
      * Postprocessing...
      * =================================================================== */
 
-    libbalsa_mailbox_local_remove_files(LIBBALSA_MAILBOX_LOCAL(tmp_mailbox));
-    libbalsa_mailbox_pop3_config_changed(m);
-    g_object_unref(G_OBJECT(tmp_mailbox));
-
+    unlink(tmp_path);
     g_free(tmp_path);
-    g_free(mhs);
 }
 
 static void
