@@ -43,6 +43,7 @@
 
 /* needed for truncate_string */
 #include "misc.h"
+
 #include "mime-stream-shared.h"
 #include "i18n.h"
 
@@ -294,7 +295,7 @@ canonize_header_value(gchar *value)
    The list has to be freed by the following chunk of code:
     FREE_HEADER_LIST(list);
 */
-gchar **
+static gchar **
 libbalsa_create_hdr_pair(const gchar * name, gchar * value)
 {
     gchar **item = g_new(gchar *, 3);
@@ -309,19 +310,18 @@ libbalsa_create_hdr_pair(const gchar * name, gchar * value)
 /** libbalsa_message_find_user_hdr:
     returns.... list element matching given header.
 */
-GList *
+static GList *
 libbalsa_message_find_user_hdr(LibBalsaMessage * message, const gchar * find)
 {
     GList* list;
-    gchar** tmp;
     LibBalsaMessageHeaders *headers = message->headers;
     
     g_return_val_if_fail(headers, NULL);
     if (!headers->user_hdrs && message->mailbox) 
         libbalsa_mailbox_set_msg_headers(message->mailbox, message);
 
-    for (list = headers->user_hdrs; list; list = g_list_next(list)) {
-        tmp = list->data;
+    for (list = headers->user_hdrs; list; list = list->next) {
+        const gchar * const *tmp = list->data;
         
         if (g_ascii_strncasecmp(tmp[0], find, strlen(find)) == 0) 
             return list;
@@ -330,9 +330,55 @@ libbalsa_message_find_user_hdr(LibBalsaMessage * message, const gchar * find)
     return NULL;
 }
 
+/* 
+ * Public user header methods
+ */
+const gchar *
+libbalsa_message_get_user_header(LibBalsaMessage * message,
+                                 const gchar * name)
+{
+    GList *header;
+    const gchar *const *pair;
+    
+    g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), NULL);
+    g_return_val_if_fail(name != NULL, NULL);
+
+    if (!(header = libbalsa_message_find_user_hdr(message, name)))
+        return NULL;
+
+    pair = header->data;
+    return pair[1];
+}
+
+void
+libbalsa_message_set_user_header(LibBalsaMessage * message,
+                                 const gchar * name, const gchar * value)
+{
+    LibBalsaMessageHeaders *headers;
+    GList *header;
+
+    g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
+    g_return_if_fail(name != NULL);
+
+    headers = message->headers;
+    g_return_if_fail(headers != NULL);
+
+    if ((header = libbalsa_message_find_user_hdr(message, name))) {
+        headers->user_hdrs =
+            g_list_remove_link(headers->user_hdrs, header);
+        FREE_HEADER_LIST(header);
+    }
+
+    if (value && *value)
+        headers->user_hdrs =
+            g_list_prepend(headers->user_hdrs,
+                           libbalsa_create_hdr_pair(name,
+                                                    g_strdup(value)));
+}
+
 static void
 prepend_header_misc(const char *name, const char *value,
-			 gpointer user_data)
+                    gpointer user_data)
 {
     char lcname[17]; /* one byte longer than the longest ignored header */
     static const char ignored_headers[] =
@@ -357,7 +403,7 @@ prepend_header_misc(const char *name, const char *value,
     *(GList **)user_data = res;
 }
 
-/* libbalsa_message_user_hdrs,
+/* 
  * libbalsa_message_user_hdrs_from_gmime:
  * 
  * returns allocated GList containing (header=>value) ALL headers
@@ -388,6 +434,7 @@ libbalsa_message_user_hdrs_from_gmime(GMimeMessage * message)
     */
     value = g_mime_message_get_header(message, "References");
     if (value) {
+#if BALSA_NEEDS_SEPARATE_USER_HEADERS
 	GMimeReferences *references, *reference;
 	reference = references = g_mime_references_decode(value);
 	while (reference) {
@@ -401,6 +448,11 @@ libbalsa_message_user_hdrs_from_gmime(GMimeMessage * message)
 	    reference = reference->next;
 	}
 	g_mime_references_clear(&references);
+#else
+        res = g_list_prepend(res,
+                             libbalsa_create_hdr_pair("References",
+                                                      g_strdup(value)));
+#endif
     }
 
     value = g_mime_message_get_header(message, "In-Reply-To");
@@ -429,7 +481,7 @@ libbalsa_message_get_part_by_id(LibBalsaMessage* msg, const gchar* id)
     LibBalsaMessageBody* body = 
 	libbalsa_message_body_get_by_id(msg->body_list,	id);
     if(!body) return NULL;
-    return libbalsa_message_body_get_stream(body);
+    return libbalsa_message_body_get_stream(body, NULL); /*provide more info?*/
 }
 
 /* libbalsa_message_save:
@@ -452,9 +504,9 @@ libbalsa_message_save(LibBalsaMessage * message, const gchar *filename)
     if (msg_stream == NULL)
 	return FALSE;
     out_stream = g_mime_stream_file_new(outfile);
-    libbalsa_mime_stream_shared_lock(msg_stream);
+    libbalsa_mailbox_lock_store(message->mailbox);
     res = g_mime_stream_write_to_stream(msg_stream, out_stream);
-    libbalsa_mime_stream_shared_unlock(msg_stream);
+    libbalsa_mailbox_unlock_store(message->mailbox);
 
     g_object_unref(msg_stream);
     g_object_unref(out_stream);
@@ -1035,9 +1087,7 @@ void
 libbalsa_message_set_in_reply_to_from_string(LibBalsaMessage * message,
 					     const gchar * str)
 {
-    g_return_if_fail(message->in_reply_to == NULL);
-
-    if (str) {
+    if (!message->in_reply_to && str) {
 	/* FIXME for Balsa's old non-compliant header */
 	gchar *p = strrchr(str, ';');
 	p = p ? g_strndup(str, p - str) : g_strdup(str);
@@ -1144,20 +1194,18 @@ libbalsa_message_set_headers_from_string(LibBalsaMessage *message,
 }
 
 void
-libbalsa_message_load_envelope(LibBalsaMessage *message)
+libbalsa_message_load_envelope_from_stream(LibBalsaMessage * message,
+                                           GMimeStream *gmime_stream)
 {
-    GMimeStream *gmime_stream;
     GMimeStream *gmime_stream_buffer;
     GByteArray *line;
     guchar lookahead;
     gboolean ret = FALSE;
 
-    gmime_stream = libbalsa_message_stream(message);
-    if (!gmime_stream)
-	return;
     libbalsa_mime_stream_shared_lock(gmime_stream);
-    gmime_stream_buffer = g_mime_stream_buffer_new(gmime_stream,
-					GMIME_STREAM_BUFFER_BLOCK_READ);
+    gmime_stream_buffer =
+        g_mime_stream_buffer_new(gmime_stream,
+                                 GMIME_STREAM_BUFFER_BLOCK_READ);
 
     line = g_byte_array_new();
     do {
@@ -1194,10 +1242,22 @@ libbalsa_message_load_envelope(LibBalsaMessage *message)
     }
     g_object_unref(gmime_stream_buffer);
     libbalsa_mime_stream_shared_unlock(gmime_stream);
-    g_object_unref(gmime_stream);
     g_byte_array_free(line, TRUE);
 }
 
+void
+libbalsa_message_load_envelope(LibBalsaMessage *message)
+{
+    GMimeStream *gmime_stream;
+
+    gmime_stream = libbalsa_message_stream(message);
+    if (!gmime_stream)
+	return;
+
+    libbalsa_message_load_envelope_from_stream(message, gmime_stream);
+    g_object_unref(gmime_stream);
+}
+
 GMimeStream *
 libbalsa_message_stream(LibBalsaMessage * message)
 {
@@ -1220,3 +1280,32 @@ libbalsa_message_stream(LibBalsaMessage * message)
 
     return mime_stream;
 }
+
+gboolean
+libbalsa_message_copy(LibBalsaMessage * message, LibBalsaMailbox * dest,
+                      GError ** err)
+{
+    LibBalsaMailbox *mailbox;
+    gboolean retval;
+
+    g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), FALSE);
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX(dest), FALSE);
+    mailbox = message->mailbox;
+    g_return_val_if_fail(mailbox != NULL || message->mime_msg != NULL,
+                         FALSE);
+
+    if (mailbox) {
+        GArray *msgnos = g_array_sized_new(FALSE, FALSE, sizeof(guint), 1);
+        g_array_append_val(msgnos, message->msgno);
+        retval =
+            libbalsa_mailbox_messages_copy(mailbox, msgnos, dest, err);
+        g_array_free(msgnos, TRUE);
+    } else {
+        GMimeStream *mime_stream = libbalsa_message_stream(message);
+        retval = libbalsa_mailbox_add_message(dest, mime_stream,
+                                              message->flags, err);
+        g_object_unref(mime_stream);
+    }
+
+    return retval;
+}
