@@ -305,8 +305,6 @@ gboolean libbalsa_message_postpone(LibBalsaMessage * message,
 /*
  * misc message releated functions
  */
-gchar **libbalsa_create_hdr_pair(const gchar * name, gchar * value);
-
 const gchar *libbalsa_message_pathname(LibBalsaMessage * message);
 const gchar *libbalsa_message_body_charset(LibBalsaMessageBody * body);
 gboolean libbalsa_message_is_multipart(LibBalsaMessage * message);
@@ -318,8 +316,12 @@ gboolean libbalsa_message_is_pgp_signed(LibBalsaMessage * message);
 gboolean libbalsa_message_is_pgp_encrypted(LibBalsaMessage * message);
 #endif
 
-GList *libbalsa_message_find_user_hdr(LibBalsaMessage * message, 
-                                      const gchar * find);
+const gchar *libbalsa_message_get_user_header(LibBalsaMessage * message,
+                                              const gchar * name);
+void libbalsa_message_set_user_header(LibBalsaMessage * message,
+                                      const gchar * name,
+                                      const gchar * value);
+
 GMimeStream *libbalsa_message_get_part_by_id(LibBalsaMessage * message,
                                              const gchar * id);
 
@@ -337,11 +339,13 @@ glong libbalsa_message_get_length(LibBalsaMessage* msg);
 glong libbalsa_message_get_no(LibBalsaMessage* msg);
 LibBalsaMessageAttach libbalsa_message_get_attach_icon(LibBalsaMessage *
 						       message);
-#define libbalsa_message_date_to_gchar(m, f) libbalsa_date_to_gchar(&(m)->headers->date, (f))
-#define libbalsa_message_headers_date_to_gchar(h, f) libbalsa_date_to_gchar(&(h)->date, (f))
+#define libbalsa_message_date_to_utf8(m, f) libbalsa_date_to_utf8(&(m)->headers->date, (f))
+#define libbalsa_message_headers_date_to_utf8(h, f) libbalsa_date_to_utf8(&(h)->date, (f))
 
 GList *libbalsa_message_refs_for_threading(LibBalsaMessage* msg);
 
+void libbalsa_message_load_envelope_from_stream(LibBalsaMessage * message,
+                                                GMimeStream * stream);
 void libbalsa_message_load_envelope(LibBalsaMessage *message);
 gboolean libbalsa_message_set_headers_from_string(LibBalsaMessage *message,
                                                   const gchar *str);
@@ -350,4 +354,6 @@ void libbalsa_message_set_references_from_string(LibBalsaMessage * message,
 void libbalsa_message_set_in_reply_to_from_string(LibBalsaMessage * message,
 						  const gchar *str);
 GMimeStream *libbalsa_message_stream(LibBalsaMessage * message);
+gboolean libbalsa_message_copy(LibBalsaMessage * message,
+                               LibBalsaMailbox * dest, GError ** err);
 #endif				/* __LIBBALSA_MESSAGE_H__ */
