@@ -47,7 +47,6 @@
 #  include "misc.h"
 #endif
 
-#include "mime-stream-shared.h"
 #include "padlock-keyhole.xpm"
 #include "i18n.h"
 
@@ -488,7 +487,6 @@ libbalsa_body_check_signature(LibBalsaMessageBody * body,
     GMimeCipherContext *ctx;
     GMimeSignatureValidity *valid;
     GError *error = NULL;
-    GMimeStream *stream;
 
     /* paranoia checks */
     g_return_val_if_fail(body, FALSE);
@@ -503,8 +501,10 @@ libbalsa_body_check_signature(LibBalsaMessageBody * body,
 	return FALSE;
 
     /* check if the body is really a multipart/signed */
-    if (!GMIME_IS_MULTIPART_SIGNED(body->mime_part))
-	return FALSE;
+    if (!GMIME_IS_MULTIPART_SIGNED(body->mime_part)
+        || (g_mime_multipart_get_number
+            (((GMimeMultipart *) body->mime_part))) < 2)
+        return FALSE;
     if (body->parts->next->sig_info)
 	g_object_unref(G_OBJECT(body->parts->next->sig_info));
 
@@ -539,12 +539,10 @@ libbalsa_body_check_signature(LibBalsaMessageBody * body,
 
     /* verify the signature */
 
-    stream = libbalsa_message_stream(body->message);
-    libbalsa_mime_stream_shared_lock(stream);
+    libbalsa_mailbox_lock_store(body->message->mailbox);
     valid = g_mime_multipart_signed_verify(GMIME_MULTIPART_SIGNED
 					   (body->mime_part), ctx, &error);
-    libbalsa_mime_stream_shared_unlock(stream);
-    g_object_unref(stream);
+    libbalsa_mailbox_unlock_store(body->message->mailbox);
 
     if (valid == NULL) {
 	if (error) {
@@ -646,6 +644,7 @@ libbalsa_body_decrypt(LibBalsaMessageBody * body,
 			  _("Enter passphrase to decrypt message"));
     }
 
+    libbalsa_mailbox_lock_store(body->message->mailbox);
     if (protocol == GPGME_PROTOCOL_OpenPGP)
 	mime_obj =
 	    g_mime_multipart_encrypted_decrypt(GMIME_MULTIPART_ENCRYPTED(body->mime_part),
@@ -668,6 +667,7 @@ libbalsa_body_decrypt(LibBalsaMessageBody * body,
 					       GMIME_CIPHER_CONTEXT(ctx),
 					       &error);
 #endif
+    libbalsa_mailbox_unlock_store(body->message->mailbox);
 
     /* check the result */
     if (mime_obj == NULL) {
@@ -1496,7 +1496,7 @@ select_key_from_list(const gchar * name, gboolean is_secret,
 
 	renderer = gtk_cell_renderer_text_new();
 	column =
-	    gtk_tree_view_column_new_with_attributes(col_titles[i],
+	    gtk_tree_view_column_new_with_attributes(_(col_titles[i]),
 						     renderer, "text", i,
 						     NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);
@@ -1671,9 +1671,10 @@ get_passphrase_cb(void *opaque, const char *uid_hint,
 {
     GMimeGpgmeContext *context;
     gchar *passwd = NULL;
+    int foo;
 
     if (!opaque || !GMIME_IS_GPGME_CONTEXT(opaque)) {
-	write(fd, "\n", 1);
+	foo = write(fd, "\n", 1);
 	return GPG_ERR_USER_1;
     }
     context = GMIME_GPGME_CONTEXT(opaque);
@@ -1684,8 +1685,8 @@ get_passphrase_cb(void *opaque, const char *uid_hint,
 
     /* check if we have the passphrase already cached... */
     if ((passwd = check_cache(pcache, uid_hint, prev_was_bad))) {
-	write(fd, passwd, strlen(passwd));
-	write(fd, "\n", 1);
+	foo = write(fd, passwd, strlen(passwd));
+	foo = write(fd, "\n", 1);
 	wipe_string(passwd);
 	g_free(passwd);
 	return GPG_ERR_NO_ERROR;
@@ -1723,15 +1724,15 @@ get_passphrase_cb(void *opaque, const char *uid_hint,
 #endif				/* BALSA_USE_THREADS */
 
     if (!passwd) {
-	write(fd, "\n", 1);
+	foo = write(fd, "\n", 1);
 	return GPG_ERR_CANCELED;
     }
 
     /* send the passphrase and erase the string */
-    write(fd, passwd, strlen(passwd));
+    foo = write(fd, passwd, strlen(passwd));
     wipe_string(passwd);
     g_free(passwd);
-    write(fd, "\n", 1);
+    foo = write(fd, "\n", 1);
     return GPG_ERR_NO_ERROR;
 }
 
