@@ -28,6 +28,7 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <time.h>
 
 #ifdef BALSA_USE_THREADS
 #include <pthread.h>
@@ -52,8 +53,6 @@
 
 #include <sys/utsname.h>
 
-#include "mime-stream-shared.h"
-
 #include "i18n.h"
 
 typedef struct _MessageQueueItem MessageQueueItem;
@@ -384,6 +383,7 @@ static void dump_queue(const char*msg)
 /* libbalsa_message_queue:
    places given message in the outbox.
 */
+static void libbalsa_set_message_id(GMimeMessage * mime_message);
 LibBalsaMsgCreateResult
 libbalsa_message_queue(LibBalsaMessage * message, LibBalsaMailbox * outbox,
 		       LibBalsaMailbox * fccbox,
@@ -393,6 +393,11 @@ libbalsa_message_queue(LibBalsaMessage * message, LibBalsaMailbox * outbox,
 		       gboolean flow)
 {
     LibBalsaMsgCreateResult result;
+#if ENABLE_ESMTP
+    guint big_message;
+#endif /* ESMTP */
+    gboolean rc;
+    GError *err = NULL;
 
     g_return_val_if_fail(message, LIBBALSA_MESSAGE_CREATE_ERROR);
 
@@ -406,10 +411,51 @@ libbalsa_message_queue(LibBalsaMessage * message, LibBalsaMailbox * outbox,
 #if ENABLE_ESMTP
     g_mime_message_set_header(message->mime_msg, "X-Balsa-SmtpServer",
 	                      libbalsa_smtp_server_get_name(smtp_server));
-#endif /* ESMTP */
-    if(libbalsa_mailbox_copy_message( message, outbox, NULL) < 0)
-        return LIBBALSA_MESSAGE_QUEUE_ERROR;
-    return LIBBALSA_MESSAGE_CREATE_OK;
+
+    big_message = libbalsa_smtp_server_get_big_message(smtp_server);
+    if (big_message > 0) {
+        GMimeMessage *mime_msg;
+        GMimeMessage **mime_msgs;
+        size_t nparts;
+        guint i;
+
+        mime_msg = message->mime_msg;
+        mime_msgs =
+            g_mime_message_partial_split_message(mime_msg, big_message,
+                                                 &nparts);
+        rc = TRUE;
+        for (i = 0; i < nparts; ++i) {
+            if (nparts > 1) {
+                /* RFC 2046, 5.2.2: "...it is specified that entities of
+                 * type "message/partial" must always have a content-
+                 * transfer-encoding of 7bit (the default)" */
+                g_mime_part_set_encoding(GMIME_PART
+                                         (mime_msgs[i]->mime_part),
+                                         GMIME_PART_ENCODING_7BIT);
+                libbalsa_set_message_id(mime_msgs[i]);
+            }
+            if (rc) {
+                message->mime_msg = mime_msgs[i];
+                rc = libbalsa_message_copy(message, outbox, &err);
+            }
+            g_object_unref(mime_msgs[i]);
+        }
+        g_free(mime_msgs);
+        message->mime_msg = mime_msg;
+    } else
+        rc = libbalsa_message_copy(message, outbox, &err);
+#else                           /* ESMTP */
+    rc = libbalsa_message_copy(message, outbox, &err);
+#endif                          /* ESMTP */
+
+    if (!rc) {
+        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                             _("Copying message to outbox failed: %s"),
+                             err ? err->message : "?");
+        g_clear_error(&err);
+    }
+
+    return rc ?  LIBBALSA_MESSAGE_CREATE_OK : LIBBALSA_MESSAGE_QUEUE_ERROR;
 }
 
 static void
@@ -580,7 +626,6 @@ lbs_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
 
     for (msgno = libbalsa_mailbox_total_messages(outbox);
 	 msgno > 0; msgno--) {
-	GList *smtp_server_list;
 	const gchar *smtp_server_name;
         LibBalsaMsgCreateResult created;
 
@@ -593,11 +638,10 @@ lbs_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
 	}
 
         libbalsa_message_body_ref(msg, TRUE, TRUE);
-        smtp_server_list =
-            libbalsa_message_find_user_hdr(msg, "X-Balsa-SmtpServer");
-        smtp_server_name = smtp_server_list ?
-            ((gchar **) smtp_server_list->data)[1] :
-            libbalsa_smtp_server_get_name(NULL);
+        smtp_server_name =
+            libbalsa_message_get_user_header(msg, "X-Balsa-SmtpServer");
+        if (!smtp_server_name)
+            smtp_server_name = libbalsa_smtp_server_get_name(NULL);
         if (strcmp(smtp_server_name,
                    libbalsa_smtp_server_get_name(smtp_server)) != 0) {
             libbalsa_message_body_unref(msg);
@@ -832,7 +876,7 @@ disp_recipient_status(smtp_recipient_t recipient,
 {
   const smtp_status_t *status = smtp_recipient_status (recipient);
 
-  if(status->code != 0) {
+  if(status->code != 0 && status->code != 250) {
       libbalsa_information(
                            LIBBALSA_INFORMATION_WARNING, 
                            _("Could not send the message to %s:\n"
@@ -866,24 +910,23 @@ handle_successful_send(smtp_message_t message, void *be_verbose)
 	if (mqi != NULL && mqi->orig != NULL && mqi->refcount <= 0 &&
             mqi->orig->mailbox) {
             gboolean remove = TRUE;
-            GList* fcclist =
-                libbalsa_message_find_user_hdr(mqi->orig, "X-Balsa-Fcc");
-            const gchar **fccurl = fcclist ? fcclist->data : NULL;
+            const gchar *fccurl =
+                libbalsa_message_get_user_header(mqi->orig, "X-Balsa-Fcc");
 
 	    if (mqi->orig->mailbox && fccurl) {
-                LibBalsaMailbox *fccbox = mqi->finder(fccurl[1]);
+                LibBalsaMailbox *fccbox = mqi->finder(fccurl);
                 GError *err = NULL;
                 lbs_change_flags(mqi->orig, 0, LIBBALSA_MESSAGE_FLAG_NEW |
                                  LIBBALSA_MESSAGE_FLAG_FLAGGED);
 		libbalsa_mailbox_sync_storage(mqi->orig->mailbox, FALSE);
-                remove =
-                    libbalsa_mailbox_copy_message(mqi->orig, fccbox, &err)>=0;
-                if(!remove) 
+                remove = libbalsa_message_copy(mqi->orig, fccbox, &err);
+                if(!remove) {
                     libbalsa_information
                         (LIBBALSA_INFORMATION_ERROR, 
                          _("Saving sent message to %s failed: %s"),
                          fccbox->url, err ? err->message : "?");
-                g_clear_error(&err);
+                    g_clear_error(&err);
+                }
             }
             /* If copy failed, mark the message again as flagged -
                otherwise it will get resent again. And again, and
@@ -896,10 +939,10 @@ handle_successful_send(smtp_message_t message, void *be_verbose)
 	/* XXX - Show the poor user the status codes and message. */
         if(*(gboolean*)be_verbose) {
             int cnt = 0;
-            status = smtp_reverse_path_status(message);
-            if(status->code != 250) {
+            /* status = smtp_reverse_path_status(message); */
+            if(status->code != 250 && status->code != 0) {
                 libbalsa_information(
-                                     LIBBALSA_INFORMATION_ERROR, 
+                                     LIBBALSA_INFORMATION_WARNING,
                                      _("Relaying refused:\n"
                                        "%d: %s\n"
                                        "Message left in your outbox.\n"), 
@@ -947,10 +990,13 @@ libbalsa_smtp_event_cb (smtp_session_t session, int event_no, void *arg, ...)
         mailbox = va_arg (ap, const char *);
         message = va_arg (ap, smtp_message_t);
 	status = smtp_reverse_path_status (message);
-	snprintf (buf, sizeof buf, "%s %d <%s>", _("From:"), status->code, mailbox);
+
+        /* status code, mailbox */
+        snprintf (buf, sizeof buf, _("From: %d <%s>"), status->code, mailbox);
 	MSGSENDTHREAD(threadmsg, MSGSENDTHREADPROGRESS, buf, NULL, NULL, 0);
 
-	snprintf (buf, sizeof buf, "%s %s: %d %s", _("From"),
+        /* mailbox, status code, status text */
+        snprintf (buf, sizeof buf, _("From %s: %d %s"), 
 	          mailbox, status->code, status->text);
         g_strchomp(buf);
 	libbalsa_information(LIBBALSA_INFORMATION_MESSAGE, buf);
@@ -959,10 +1005,13 @@ libbalsa_smtp_event_cb (smtp_session_t session, int event_no, void *arg, ...)
         mailbox = va_arg (ap, const char *);
         recipient = va_arg (ap, smtp_recipient_t);
 	status = smtp_recipient_status (recipient);
-	snprintf (buf, sizeof buf, "%s %d <%s>", _("To:"), status->code, mailbox);
+
+        /* status code, mailbox */
+        snprintf (buf, sizeof buf, _("To: %d <%s>"),  status->code, mailbox);
 	MSGSENDTHREAD(threadmsg, MSGSENDTHREADPROGRESS, buf, NULL, NULL, 0);
 
-	snprintf (buf, sizeof buf, "%s %s: %d %s", _("To"),
+        /* mailbox, status code, status text */
+        snprintf (buf, sizeof buf, _("To %s: %d %s"),
 	          mailbox, status->code, status->text);
         g_strchomp(buf);
 	libbalsa_information(LIBBALSA_INFORMATION_MESSAGE, buf);
@@ -1020,7 +1069,6 @@ libbalsa_smtp_event_cb (smtp_session_t session, int event_no, void *arg, ...)
 	    *ok = libbalsa_is_cert_known(cert, vfy_result);
 	    X509_free(cert);
 	}
-	printf("vfy_result: %ld ok=%d\n", vfy_result, *ok);
         break;
     }
     case SMTP_EV_NO_PEER_CERTIFICATE:
@@ -1073,7 +1121,6 @@ libbalsa_smtp_event_cb_serial(smtp_session_t session, int event_no,
 #endif
     {
 	int *ok;
-	printf("SMTP-TLS event_no=%d\n", event_no);
 	ok = va_arg(ap, int*);
 	*ok = 1;
 	break;
@@ -1176,18 +1223,17 @@ handle_successful_send(MessageQueueItem *mqi, LibBalsaFccboxFinder finder)
 {
     if (mqi->orig->mailbox) {
         gboolean remove = TRUE;
-        GList* fcclist =
-            libbalsa_message_find_user_hdr(mqi->orig, "X-Balsa-Fcc");
-        const gchar **fccurl = fcclist ? fcclist->data : NULL;
+        const gchar *fccurl =
+            libbalsa_message_get_user_header(mqi->orig, "X-Balsa-Fcc");
 
         lbs_change_flags(mqi->orig, 0, LIBBALSA_MESSAGE_FLAG_NEW |
                          LIBBALSA_MESSAGE_FLAG_FLAGGED);
 	libbalsa_mailbox_sync_storage(mqi->orig->mailbox, FALSE);
 
         if (mqi->orig->mailbox && fccurl) {
-            LibBalsaMailbox *fccbox = mqi->finder(fccurl[1]);
+            LibBalsaMailbox *fccbox = mqi->finder(fccurl);
             remove =
-                libbalsa_mailbox_copy_message(mqi->orig, fccbox, NULL)>=0;
+                libbalsa_message_copy(mqi->orig, fccbox, NULL)>=0;
         }
         /* If copy failed, mark the message again as flagged -
            otherwise it will get resent again. And again, and
@@ -1225,12 +1271,14 @@ monitor_cb (const char *buf, int buflen, int writing, void *arg)
   if (writing == SMTP_CB_HEADERS)
     {
       fputs ("H: ", fp);
-      fwrite (buf, 1, buflen, fp);
+      if (fwrite (buf, 1, buflen, fp) != (size_t) buflen)
+        /* FIXME */ return;
       return;
     }
 
  fputs (writing ? "C: " : "S: ", fp);
- fwrite (buf, 1, buflen, fp);
+ if (fwrite (buf, 1, buflen, fp) != (size_t) buflen)
+   /* FIXME */ return;
  if (buf[buflen - 1] != '\n')
    putc ('\n', fp);
 }
@@ -1488,6 +1536,34 @@ parse_content_type(const char* content_type)
     return ret;
 }
 
+/* get_tz_offset() returns tz offset in minutes. NOTE: not all hours
+   have 60 seconds! Once in a while they get corrected.  */
+#define MIN_SEC		60		/* seconds in a minute */
+#define	HOUR_MIN	60		/* minutes in an hour */
+#define DAY_MIN		(24 * HOUR_MIN)	/* minutes in a day */
+
+static int
+get_tz_offset(time_t *t)
+{
+    struct tm gmt, lt;
+    int off;
+    gmtime_r(t, &gmt);
+    localtime_r(t, &lt);
+
+    off = (lt.tm_hour - gmt.tm_hour) * HOUR_MIN + lt.tm_min - gmt.tm_min;
+    if (lt.tm_year < gmt.tm_year)       off -= DAY_MIN;
+    else if (lt.tm_year > gmt.tm_year)	off += DAY_MIN;
+    else if (lt.tm_yday < gmt.tm_yday)  off -= DAY_MIN;
+    else if (lt.tm_yday > gmt.tm_yday)  off += DAY_MIN;
+
+    /* special case: funny minutes */
+    if (lt.tm_sec <= gmt.tm_sec - MIN_SEC) off -= 1;
+    else if (lt.tm_sec >= gmt.tm_sec + MIN_SEC)	off += 1;
+
+    return (off*100)/60; /* time zone offset in hundreds of hours (funny
+                          * unit required by gmime) */
+}
+
 static LibBalsaMsgCreateResult
 libbalsa_message_create_mime_message(LibBalsaMessage* message, gboolean flow,
 				     gboolean postponing)
@@ -1694,7 +1770,8 @@ libbalsa_message_create_mime_message(LibBalsaMessage* message, gboolean flow,
 	g_mime_message_set_subject(mime_message,
 				   LIBBALSA_MESSAGE_GET_SUBJECT(message));
 
-    g_mime_message_set_date(mime_message, message->headers->date, 0); /* FIXME: Set GMT offset */
+    g_mime_message_set_date(mime_message, message->headers->date,
+                            get_tz_offset(&message->headers->date));
 
     tmp = internet_address_list_to_string(message->headers->to_list, TRUE);
     g_mime_message_add_recipients_from_string(mime_message,
@@ -1721,13 +1798,13 @@ libbalsa_message_create_mime_message(LibBalsaMessage* message, gboolean flow,
 	}
     }
 
-    for (list = message->headers->user_hdrs; list; list = g_list_next(list)) {
-	gchar **pair;
-	pair = g_strsplit(list->data, ":", 1);
+    for (list = message->headers->user_hdrs; list; list = list->next) {
+	gchar **pair = list->data;
 	g_strchug(pair[1]);
 	g_mime_message_add_header(mime_message, pair[0], pair[1]);
+#if DEBUG_USER_HEADERS
         printf("adding header '%s:%s'\n", pair[0], pair[1]);
-	g_strfreev(pair);
+#endif
     }
 
     tmp = g_strdup_printf("Balsa %s", VERSION);
@@ -1747,24 +1824,32 @@ libbalsa_message_create_mime_message(LibBalsaMessage* message, gboolean flow,
  */
 gboolean
 libbalsa_message_postpone(LibBalsaMessage * message,
-			  LibBalsaMailbox * draftbox,
-			  LibBalsaMessage * reply_message,
-			  gchar * fcc, gboolean flow)
+                          LibBalsaMailbox * draftbox,
+                          LibBalsaMessage * reply_message,
+                          gchar * fcc, gboolean flow)
 {
-    int thereturn; 
+    gboolean retval;
+    GError *err = NULL;
 
     if (!message->mime_msg
-	&& libbalsa_message_create_mime_message(message, flow,
-						TRUE) !=
-	LIBBALSA_MESSAGE_CREATE_OK)
-	return FALSE;
+        && libbalsa_message_create_mime_message(message, flow,
+                                                TRUE) !=
+        LIBBALSA_MESSAGE_CREATE_OK)
+        return FALSE;
 
     if (fcc)
-	g_mime_message_set_header(message->mime_msg, "X-Balsa-Fcc", fcc);
+        g_mime_message_set_header(message->mime_msg, "X-Balsa-Fcc", fcc);
 
-    thereturn = libbalsa_mailbox_copy_message( message, draftbox, NULL );
+    retval = libbalsa_message_copy(message, draftbox, &err);
 
-    return thereturn!=-1;
+    if (!retval) {
+        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                             _("Postponing message failed: %s"),
+                             err ? err->message : "?");
+        g_clear_error(&err);
+    }
+
+    return retval;
 }
 
 
@@ -1827,8 +1912,6 @@ libbalsa_fill_msg_queue_item_from_queu(LibBalsaMessage * message,
     mqi->orig = message;
     g_object_ref(mqi->orig);
     if (message->mime_msg) {
-        GMimeStream *tmp;
-
         g_mime_object_remove_header(GMIME_OBJECT(message->mime_msg),
                                     "Status");
         g_mime_object_remove_header(GMIME_OBJECT(message->mime_msg),
@@ -1838,12 +1921,10 @@ libbalsa_fill_msg_queue_item_from_queu(LibBalsaMessage * message,
         g_mime_object_remove_header(GMIME_OBJECT(message->mime_msg),
                                     "X-Balsa-SmtpServer");
 	mqi->stream = g_mime_stream_mem_new();
-        tmp = libbalsa_message_stream(message);
-        libbalsa_mime_stream_shared_lock(tmp);
+        libbalsa_mailbox_lock_store(message->mailbox);
 	g_mime_object_write_to_stream(GMIME_OBJECT(message->mime_msg),
                                       mqi->stream);
-        libbalsa_mime_stream_shared_unlock(tmp);
-        g_object_unref(tmp);
+        libbalsa_mailbox_unlock_store(message->mailbox);
 	g_mime_stream_reset(mqi->stream);
     } else
 	mqi->stream = libbalsa_message_stream(message);
