@@ -20,11 +20,24 @@
  */
 
 #include "config.h"
-#include <gnome.h>
-#include "helper.h"
+
+#include <gtk/gtk.h>
+
 #include "init_balsa.h"
+#ifdef HAVE_GNOME
+#include "helper.h"
 #include "balsa-initdruid.h"
 #include "balsa-druid-page-welcome.h"
+#endif /* HAVE_GNOME */
+
+#include "i18n.h"	/* Must come after helper.h. */
+
+static gboolean
+dismiss_the_wizard(GtkWidget *wizard)
+{
+    gtk_widget_destroy(wizard);
+    return FALSE;
+}
 
 void
 balsa_init_begin(void)
@@ -33,14 +46,18 @@ balsa_init_begin(void)
 
     window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW(window), _("Configure Balsa"));
+    gtk_window_set_wmclass(GTK_WINDOW(window), "druid", "Balsa");
 
+#ifdef HAVE_GNOME
     balsa_initdruid(GTK_WINDOW(window));
-
     gtk_widget_show_all(window);
 
     gdk_threads_enter();
     gtk_main();
     gdk_threads_leave();
+#endif
 
-    gtk_widget_destroy(window);
+    /* we do not want to destroy wizard immediately to avoid confusing
+       delay between the wizard that left and balsa that entered. */
+    g_idle_add((GSourceFunc)dismiss_the_wizard, window);
 }
