@@ -29,7 +29,6 @@
 
 #include "balsa-app.h"
 #include "address-book-config.h"
-#include "address-book-gpe.h"
 #include "i18n.h"
 
 typedef struct _AddressBookConfig AddressBookConfig;
@@ -52,6 +51,7 @@ struct _AddressBookConfig {
 	    GtkWidget *host_name;
 	    GtkWidget *base_dn;
 	    GtkWidget *bind_dn;
+	    GtkWidget *book_dn;
 	    GtkWidget *passwd;
 	    GtkWidget *enable_tls;
 	} ldap;
@@ -111,6 +111,22 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book,
     gtk_widget_show_all(GTK_WIDGET(abc->window));
 }
 
+void
+balsa_address_book_config_new_from_type(GType type,
+                                        BalsaAddressBookCallback callback,
+                                        GtkWindow* parent)
+{
+    AddressBookConfig *abc;
+
+    abc = g_new0(AddressBookConfig, 1);
+    abc->address_book = NULL;
+    abc->callback = callback;
+    abc->type = type;
+    abc->window = create_dialog_from_type(abc);
+
+    gtk_widget_show_all(GTK_WIDGET(abc->window));
+}
+
 static void
 edit_book_response(GtkWidget * dialog, gint response,
                    AddressBookConfig * abc)
@@ -132,6 +148,77 @@ edit_book_response(GtkWidget * dialog, gint response,
     g_free(abc);
 }
 
+/* Duplicated from balsa-app.c, for use in balsa-ab. */
+/* abc_create_label:
+   Create a label and add it to a table in the first column of given row,
+   setting the keyval to found accelerator value, that can be later used 
+   in abc_create_entry.
+*/
+static GtkWidget *
+abc_create_label(const gchar * label, GtkWidget * table, gint row)
+{
+    GtkWidget *w = gtk_label_new_with_mnemonic(label);
+
+    gtk_misc_set_alignment(GTK_MISC(w), 1.0, 0.5);
+    gtk_table_attach(GTK_TABLE(table), w, 0, 1, row, row + 1,
+		     GTK_FILL, GTK_FILL, 5, 5);
+    gtk_widget_show(w);
+    return w;
+}
+
+/* abc_create_check:
+   creates a checkbox with a given label and places them in given array.
+*/
+static GtkWidget *
+abc_create_check(GtkDialog *mcw, const gchar *label, GtkWidget *table, gint row,
+             gboolean initval)
+{
+    GtkWidget *cb, *l;
+    
+    cb = gtk_check_button_new();
+
+    l = gtk_label_new_with_mnemonic(label);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(l), cb);
+    gtk_misc_set_alignment(GTK_MISC(l), 0.0, 0.5);
+    gtk_widget_show(l);
+
+    gtk_container_add(GTK_CONTAINER(cb), l);
+
+    gtk_table_attach(GTK_TABLE(table), cb, 1, 2, row, row+1,
+		     GTK_FILL, GTK_FILL, 5, 5);
+
+    if(initval) 	
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cb), TRUE);
+
+    return cb;
+}
+
+/* Create a text entry and add it to the table */
+static GtkWidget *
+abc_create_entry(GtkDialog *mcw, GtkWidget * table, 
+	     GtkSignalFunc changed_func, gpointer data, gint row, 
+	     const gchar * initval, GtkWidget* hotlabel)
+{
+    GtkWidget *entry = gtk_entry_new();
+    gtk_table_attach(GTK_TABLE(table), entry, 1, 2, row, row + 1,
+		     GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 5);
+    if (initval) {
+        gint zero = 0;
+        
+        gtk_editable_insert_text(GTK_EDITABLE(entry), initval, -1, &zero);
+    }
+
+    gtk_label_set_mnemonic_widget(GTK_LABEL(hotlabel), entry);
+
+    /* Watch for changes... */
+    if(changed_func)
+	g_signal_connect(G_OBJECT(entry), "changed", 
+			 G_CALLBACK(changed_func), data);
+
+    gtk_widget_show(entry);
+    return entry;
+}
+
 static GtkWidget *
 create_local_dialog(AddressBookConfig * abc)
 {
@@ -164,21 +251,26 @@ create_local_dialog(AddressBookConfig * abc)
 
     table = gtk_table_new(2, 2, FALSE);
     gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(dialog), table);
-    label = create_label(_("A_ddress Book Name"), table, 0);
+    label = abc_create_label(_("A_ddress Book Name"), table, 0);
     abc->name_entry =
-        create_entry(NULL, table, NULL, NULL, 0, name, label);
+        abc_create_entry(NULL, table, NULL, NULL, 0, name, label);
     abc->expand_aliases_button =
-        create_check(NULL, _("_Expand aliases as you type"), table, 1,
+        abc_create_check(NULL, _("_Expand aliases as you type"), table, 1,
                      ab ? ab->expand_aliases : TRUE);
 
     if (ab) {
-        const gchar *path;
+        const gchar *path = LIBBALSA_ADDRESS_BOOK_TEXT(ab)->path;
+        gchar *folder;
+        gchar *utf8name;
 
-        if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_VCARD)
-            path = LIBBALSA_ADDRESS_BOOK_VCARD(ab)->path;
-        else
-            path = LIBBALSA_ADDRESS_BOOK_LDIF(ab)->path;
         gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), path);
+        /* Name entry will be blank unless we set it. */
+        folder = g_path_get_basename(path);
+        utf8name = g_filename_to_utf8(folder, -1, NULL, NULL, NULL);
+        g_free(folder);
+        gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog),
+                                          utf8name);
+        g_free(utf8name);
     }
     g_signal_connect(G_OBJECT(dialog), "response",
                      G_CALLBACK(edit_book_response), abc);
@@ -268,8 +360,8 @@ create_externq_dialog(AddressBookConfig * abc)
 
     /* mailbox name */
 
-    label = create_label(_("A_ddress Book Name"), table, 0);
-    abc->name_entry = create_entry(mcw, table, NULL, NULL, 0, 
+    label = abc_create_label(_("A_ddress Book Name"), table, 0);
+    abc->name_entry = abc_create_entry(mcw, table, NULL, NULL, 0, 
 				   ab ? abc->address_book->name : NULL, 
 				   label);
 
@@ -312,7 +404,7 @@ create_externq_dialog(AddressBookConfig * abc)
                                   abc->ab_specific.externq.save);
     
     abc->expand_aliases_button =
-	create_check(mcw, _("_Expand aliases as you type"), table, 3,
+	abc_create_check(mcw, _("_Expand aliases as you type"), table, 3,
 		     ab ? abc->address_book->expand_aliases : TRUE);
 
     if (ab) {
@@ -345,7 +437,7 @@ static GtkWidget *
 create_ldap_dialog(AddressBookConfig * abc)
 {
     GtkWidget *dialog;
-    GtkWidget *table = gtk_table_new(2, 6, FALSE);
+    GtkWidget *table = gtk_table_new(2, 7, FALSE);
 
     LibBalsaAddressBookLdap* ab;
     GtkDialog* mcw = GTK_DIALOG(abc->window);
@@ -358,38 +450,43 @@ create_ldap_dialog(AddressBookConfig * abc)
 
     /* mailbox name */
 
-    label = create_label(_("A_ddress Book Name"), table, 0);
-    abc->name_entry = create_entry(mcw, table, NULL, NULL, 0, 
+    label = abc_create_label(_("A_ddress Book Name"), table, 0);
+    abc->name_entry = abc_create_entry(mcw, table, NULL, NULL, 0, 
 				   ab ? abc->address_book->name : name, 
 				   label);
 
-    label = create_label(_("_Host Name"), table, 1);
+    label = abc_create_label(_("_Host Name"), table, 1);
     abc->ab_specific.ldap.host_name = 
-	create_entry(mcw, table, NULL, NULL, 1, 
+	abc_create_entry(mcw, table, NULL, NULL, 1, 
 		     ab ? ab->host : host, label);
 
-    label = create_label(_("Base Domain _Name"), table, 2);
+    label = abc_create_label(_("Base Domain _Name"), table, 2);
     abc->ab_specific.ldap.base_dn = 
-	create_entry(mcw, table, NULL, NULL, 2, 
+	abc_create_entry(mcw, table, NULL, NULL, 2, 
 		     ab ? ab->base_dn : base, label);
 
-    label = create_label(_("_User Name (Bind DN)"), table, 3);
+    label = abc_create_label(_("_User Name (Bind DN)"), table, 3);
     abc->ab_specific.ldap.bind_dn = 
-	create_entry(mcw, table, NULL, NULL, 3, 
+	abc_create_entry(mcw, table, NULL, NULL, 3, 
 		     ab ? ab->bind_dn : "", label);
 
-    label = create_label(_("_Password"), table, 4);
+    label = abc_create_label(_("_Password"), table, 4);
     abc->ab_specific.ldap.passwd = 
-	create_entry(mcw, table, NULL, NULL, 4, 
+	abc_create_entry(mcw, table, NULL, NULL, 4, 
 		     ab ? ab->passwd : "", label);
     gtk_entry_set_visibility(GTK_ENTRY(abc->ab_specific.ldap.passwd), FALSE);
 
+    label = abc_create_label(_("_User Address Book DN"), table, 5);
+    abc->ab_specific.ldap.book_dn = 
+	abc_create_entry(mcw, table, NULL, NULL, 5,
+		     ab ? ab->priv_book_dn : "", label);
+
     abc->ab_specific.ldap.enable_tls =
-	create_check(mcw, _("Enable _TLS"), table, 5,
+	abc_create_check(mcw, _("Enable _TLS"), table, 6,
 		     ab ? ab->enable_tls : FALSE);
 
     abc->expand_aliases_button =
-	create_check(mcw, _("_Expand aliases as you type"), table, 6,
+	abc_create_check(mcw, _("_Expand aliases as you type"), table, 7,
 		     ab ? abc->address_book->expand_aliases : TRUE);
 
     gtk_widget_show(table);
@@ -419,12 +516,12 @@ create_gpe_dialog(AddressBookConfig * abc)
 
     /* mailbox name */
 
-    label = create_label(_("A_ddress Book Name"), table, 0);
-    abc->name_entry = create_entry(mcw, table, NULL, NULL, 0, 
+    label = abc_create_label(_("A_ddress Book Name"), table, 0);
+    abc->name_entry = abc_create_entry(mcw, table, NULL, NULL, 0, 
 				   ab ? ab->name : _("GPE Address Book"), 
 				   label);
     abc->expand_aliases_button =
-	create_check(mcw, _("_Expand aliases as you type"), table, 3,
+	abc_create_check(mcw, _("_Expand aliases as you type"), table, 3,
 		     ab ? ab->expand_aliases : TRUE);
 
     dialog = create_generic_dialog(abc);
@@ -441,9 +538,9 @@ help_button_cb(AddressBookConfig * abc)
     gnome_help_display("balsa", "preferences-1", &err);
 
     if (err) {
-	balsa_information(LIBBALSA_INFORMATION_WARNING,
-		_("Error displaying help: %s\n"),
-		err->message);
+        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                             _("Error displaying help: %s\n"),
+                             err->message);
         g_error_free(err);
     }
 }
@@ -617,12 +714,15 @@ create_book(AddressBookConfig * abc)
             gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.bind_dn));
         const gchar *passwd =
             gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.passwd));
+        const gchar *book_dn =
+            gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.book_dn));
         gboolean enable_tls =
             gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
                                          (abc->ab_specific.ldap.enable_tls));
         address_book =
             libbalsa_address_book_ldap_new(name, host_name, base_dn,
-                                           bind_dn, passwd, enable_tls);
+                                           bind_dn, passwd, book_dn,
+                                           enable_tls);
 #endif
 #ifdef HAVE_SQLITE
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_GPE) {
@@ -649,15 +749,16 @@ modify_book(AddressBookConfig * abc)
     address_book->name =
         g_strdup(gtk_entry_get_text(GTK_ENTRY(abc->name_entry)));
 
-    if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_VCARD) {
-        LibBalsaAddressBookVcard *vcard;
+    if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_VCARD
+        || abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF) {
+        LibBalsaAddressBookText *ab_text =
+            LIBBALSA_ADDRESS_BOOK_TEXT(address_book);
         gchar *path =
             gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(abc->window));
 
-        vcard = LIBBALSA_ADDRESS_BOOK_VCARD(address_book);
         if (path) {
-            g_free(vcard->path);
-            vcard->path = path;
+            g_free(ab_text->path);
+            ab_text->path = path;
         }
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN) {
         LibBalsaAddressBookExtern *externq;
@@ -688,16 +789,6 @@ modify_book(AddressBookConfig * abc)
             g_free(externq->save);
             externq->save = save;
         }
-    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF) {
-        LibBalsaAddressBookLdif *ldif;
-        gchar *path =
-            gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(abc->window));
-
-        ldif = LIBBALSA_ADDRESS_BOOK_LDIF(address_book);
-        if (path) {
-            g_free(ldif->path);
-            ldif->path = path;
-        }
 #ifdef ENABLE_LDAP
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDAP) {
         LibBalsaAddressBookLdap *ldap;
@@ -709,6 +800,8 @@ modify_book(AddressBookConfig * abc)
             gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.bind_dn));
         const gchar *passwd =
             gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.passwd));
+        const gchar *book_dn =
+            gtk_entry_get_text(GTK_ENTRY(abc->ab_specific.ldap.book_dn));
         gboolean enable_tls =
             gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
                                          (abc->ab_specific.ldap.enable_tls));
@@ -719,9 +812,14 @@ modify_book(AddressBookConfig * abc)
         g_free(ldap->base_dn);  ldap->base_dn = g_strdup(base_dn);
         g_free(ldap->bind_dn);  ldap->bind_dn = g_strdup(bind_dn);
         g_free(ldap->passwd);   ldap->passwd  = g_strdup(passwd);
+        g_free(ldap->priv_book_dn);
+        ldap->priv_book_dn = g_strdup(book_dn && *book_dn ? book_dn : bind_dn);
         ldap->enable_tls = enable_tls;
         libbalsa_address_book_ldap_close_connection(ldap);
 #endif
+#if HAVE_SQLITE
+    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_GPE) {
+#endif /* HAVE_SQLITE */
     } else
         g_assert_not_reached();
 
@@ -739,7 +837,7 @@ add_vcard_cb(GtkWidget * widget, AddressBookConfig * abc)
     g_object_weak_unref(G_OBJECT(gtk_widget_get_parent(widget)),
                         (GWeakNotify) g_free, abc);
     abc->type = LIBBALSA_TYPE_ADDRESS_BOOK_VCARD;
-    abc->window = create_local_dialog(abc);
+    abc->window = create_vcard_dialog(abc);
     gtk_widget_show_all(abc->window);
 }
 
@@ -759,7 +857,7 @@ add_ldif_cb(GtkWidget * widget, AddressBookConfig * abc)
     g_object_weak_unref(G_OBJECT(gtk_widget_get_parent(widget)),
                         (GWeakNotify) g_free, abc);
     abc->type = LIBBALSA_TYPE_ADDRESS_BOOK_LDIF;
-    abc->window = create_local_dialog(abc);
+    abc->window = create_ldif_dialog(abc);
     gtk_widget_show_all(abc->window);
 }
 
