@@ -54,13 +54,6 @@ const gchar *pspell_suggest_modes[] = {
     "bad-spellers"
 };
 
-static void
-rememb_toggle_cb(GObject* button, LibBalsaServer *s)
-{
-    s->remember_passwd = 
-        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
-}
-
 /* ask_password:
    asks the user for the password to the mailbox on given remote server.
 */
@@ -99,16 +92,21 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
     gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), rememb);
     if(server->remember_passwd)
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rememb), TRUE);
-    g_signal_connect(G_OBJECT(rememb), "toggled", 
-                     G_CALLBACK(rememb_toggle_cb), server);
 
     gtk_widget_show_all(GTK_WIDGET(GTK_DIALOG(dialog)->vbox));
     gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
     gtk_widget_grab_focus (entry);
 
-    if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
+    if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
+        unsigned old_rem = server->remember_passwd;
         passwd = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
+        server->remember_passwd = 
+            !!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rememb));
+        libbalsa_server_set_password(server, passwd);
+        if(old_rem != server->remember_passwd)
+            libbalsa_server_config_changed(server);
+    }
     gtk_widget_destroy(dialog);
     return passwd;
 }
@@ -376,6 +374,9 @@ balsa_app_init(void)
     balsa_app.filters=NULL;
 
     /* spell check */
+#if HAVE_GTKSPELL
+    balsa_app.spell_check = FALSE;
+#else                           /* HAVE_GTKSPELL */
     balsa_app.module = SPELL_CHECK_MODULE_ASPELL;
     balsa_app.suggestion_mode = SPELL_CHECK_SUGGEST_NORMAL;
     balsa_app.ignore_size = 0;
@@ -383,6 +384,7 @@ balsa_app_init(void)
 
     spell_check_modules_name = pspell_modules;
     spell_check_suggest_mode_name = pspell_suggest_modes;
+#endif                          /* HAVE_GTKSPELL */
 
     /* Information messages */
     balsa_app.information_message = 0;
@@ -423,13 +425,18 @@ balsa_app_destroy(void)
     config_save();
 
     g_list_foreach(balsa_app.address_book_list, (GFunc)g_object_unref, NULL);
-    g_slist_foreach(balsa_app.filters,          (GFunc)libbalsa_filter_free, 
+    g_list_free(balsa_app.address_book_list);
+    balsa_app.address_book_list = NULL;
+
+    /* close all mailboxes */
+    gtk_widget_destroy(balsa_app.notebook);
+
+    /* now free filters */
+    g_slist_foreach(balsa_app.filters, (GFunc)libbalsa_filter_free, 
 		    GINT_TO_POINTER(TRUE));
     g_slist_free(balsa_app.filters);
     balsa_app.filters = NULL;
 
-    /* close all mailboxes */
-    gtk_widget_destroy(balsa_app.notebook);
     /* g_slist_free(opt_attach_list); */
     g_object_unref(balsa_app.colormap);
     if(balsa_app.debug) g_print("balsa_app: Finished cleaning up.\n");
