@@ -226,6 +226,12 @@ bndx_destroy(GtkObject * obj)
 
     g_return_if_fail(obj != NULL);
     index = BALSA_INDEX(obj);
+    if(index->selection_changed_id) {
+        GtkTreeView *tree_view = GTK_TREE_VIEW(index);
+        GtkTreeSelection* selection = gtk_tree_view_get_selection(tree_view);
+        g_signal_handler_disconnect(selection, index->selection_changed_id);
+        index->selection_changed_id = 0;
+    }
 
     if (index->mailbox_node) {
 	LibBalsaMailbox* mailbox;
@@ -235,6 +241,11 @@ bndx_destroy(GtkObject * obj)
 						 G_SIGNAL_MATCH_DATA,
 						 0, 0, NULL, NULL, index);
 	    gtk_tree_view_set_model(GTK_TREE_VIEW(index), NULL);
+            /* Undo any temporary filters. */
+            libbalsa_mailbox_set_view_filter(mailbox,
+                                             balsa_window_get_view_filter
+                                             (balsa_app.main_window, TRUE),
+                                             TRUE);
 	    libbalsa_mailbox_close(mailbox, balsa_app.expunge_on_close);
 
 	    libbalsa_mailbox_search_iter_free(index->search_iter);
@@ -261,7 +272,7 @@ bndx_destroy(GtkObject * obj)
 	index->selected = NULL;
     }
 
-    g_free(index->sos_filter); index->sos_filter = NULL;
+    g_free(index->filter_string); index->filter_string = NULL;
 
     if (GTK_OBJECT_CLASS(parent_class)->destroy)
         (*GTK_OBJECT_CLASS(parent_class)->destroy) (obj);
@@ -504,18 +515,10 @@ bndx_deselected_idle(BalsaIndex * index)
                                            LIBBALSA_MESSAGE_FLAG_SELECTED);
 
     if (current_message) {
-	/* We had a message... */
-	if (!index->current_message)
-	    /* ...but we don't any more; restore it. */
-	    index->current_message = current_message;
-	else
-	    /* Discard. */
-	    g_object_unref(current_message);
-    }
-
-    if (index->current_message) {
+        /* Make sure it's selected. */
         GtkTreePath *path;
-        if (bndx_find_message(index, &path, NULL, index->current_message)) {
+
+        if (bndx_find_message(index, &path, NULL, current_message)) {
             GtkTreeSelection *selection =
                 gtk_tree_view_get_selection(GTK_TREE_VIEW(index));
             if (!gtk_tree_selection_path_is_selected(selection, path)) {
@@ -523,11 +526,8 @@ bndx_deselected_idle(BalsaIndex * index)
                 bndx_select_row(index, path);
             }
             gtk_tree_path_free(path);
-        } else {
-	    /* Can this happen? */
-	    g_object_unref(index->current_message);
-	    index->current_message = NULL;
-	}
+        }
+        g_object_unref(current_message);
     }
 
     g_object_set_data(G_OBJECT(index), BALSA_INDEX_DESELECTED_ARRAY, NULL);
@@ -736,6 +736,51 @@ bndx_row_activated(GtkTreeView * tree_view, GtkTreePath * path,
     g_object_unref(message);
 }
 
+/*
+ * Scroll in an idle handler, otherwise it gets ignored.
+ */
+#define BALSA_INDEX_ROW_REF_KEY "balsa-index-row-ref-key"
+static gboolean
+bndx_scroll_idle(BalsaIndex * index)
+{
+    GtkTreeRowReference *row_ref;
+    GtkTreePath *path;
+
+    gdk_threads_enter();
+
+    row_ref = g_object_get_data(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY);
+    if (row_ref && (path = gtk_tree_row_reference_get_path(row_ref))) {
+        gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(index), path, NULL,
+                                     FALSE, 0, 0);
+        gtk_tree_path_free(path);
+    }
+
+    g_object_set_data(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY, NULL);
+    g_object_unref(index);
+
+    gdk_threads_leave();
+
+    return FALSE;
+}
+
+static void
+bndx_scroll_to_row(BalsaIndex * index, GtkTreePath * path)
+{
+    GtkTreeRowReference *row_ref;
+
+    row_ref = g_object_get_data(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY);
+    if (!row_ref) {
+        g_object_ref(index);
+        g_idle_add((GSourceFunc) bndx_scroll_idle, index);
+    }
+
+    row_ref =
+        gtk_tree_row_reference_new(gtk_tree_view_get_model
+                                   (GTK_TREE_VIEW(index)), path);
+    g_object_set_data_full(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY, row_ref,
+                           (GDestroyNotify) gtk_tree_row_reference_free);
+}
+
 /* bndx_tree_expand_cb:
  * callback on expand events
  * set/reset unread style, as appropriate
@@ -766,9 +811,7 @@ bndx_tree_expand_cb(GtkTreeView * tree_view, GtkTreeIter * iter,
 		    && msgno == (guint) index->current_message->msgno) {
 		    gtk_tree_view_set_cursor(tree_view, current_path,
 			                     NULL, FALSE);
-                    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(index),
-                                                 current_path, NULL, FALSE,
-                                                 0, 0);
+                    bndx_scroll_to_row(index, current_path);
                 }
             }
             gtk_tree_path_free(current_path);
@@ -873,8 +916,7 @@ balsa_index_scroll_on_open(BalsaIndex *index)
     if(msgno>0 &&
        libbalsa_mailbox_msgno_find(mailbox, msgno, &path, &iter)) {
         bndx_expand_to_row(index, path);
-	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(index), path, NULL,
-				     FALSE, 0, 0);
+	bndx_scroll_to_row(index, path);
         if(balsa_app.view_message_on_open)
             bndx_select_row(index, path);
         gtk_tree_path_free(path);
@@ -964,15 +1006,6 @@ bndx_mailbox_row_inserted_cb(LibBalsaMailbox * mailbox, GtkTreePath * path,
 
 /* balsa_index_load_mailbox_node:
    open mailbox_node, the opening is done in thread to keep UI alive.
-   NOTES:
-   it uses module-wide is_opening variable. This variable, as well as the
-   mutex should be a property of BalsaIndex but since we cannot open 
-   two indexes at once because of libmutt limits, we get away with this
-   solution. When the backend is changed, feel free to introduce these
-   changes.
-   Also, the waiting list is not a top hack. I mean it is perfectly 
-   functional (and that's MOST important; think long before modifying it)
-   but perhaps we could write it nicer?
 */
 
 gboolean
@@ -1003,8 +1036,9 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
         gdk_threads_enter();
         if (!balsa_app.main_window)
             return FALSE;
-        if(successp &&
-           !(*err && (*err)->code == LIBBALSA_MAILBOX_TOOMANYOPEN_ERROR))
+
+        if(successp) break;
+        if(*err && (*err)->code != LIBBALSA_MAILBOX_TOOMANYOPEN_ERROR)
             break;
         balsa_mblist_close_lru_peer_mbx(balsa_app.mblist, mailbox);
     } while(try_cnt++<3);
@@ -1027,7 +1061,7 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
     if (libbalsa_mailbox_get_show(mailbox) == LB_MAILBOX_SHOW_TO) {
         GtkTreeViewColumn *column =
 	    gtk_tree_view_get_column(tree_view, LB_MBOX_FROM_COL);
-
+        index->filter_no = 1; /* FIXME: this is hack! */
         gtk_tree_view_column_set_title(column, _("To"));
     }
 
@@ -1038,19 +1072,13 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
 	    	     G_CALLBACK(bndx_mailbox_row_inserted_cb), index);
 
     balsa_window_enable_mailbox_menus(balsa_app.main_window, index);
-    /* libbalsa functions must be called with gdk unlocked
-     * but balsa_index - locked!
-     */
-    gdk_threads_leave();
     libbalsa_mailbox_set_view_filter(mailbox,
                                      balsa_window_get_view_filter
-                                     (balsa_app.main_window), FALSE);
+                                     (balsa_app.main_window, TRUE), FALSE);
     libbalsa_mailbox_set_threading(mailbox,
                                    libbalsa_mailbox_get_threading_type
                                    (mailbox));
 
-    gdk_threads_enter();
-
     /* Set the tree store*/
 #ifndef GTK2_FETCHES_ONLY_VISIBLE_CELLS
     g_object_set_data(G_OBJECT(mailbox), "tree-view", tree_view);
@@ -1389,8 +1417,6 @@ bndx_mailbox_changed_func(BalsaIndex * bindex)
 	&& libbalsa_mailbox_msgno_find(mailbox, mailbox->first_unread,
 				       &path, NULL)) {
 	bndx_expand_to_row(bindex, path);
-	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(bindex), path, NULL,
-		FALSE, 0, 0);
         gtk_tree_path_free(path);
 	mailbox->first_unread = 0;
     }
@@ -1399,8 +1425,6 @@ bndx_mailbox_changed_func(BalsaIndex * bindex)
 	/* The thread containing the current message may have been
 	 * collapsed by rethreading; re-expand it. */
 	bndx_expand_to_row(bindex, path);
-	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(bindex), path, NULL,
-				     FALSE, 0, 0);
         gtk_tree_path_free(path);
     }
 
@@ -1971,10 +1995,55 @@ balsa_index_update_tree(BalsaIndex * index, gboolean expand)
      * deselected the current message */
     if (bndx_find_message(index, NULL, &iter, index->current_message))
         bndx_expand_to_row_and_select(index, &iter);
+    else {
+        /* Collapse can leave the display empty: scroll to one of the
+         * messages that was previously visible. */
+        guint msgno = GPOINTER_TO_UINT(g_object_get_data
+                                       (G_OBJECT(index),
+                                        "balsa-index-scroll-msgno"));
+        GtkTreePath *path;
+        if (msgno 
+            && libbalsa_mailbox_msgno_find(index->mailbox_node->mailbox,
+                                           msgno, &path, NULL)) {
+            bndx_scroll_to_row(index, path);
+            gtk_tree_path_free(path);
+        }
+    }
+    g_object_set_data(G_OBJECT(index), "balsa-index-scroll-msgno", NULL);
+
     bndx_changed_find_row(index);
 }
 
 /* balsa_index_set_threading_type: public method. */
+static void
+bndx_save_scroll_path(BalsaIndex * index)
+{
+    GtkTreeView *tree_view = GTK_TREE_VIEW(index);
+    GdkRectangle rect;
+    GtkTreePath *path;
+
+    gtk_tree_view_get_visible_rect(tree_view, &rect);
+    rect.x += rect.width/2;
+    rect.y += rect.height/2;
+    gtk_tree_view_tree_to_widget_coords(tree_view, rect.x, rect.y,
+                                        &rect.x, &rect.y);
+
+    if (gtk_tree_view_get_path_at_pos(tree_view, rect.x, rect.y, &path,
+                                      NULL, NULL, NULL)) {
+        guint msgno;
+
+        GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
+        GtkTreeIter iter;
+
+        gtk_tree_model_get_iter(model, &iter, path);
+        gtk_tree_path_free(path);
+
+        gtk_tree_model_get(model, &iter, LB_MBOX_MSGNO_COL, &msgno, -1);
+        g_object_set_data(G_OBJECT(index), "balsa-index-scroll-msgno",
+                          GUINT_TO_POINTER(msgno));
+    }
+}
+
 void
 balsa_index_set_threading_type(BalsaIndex * index, int thtype)
 {
@@ -1986,6 +2055,11 @@ balsa_index_set_threading_type(BalsaIndex * index, int thtype)
     mailbox = index->mailbox_node->mailbox;
     g_return_if_fail(mailbox != NULL);
 
+    /* Save a message to scroll back to. */
+    bndx_save_scroll_path(index);
+
+    if (thtype != LB_MAILBOX_THREADING_FLAT) 
+        libbalsa_mailbox_prepare_threading(mailbox, NULL, 0);
     libbalsa_mailbox_set_threading_type(mailbox, thtype);
 
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(index));
@@ -1996,98 +2070,19 @@ balsa_index_set_threading_type(BalsaIndex * index, int thtype)
 }
 
 void
-balsa_index_set_sos_filter(BalsaIndex *bindex, const gchar *sos_filter,
-                           LibBalsaCondition *flag_filter)
+balsa_index_set_view_filter(BalsaIndex *bindex, int filter_no,
+                            const gchar *filter_string,
+                            LibBalsaCondition *filter)
 {
     LibBalsaMailbox * mailbox;
 
     g_return_if_fail(BALSA_IS_INDEX(bindex));
     mailbox = bindex->mailbox_node->mailbox;
 
-    g_free(bindex->sos_filter);
-    bindex->sos_filter = g_strdup(sos_filter);
-
-    if(sos_filter && sos_filter[0] != '\0') {
-	/* Check the To: field for sentbox and From for others */
-        LibBalsaCondition *name = 
-            libbalsa_condition_new_bool_ptr
-            (FALSE, CONDITION_OR,
-             libbalsa_condition_new_string
-             (FALSE, CONDITION_MATCH_SUBJECT, g_strdup(sos_filter), NULL),
-             libbalsa_condition_new_string
-             (FALSE, (libbalsa_mailbox_get_show(mailbox) == LB_MAILBOX_SHOW_TO) ? CONDITION_MATCH_TO : CONDITION_MATCH_FROM,
-	      g_strdup(sos_filter), NULL));
-        
-        if(flag_filter)
-            flag_filter = libbalsa_condition_new_bool_ptr
-                (FALSE, CONDITION_AND, name, flag_filter);
-        else 
-            flag_filter = name;
-    }
-
-    libbalsa_mailbox_set_view_filter(mailbox, flag_filter, TRUE);
-
-}
-
-/* Find messages with the same ID, and remove all but one of them; if
- * any has the `replied' flag set, make sure the one we keep is one of
- * them.
- *
- * NOTE: assumes that index != NULL. */
-void
-balsa_index_remove_duplicates(BalsaIndex * index)
-{
-#if 1
-    g_warning("balsa_index_remove_duplicates requires knowledge of "
-              "msg-id of of all messages\n which may not be available."
-              "This method should be backend-dependent");
-#else
-    LibBalsaMailbox *mailbox;
-    GHashTable *table;
-    GList *list;
-    GList *messages = NULL;
-
-    mailbox = BALSA_INDEX(index)->mailbox_node->mailbox;
-    table = g_hash_table_new(g_str_hash, g_str_equal);
-    for (list = mailbox->message_list; list; list = list->next) {
-        LibBalsaMessage *message = list->data;
-        LibBalsaMessage *master;
-
-        if (LIBBALSA_MESSAGE_IS_DELETED(message) || !message->message_id)
-            continue;
-
-        master =g_hash_table_lookup(table, message->message_id);
-        if (!master || LIBBALSA_MESSAGE_IS_REPLIED(message)) {
-            g_hash_table_insert(table, message->message_id, message);
-            message = master;
-        }
-
-        if (message) {
-            GtkTreePath *path;
-
-            messages = g_list_prepend(messages, message);
-            if (bndx_find_message(index, &path, NULL, message)) {
-                bndx_expand_to_row(index, path);
-                gtk_tree_path_free(path);
-            }
-        }
-    }
-
-    if (messages) {
-	if (mailbox != balsa_app.trash) {
-	    libbalsa_messages_move(messages, balsa_app.trash, NULL);
-	    enable_empty_trash(TRASH_FULL);
-	} else {
-	    libbalsa_messages_change_flag(messages, 
-                                          LIBBALSA_MESSAGE_FLAG_DELETED,
-                                          TRUE);
-            enable_empty_trash(TRASH_CHECK);
-	}
-	g_list_free(messages);
-    }
-
-    g_hash_table_destroy(table);
-#endif
+    g_free(bindex->filter_string);
+    bindex->filter_no = filter_no;
+    bindex->filter_string = g_strdup(filter_string);
+    libbalsa_mailbox_set_view_filter(mailbox, filter, TRUE);
 }
 
 /* Public method. */
@@ -2220,8 +2215,7 @@ static void
 bndx_select_row(BalsaIndex * index, GtkTreePath * path)
 {
     gtk_tree_view_set_cursor(GTK_TREE_VIEW(index), path, NULL, FALSE);
-    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(index), path, NULL,
-                                 FALSE, 0, 0);
+    bndx_scroll_to_row(index, path);
 }
 
 /* Check that all parents are expanded. */
@@ -2316,7 +2310,9 @@ bndx_pipe(LibBalsaMailbox * mailbox, guint msgno, const gchar * pipe_cmd)
 
         pipe = g_mime_stream_file_new(fprog);
         g_mime_stream_file_set_owner(GMIME_STREAM_FILE(pipe), FALSE);
+        libbalsa_mailbox_lock_store(mailbox);
         g_mime_stream_write_to_stream(stream, pipe);
+        libbalsa_mailbox_unlock_store(mailbox);
         g_object_unref(pipe);
         g_object_unref(stream);
         if (pclose(fprog) == -1)
@@ -2434,6 +2430,7 @@ balsa_index_pipe(BalsaIndex * index)
                                     _("_Run"), GTK_RESPONSE_OK,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     NULL);
+    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
 
     vbox = GTK_DIALOG(dialog)->vbox;
     gtk_box_set_spacing(GTK_BOX(vbox), HIG_PADDING);
