@@ -76,7 +76,6 @@ extern GtkWidget *fe_name_label;
 extern GtkWidget *fe_name_entry;
     
 /* widgets for the matching fields */
-GtkWidget *fe_match_frame;
 GtkWidget *fe_matching_fields_body;
 GtkWidget *fe_matching_fields_to;
 GtkWidget *fe_matching_fields_from;
@@ -283,20 +282,27 @@ fe_combo_box_get_value(GtkWidget * combo_box)
  * When they are toggled, the notebook page must change
  */
 static void
-fe_typesmenu_cb(GtkWidget* widget, gpointer data)
+fe_typesmenu_cb(GtkWidget * widget, GtkWidget * field_frame)
 {
     ConditionMatchType type =
-	(ConditionMatchType) fe_combo_box_get_value(widget);
+        (ConditionMatchType) fe_combo_box_get_value(widget);
 
-    condition_has_changed=TRUE;
+    condition_has_changed = TRUE;
     gtk_notebook_set_current_page(GTK_NOTEBOOK(fe_type_notebook),
                                   type - 1);
-    /* For certain types (date, flag)
-     * match fields have no meaning so we disable them
-     */
-    gtk_widget_set_sensitive(fe_match_frame,
-                             (type!=CONDITION_DATE && type!=CONDITION_FLAG));
-}                       /* end fe_typesmenu_cb() */
+
+    switch (type) {
+    case CONDITION_STRING:
+    case CONDITION_REGEX:
+        gtk_widget_show(field_frame);
+        break;
+    case CONDITION_DATE:
+    case CONDITION_FLAG:
+        gtk_widget_hide(field_frame);
+    default:
+        break;
+    }
+}                               /* end fe_typesmenu_cb() */
 
 typedef struct {
     const gchar * normal_str, *negate_str;
@@ -609,7 +615,7 @@ condition_validate(LibBalsaCondition* new_cnd)
         for (row=0;row<2;row++)
             for (col=0;col<2;col++)
                 new_cnd->match.flags|=
-                    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_type_flag_buttons[row*2+col])) ? 1 << (row*2+col+1): 0;
+                    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_type_flag_buttons[row*2+col])) ? 1 << (row*2+col): 0;
 
     case CONDITION_NONE:
     case CONDITION_AND: /*FIXME: verify this! */
@@ -646,12 +652,27 @@ clear_condition_widgets()
 static void
 set_button_sensitivities(gboolean sensitive)
 {
+    GtkTreeModel *condition_model =
+        gtk_tree_view_get_model(fe_conditions_list);
+    GtkTreeSelection *filter_selection =
+        gtk_tree_view_get_selection(fe_filters_list);
+    gint n_conditions;
+    gboolean filter_selected;
+    gboolean filter_ok;
+
     gtk_widget_set_sensitive(fe_apply_button, sensitive);
     gtk_widget_set_sensitive(fe_revert_button, sensitive);
-    gtk_widget_set_sensitive(fe_new_button, !sensitive);
+
+    n_conditions = gtk_tree_model_iter_n_children(condition_model, NULL);
+    filter_selected =
+        gtk_tree_selection_get_selected(filter_selection, NULL, NULL);
+    filter_ok = !sensitive && (n_conditions > 0 || !filter_selected);
+
+    gtk_widget_set_sensitive(fe_new_button, filter_ok);
     gtk_dialog_set_response_sensitive(GTK_DIALOG(fe_window),
-                                      GTK_RESPONSE_OK, !sensitive);
-    gtk_widget_set_sensitive(GTK_WIDGET(fe_filters_list), !sensitive);
+                                      GTK_RESPONSE_OK, filter_ok);
+    gtk_widget_set_sensitive(GTK_WIDGET(fe_filters_list), filter_ok);
+    gtk_widget_set_sensitive(fe_op_codes_option_menu, n_conditions > 1);
 }
 
 /* fill_condition_widget : Fill all widget according to condition data
@@ -688,7 +709,6 @@ fill_condition_widgets(LibBalsaCondition* cnd)
     /* First update matching fields
      * but if type is date or flag, these are meaning less so we disable them */
     andmask = (cnd->type!=CONDITION_FLAG && cnd->type!=CONDITION_DATE);
-    gtk_widget_set_sensitive(fe_match_frame,andmask);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_matching_fields_body),
                                  CONDITION_CHKMATCH(cnd,CONDITION_MATCH_BODY) && andmask);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_matching_fields_to),
@@ -754,7 +774,7 @@ fill_condition_widgets(LibBalsaCondition* cnd)
         for (row=0;row<2;row++)
             for (col=0;col<2;col++)
                 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_type_flag_buttons[row*2+col]),
-                                             cnd->match.flags & (1 << (row*2+col+1)));
+                                             cnd->match.flags & (1 << (row*2+col)));
         fe_update_label(fe_type_flag_label,   &flags_label);
         break;
     case CONDITION_NONE:
@@ -787,7 +807,7 @@ condition_dialog_response(GtkWidget * dialog, gint response,
                 return;
             }
 
-            /* No error occured, condition is valid, so change/add it
+            /* No error occurred, condition is valid, so change/add it
              * based on is_new_condition and only if something has
              * changed of course */
             selection = gtk_tree_view_get_selection(fe_conditions_list);
@@ -842,15 +862,85 @@ condition_dialog_response(GtkWidget * dialog, gint response,
     gtk_widget_set_sensitive(fe_window, TRUE);
 }
 
-/* build_type_notebook
- * build the notebook containing one page for each condition type (simple, regex, date, flag)
+/* build_type_notebook() builds the notebook containing one page for each
+ * condition type (simple, regex, date, flag)
  */
-static void build_type_notebook()
+static void
+add_button(GtkWidget *table, const char *label, int row,
+           GCallback action, int p)
+{
+    GtkWidget* button = gtk_button_new_with_mnemonic(label);
+    gtk_table_attach(GTK_TABLE(table),
+                     button,
+                     row, row+1, 4, 5,
+                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2); 
+    g_signal_connect(G_OBJECT(button), "clicked",
+                     action,
+                     GINT_TO_POINTER(p));
+}
+static GtkWidget*
+add_check(GtkWidget *table, const char *label, int col, int row)
+{
+    GtkWidget* res = gtk_check_button_new_with_mnemonic(label);
+    gtk_table_attach(GTK_TABLE(table), res,
+                     col, col+1, row, row+1,
+                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
+    g_signal_connect(G_OBJECT(res), "toggled",
+                     G_CALLBACK(fe_condition_changed_cb), NULL);
+    return res;
+}
+
+static GtkWidget*
+get_field_frame(void)
+{
+    GtkWidget *table;
+    GtkWidget *frame = gtk_frame_new(_("Match Fields"));
+    GList *list;
+
+    gtk_frame_set_label_align(GTK_FRAME(frame), 
+                              GTK_POS_LEFT, GTK_POS_TOP);
+    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
+    
+    table = gtk_table_new(5, 2, TRUE);
+    gtk_container_add(GTK_CONTAINER(frame), table);
+
+    add_button(table, _("_All"),  0,G_CALLBACK(fe_match_fields_buttons_cb),1);
+    add_button(table, _("C_lear"),1,G_CALLBACK(fe_match_fields_buttons_cb),3);
+
+    fe_matching_fields_body    = add_check(table, _("_Body"),    0,0);
+    fe_matching_fields_to      = add_check(table, _("_To:"),     0,1);
+    fe_matching_fields_from    = add_check(table, _("_From:"),   1,1);
+    fe_matching_fields_subject = add_check(table, _("_Subject"), 0,2);
+    fe_matching_fields_cc      = add_check(table,  _("_Cc:"),    1,2);
+    fe_matching_fields_us_head =
+        gtk_check_button_new_with_mnemonic(_("_User header:"));
+    gtk_table_attach(GTK_TABLE(table),
+                     fe_matching_fields_us_head,
+                     0, 1, 3, 4,
+                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
+    g_signal_connect(G_OBJECT(fe_matching_fields_us_head), "toggled",
+                     G_CALLBACK(fe_match_field_user_header_cb), NULL);
+
+    fe_user_header = gtk_combo_box_entry_new_text();
+    for (list = fe_user_headers_list; list; list = list->next)
+        gtk_combo_box_append_text(GTK_COMBO_BOX(fe_user_header),
+                                  list->data);
+    g_signal_connect(G_OBJECT(fe_user_header), "changed",
+                     G_CALLBACK(fe_condition_changed_cb), NULL);
+    gtk_table_attach(GTK_TABLE(table),
+                     fe_user_header,
+                     1, 2, 3, 4,
+                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
+    return frame;
+}
+
+static void
+build_type_notebook()
 {
     GtkWidget *page,*frame;
     GtkWidget *scroll;
     GtkWidget *box;
-    GtkWidget *button;
+    GtkWidget *button, *table;
     gint row,col;
     static gchar * flag_names[]=
         {N_("Unread"), N_("Deleted"), N_("Replied"), N_("Flagged")};
@@ -863,19 +953,20 @@ static void build_type_notebook()
 
     /* The simple page of the notebook */
 
-    page = gtk_table_new(5, 3, FALSE);
-    gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), page, NULL);
+    box = gtk_vbox_new(FALSE, 5);
+    table = gtk_table_new(7, 2, FALSE);
+    gtk_box_pack_start_defaults(GTK_BOX(box), table);
 
     fe_type_simple_label = 
 	gtk_label_new_with_mnemonic(_("One of the specified f_ields contains"));
-    gtk_table_attach(GTK_TABLE(page),
+    gtk_table_attach(GTK_TABLE(table),
                      fe_type_simple_label,
                      0, 5, 0, 1,
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
     fe_type_simple_entry = gtk_entry_new();
     g_signal_connect(G_OBJECT(fe_type_simple_entry), "changed",
                      G_CALLBACK(fe_condition_changed_cb), NULL);
-    gtk_table_attach(GTK_TABLE(page),
+    gtk_table_attach(GTK_TABLE(table),
                      fe_type_simple_entry,
                      0, 5, 1, 2,
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
@@ -885,15 +976,20 @@ static void build_type_notebook()
     button = gtk_button_new_with_mnemonic(_("Contain/Does _Not Contain"));
     g_signal_connect(G_OBJECT(button), "clicked",
                      G_CALLBACK(fe_negate_condition), NULL);
-    gtk_table_attach(GTK_TABLE(page),
+    gtk_table_attach(GTK_TABLE(table),
                      button,
                      0, 5, 2, 3,
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
+
+    gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), box, NULL);
     
     /* The regex page of the type notebook */
+    box = gtk_vbox_new(FALSE, 5);
 
     page = gtk_table_new(5, 6, FALSE);
-    gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), page, NULL);
+    gtk_box_pack_start_defaults(GTK_BOX(box), page);
+
+    gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), box, NULL);
 
     fe_type_regex_label = 
         gtk_label_new_with_mnemonic(_("_One of the regular expressions matches"));
@@ -948,7 +1044,7 @@ static void build_type_notebook()
     gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), page, NULL);
 
     fe_type_date_label = 
-        gtk_label_new(_("Match when date is in the interval:"));
+        gtk_label_new(_("Match when message date is in the interval:"));
     gtk_table_attach(GTK_TABLE(page),
                      fe_type_date_label,
                      0, 5, 0, 1,
@@ -1027,117 +1123,27 @@ static void build_type_notebook()
 static
 void build_condition_dialog(GtkWidget * condition_dialog)
 {
-    GtkWidget * table,* label,* button,* page,* box;
-    GList *list;
+    GtkWidget *label,* box;
+    GtkWidget *field_frame = get_field_frame();
 
-    page = gtk_table_new(7, 2, FALSE);
-    /* builds the toggle buttons to specify fields concerned by the conditions of
-     * the filter */
-    
-    fe_match_frame = gtk_frame_new(_("Match In"));
-    gtk_frame_set_label_align(GTK_FRAME(fe_match_frame), 
-                              GTK_POS_LEFT, GTK_POS_TOP);
-    gtk_frame_set_shadow_type(GTK_FRAME(fe_match_frame), GTK_SHADOW_ETCHED_IN);
-    gtk_table_attach(GTK_TABLE(page),
-                     fe_match_frame,
-                     0, 2, 0, 2,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
-    
-    table = gtk_table_new(5, 2, TRUE);
-    gtk_container_add(GTK_CONTAINER(fe_match_frame), table);
-    
-    button = gtk_button_new_with_mnemonic(_("_All"));
-    gtk_table_attach(GTK_TABLE(table),
-                     button,
-                     0, 1, 4, 5,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2); 
-    g_signal_connect(G_OBJECT(button), "clicked",
-                     G_CALLBACK(fe_match_fields_buttons_cb),
-                     GINT_TO_POINTER(1));
-    button = gtk_button_new_with_mnemonic(_("C_lear"));
-    gtk_table_attach(GTK_TABLE(table),
-                     button,
-                     1, 2, 4, 5,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(button), "clicked",
-                     G_CALLBACK(fe_match_fields_buttons_cb),
-                     GINT_TO_POINTER(3));
-    
-    fe_matching_fields_body = gtk_check_button_new_with_mnemonic(_("_Body"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_body,
-                     0, 1, 0, 1,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_body), "toggled",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    fe_matching_fields_to = gtk_check_button_new_with_mnemonic(_("_To:"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_to,
-                     0, 1, 1, 2,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_to), "toggled",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    fe_matching_fields_from = gtk_check_button_new_with_mnemonic(_("_From:"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_from,
-                     1, 2, 1, 2,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_from), "toggled",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    fe_matching_fields_subject = 
-	gtk_check_button_new_with_mnemonic(_("_Subject"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_subject,
-                     0, 1, 2, 3,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_subject), "toggled",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    fe_matching_fields_cc = gtk_check_button_new_with_mnemonic(_("_Cc:"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_cc,
-                     1, 2, 2, 3,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_cc), "toggled",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    fe_matching_fields_us_head = gtk_check_button_new_with_mnemonic(_("_User header:"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_us_head,
-                     0, 1, 3, 4,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_us_head), "toggled",
-                     G_CALLBACK(fe_match_field_user_header_cb), NULL);
-    fe_user_header = gtk_combo_box_entry_new_text();
-    for (list = fe_user_headers_list; list; list = list->next)
-        gtk_combo_box_append_text(GTK_COMBO_BOX(fe_user_header),
-                                  list->data);
-    g_signal_connect(G_OBJECT(fe_user_header), "changed",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_user_header,
-                     1, 2, 3, 4,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    
+    /* builds the toggle buttons to specify fields concerned by the
+     * conditions of the filter */
     box = gtk_hbox_new(FALSE, 5);
-    gtk_table_attach(GTK_TABLE(page),
-                     box,
-                     0, 2, 2, 3,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
-
     label = gtk_label_new_with_mnemonic(_("Search T_ype:"));
     gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 5);
     fe_search_option_menu = 
         fe_build_option_menu(fe_search_type, ELEMENTS(fe_search_type),
-                             G_CALLBACK(fe_typesmenu_cb));
+                             G_CALLBACK(fe_typesmenu_cb), field_frame);
     gtk_box_pack_start(GTK_BOX(box), fe_search_option_menu, FALSE, FALSE, 5);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), fe_search_option_menu);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(condition_dialog)->vbox),
+                       box, FALSE, FALSE, 2);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(condition_dialog)->vbox),
+                       field_frame, FALSE, FALSE, 2);
 
     build_type_notebook();
-    gtk_table_attach(GTK_TABLE(page),
-                     fe_type_notebook,
-                     0, 2, 3, 7,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, 5, 5);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(condition_dialog)->vbox),page,FALSE,FALSE,2);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(condition_dialog)->vbox),
+                       fe_type_notebook, FALSE, FALSE, 2);
 }
 
 /*
@@ -1411,7 +1417,11 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
             LibBalsaFilter *fil;
 
             gtk_tree_model_get(model, &iter, 1, &fil, -1);
-            balsa_app.filters = g_slist_prepend(balsa_app.filters, fil);
+            if (fil->condition)
+                balsa_app.filters = g_slist_prepend(balsa_app.filters, fil);
+            else
+                libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                                     _("Filter with no condition was omitted"));
         }
 
         /* Tell the clean-up functions not to free
@@ -1673,7 +1683,7 @@ fe_new_pressed(GtkWidget * widget, gpointer data)
 
     /* Adds "New Filter" to the list of actual new filters names */
     new_filters_names=g_list_prepend(new_filters_names,g_strdup(new_item));
-    gtk_widget_grab_focus(fe_name_label);
+    gtk_widget_grab_focus(fe_name_entry);
 }                       /* end fe_new_pressed() */
 
 /*
@@ -1709,9 +1719,6 @@ fe_delete_pressed(GtkWidget * widget, gpointer data)
         gtk_tree_selection_select_path(selection, path);
     else {
         /* ...the store is empty: */
-        /* We make the filters delete,revert,apply buttons unsensitive */
-        gtk_widget_set_sensitive(fe_delete_button,FALSE);
-        set_button_sensitivities(FALSE);
         /* We clear all widgets */
         gtk_entry_set_text(GTK_ENTRY(fe_name_entry),"");
         gtk_entry_set_text(GTK_ENTRY(fe_popup_entry),"");
@@ -1724,8 +1731,13 @@ fe_delete_pressed(GtkWidget * widget, gpointer data)
                               (fe_conditions_list)));
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_sound_button),FALSE);
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_popup_button),FALSE);
+        /* We make the filters delete,revert,apply buttons unsensitive */
+        gtk_widget_set_sensitive(fe_delete_button,FALSE);
+        set_button_sensitivities(FALSE);
     }
     gtk_tree_path_free(path);
+    gtk_dialog_set_response_sensitive(GTK_DIALOG(fe_window),
+                                      GTK_RESPONSE_OK, TRUE);
 }                       /* end fe_delete_pressed() */
 
 /*
@@ -1869,6 +1881,8 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
     gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                        0, fil->name, 1, fil, -1);
     gtk_tree_selection_select_iter(selection, &iter);
+    gtk_dialog_set_response_sensitive(GTK_DIALOG(fe_window),
+                                      GTK_RESPONSE_OK, TRUE);
 }                       /* end fe_apply_pressed */
 
 
@@ -1989,6 +2003,9 @@ fe_filters_list_selection_changed(GtkTreeSelection * selection,
         gtk_widget_set_sensitive(fe_op_codes_option_menu,
                                  gtk_tree_model_iter_n_children(model,
                                                                 NULL) > 1);
+    } else {
+        gtk_widget_set_sensitive(fe_condition_edit_button, FALSE);
+        gtk_widget_set_sensitive(fe_condition_delete_button, FALSE);
     }
 
     /* We set the filters delete,revert,apply buttons sensitivity */
