@@ -381,6 +381,8 @@ imap_scan_attach_mailbox(BalsaMailboxNode * mbnode, imap_scan_item * isi)
 					 (gpointer) isi->special);
         *isi->special = LIBBALSA_MAILBOX(m);
 	g_object_add_weak_pointer(G_OBJECT(m), (gpointer) isi->special);
+        if (isi->special == &balsa_app.outbox)
+            LIBBALSA_MAILBOX(m)->no_reassemble = TRUE;
     }
 
     return TRUE;
@@ -426,7 +428,7 @@ imap_dir_cb_real(void* r)
                               &error);
     
     if(error) {
-        libbalsa_information(LIBBALSA_INFORMATION_ERROR,
+        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
                              error->code == LIBBALSA_MAILBOX_NETWORK_ERROR
                              ? _("Scanning of %s failed: %s\n"
                                  "Check network connectivity.")
@@ -464,8 +466,8 @@ imap_dir_cb_real(void* r)
     if(balsa_app.debug && mb->name)
         printf("imap_dir_cb:  main mailbox node %s mailbox is %p\n", 
                mb->name, mb->mailbox);
-    gnome_appbar_pop(balsa_app.appbar);
     if(balsa_app.debug) printf("%d: Scanning done.\n", (int)time(NULL));
+    gnome_appbar_pop(balsa_app.appbar);
     return NULL;
 }
 
@@ -543,6 +545,8 @@ balsa_mailbox_node_new_from_config(const gchar* group)
     if(balsa_app.debug)
 	printf("Server loaded, host: %s, %s.\n", folder->server->host,
 	       folder->server->use_ssl ? "SSL" : "no SSL");
+    g_signal_connect_swapped(G_OBJECT(folder->server), "config-changed", 
+                             G_CALLBACK(config_folder_update), folder);
     g_signal_connect(G_OBJECT(folder), "show-prop-dialog", 
 		     G_CALLBACK(folder_conf_imap_node), NULL);
     g_signal_connect(G_OBJECT(folder), "append-subtree", 
