@@ -50,7 +50,7 @@ struct _BalsaWindow {
     GtkWidget *hpaned;
     GtkWidget *vpaned;
     GtkWidget *current_index;
-    GtkWidget *filter_label;
+    GtkWidget *filter_choice;
     LibBalsaMessage *current_message;
 };
 
@@ -71,19 +71,24 @@ typedef enum {
     TRASH_CHECK  /* uncertain, better check */
 } TrashState;
 
+enum {
+    FILTER_SENDER  = 0,
+    FILTER_RECIPIENT = 1
+};
 
 GtkType balsa_window_get_type(void);
 GtkWidget *balsa_window_new(void);
 GtkWidget *balsa_window_find_current_index(BalsaWindow * window);
 void balsa_window_enable_mailbox_menus(BalsaWindow * window,
 				       BalsaIndex * index);
-LibBalsaCondition* balsa_window_get_view_filter(BalsaWindow *window);
+void balsa_window_update_book_menus(BalsaWindow *window);
+LibBalsaCondition* balsa_window_get_view_filter(BalsaWindow *window,
+                                                gboolean flags_only);
 void balsa_window_refresh(BalsaWindow * window);
 void balsa_window_open_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
 void balsa_window_close_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
 void balsa_identities_changed(BalsaWindow *bw);
 
-void balsa_window_set_filter_label(BalsaWindow * window, gboolean to_field);
 void balsa_window_update_tab(BalsaMailboxNode * mbnode);
 void enable_empty_trash(BalsaWindow * window, TrashState status);
 void balsa_window_enable_continue(BalsaWindow * window);
