@@ -150,12 +150,6 @@ balsa_handle_automation_options() {
            printf("Unread: %ld Unsent: %ld\n", (long)unread, (long)unsent);
        }
 
-       if (cmd_get_stats) {
-           CORBA_long unread = 0, unsent = 0;
-	   GNOME_Balsa_Application_getStats (app, &unread, &unsent, &ev);
-           printf("Unread: %ld Unsent: %ld\n", (long)unread, (long)unsent);
-       }
-
        if (cmd_line_open_mailboxes)
 	   GNOME_Balsa_Application_openMailbox (app,
 					       cmd_line_open_mailboxes,
@@ -187,12 +181,13 @@ balsa_handle_automation_options() {
 
 	   GNOME_Balsa_Composer_sendMessage(server,
 					    "",
-					    opt_compose_email,
+					    opt_compose_email ? opt_compose_email : "",
 					    "",
 					    "",
 					    attachs,
 					    0, 
 					    &ev );
+           CORBA_exception_free( &ev );
        }
 
        exit(0);
@@ -325,23 +320,7 @@ mailboxes_init(gboolean check_only)
     }
 }
 
-
 #ifdef BALSA_USE_THREADS
-/* Recursive mutex for gdk_threads_{enter,leave}. */
-static pthread_mutex_t balsa_threads_mutex;
-
-static void
-balsa_threads_enter(void)
-{
-    pthread_mutex_lock(&balsa_threads_mutex);
-}
-
-static void
-balsa_threads_leave(void)
-{
-    gdk_display_flush(gdk_display_get_default());
-    pthread_mutex_unlock(&balsa_threads_mutex);
-}
 
 static void
 threads_init(void)
@@ -354,9 +333,8 @@ threads_init(void)
         g_warning("pthread_mutexattr_init failed with %d\n", status);
     pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
     pthread_mutex_init(&mailbox_lock, &attr);
-    pthread_mutex_init(&balsa_threads_mutex, &attr);
-    gdk_threads_set_lock_functions(G_CALLBACK(balsa_threads_enter),
-                                   G_CALLBACK(balsa_threads_leave));
+
+    libbalsa_threads_init();
     gdk_threads_init();
 
     pthread_mutexattr_destroy(&attr);
@@ -389,7 +367,7 @@ threads_destroy(void)
 {
     pthread_mutex_destroy(&mailbox_lock);
     pthread_mutex_destroy(&send_messages_lock);
-    pthread_mutex_destroy(&balsa_threads_mutex);
+    libbalsa_threads_destroy();
 }
 
 #endif				/* BALSA_USE_THREADS */
@@ -408,8 +386,7 @@ initial_open_unread_mailboxes()
     if (gl) {
         for (i = g_list_first(gl); i; i = g_list_next(i)) {
             printf("opening %s..\n", (LIBBALSA_MAILBOX(i->data))->name);
-            if(0)
-                balsa_mblist_open_mailbox(LIBBALSA_MAILBOX(i->data));
+            balsa_mblist_open_mailbox(LIBBALSA_MAILBOX(i->data));
         }
         g_list_free(gl);
     }
@@ -511,9 +488,9 @@ mbnode_expunge_func(GtkTreeModel *model, GtkTreePath *path,
 {
     BalsaMailboxNode *mbnode;
 
+    gtk_tree_model_get(model, iter, 0, &mbnode, -1);
     g_return_val_if_fail(mbnode, FALSE);
 
-    gtk_tree_model_get(model, iter, 0, &mbnode, -1);
     if (mbnode->mailbox && libbalsa_mailbox_is_open(mbnode->mailbox)) {
         time_t tm = time(NULL);
         if(tm-mbnode->last_use > balsa_app.expunge_timeout)
@@ -561,12 +538,7 @@ main(int argc, char *argv[])
     bindtextdomain(PACKAGE, GNOMELOCALEDIR);
     bind_textdomain_codeset(PACKAGE, "UTF-8");
     textdomain(PACKAGE);
-#if 0	/* FIXME replacement? */
-    /* FIXME: gnome_i18n_get_language seems to have gone away; 
-     * is this a reasonable replacement? */
-    setlocale(LC_CTYPE,
-              (const char *) gnome_i18n_get_language_list("LC_CTYPE")->data);
-#endif
+    setlocale(LC_ALL, "");
 #endif
 
 
@@ -614,9 +586,7 @@ main(int argc, char *argv[])
 
     default_icon = balsa_pixmap_finder("balsa_icon.png");
     if(default_icon) { /* may be NULL for developer installations */
-#if 0	/* FIXME replacement? */
-        gnome_window_icon_set_default_from_file(default_icon);
-#endif
+        gtk_window_set_default_icon_from_file(default_icon, NULL);
         g_free(default_icon);
     }
 
