@@ -63,7 +63,6 @@ static void load_color(gchar * key, GdkColor * color);
 static void save_mru(GList  *mru, const gchar * group);
 static void load_mru(GList **mru, const gchar * group);
 
-static void config_address_books_load(void);
 static void config_address_books_save(void);
 static void config_identities_load(void);
 
@@ -171,6 +170,8 @@ load_toolbars(void)
     GSList **list;
     guint changed = 0;
 
+    tmpkey[sizeof tmpkey - 1] = '\0';
+
     libbalsa_conf_push_group("Toolbars");
     balsa_app.toolbar_wrap_button_text = d_get_gint("WrapButtonText", 1);
     libbalsa_conf_pop_group();
@@ -210,14 +211,14 @@ load_toolbars(void)
     for (i = 0; i < ELEMENTS(toolbars); i++) {
         guint type;
 
-        sprintf(tmpkey, "Toolbar%dID", i);
+        snprintf(tmpkey, sizeof tmpkey - 1, "Toolbar%dID", i);
         type = d_get_gint(tmpkey, -1);
 
         if (type >= ELEMENTS(toolbars)) {
             continue;
         }
 
-        sprintf(tmpkey, "Toolbar%dItemCount", i);
+        snprintf(tmpkey, sizeof tmpkey - 1, "Toolbar%dItemCount", i);
         items = d_get_gint(tmpkey, 0);
 
         list = toolbars[type].current;
@@ -225,7 +226,7 @@ load_toolbars(void)
             gchar *item;
 	    const gchar *real_item;
 
-            sprintf(tmpkey, "Toolbar%dItem%d", i, j);
+            snprintf(tmpkey, sizeof tmpkey - 1, "Toolbar%dItem%d", i, j);
             item = libbalsa_conf_get_string(tmpkey);
  	    if ((real_item = balsa_toolbar_sanitize_id(item))) {
  		*list = g_slist_append(*list, g_strdup(real_item));
@@ -475,29 +476,15 @@ pop3_progress_notify(LibBalsaMailbox* mailbox, int msg_type, int prog, int tot,
        * blocking on the mutt_lock, andthe pipe filling up.
        * This would give us a deadlock.
      */
-    write(mail_thread_pipes[1], (void *) &message, sizeof(void *));
+    if (write(mail_thread_pipes[1], (void *) &message, sizeof(void *))
+        != sizeof(void *))
+        g_warning("pipe error");
 #else
     while(gtk_events_pending())
         gtk_main_iteration_do(FALSE);
 #endif
 }
 
-static void
-pop3_config_updated(LibBalsaMailboxPop3* mailbox)
-{
-#ifdef BALSA_USE_THREADS
-    MailThreadMessage *threadmsg;
-    threadmsg = g_new(MailThreadMessage, 1);
-    threadmsg->message_type = LIBBALSA_NTFY_UPDATECONFIG;
-    threadmsg->mailbox = (void *) mailbox;
-    threadmsg->message_string[0] = '\0';
-    write(mail_thread_pipes[1], (void *) &threadmsg,
-          sizeof(void *));
-#else
-    config_mailbox_update(LIBBALSA_MAILBOX(mailbox));
-#endif
-}
-
 /* Initialize the specified mailbox, creating the internal data
    structures which represent the mailbox. */
 static gint
@@ -515,15 +502,12 @@ config_mailbox_init(const gchar * prefix)
 	LibBalsaServer *server = LIBBALSA_MAILBOX_REMOTE_SERVER(mailbox);
         libbalsa_server_connect_signals(server,
                                         G_CALLBACK(ask_password), mailbox);
-	g_signal_connect_swapped(server, "set-host",
+	g_signal_connect_swapped(server, "config-changed",
                                  G_CALLBACK(config_mailbox_update),
 				 mailbox);
     }
 
     if (LIBBALSA_IS_MAILBOX_POP3(mailbox)) {
-        g_signal_connect(G_OBJECT(mailbox),
-                         "config-changed", G_CALLBACK(pop3_config_updated),
-                         mailbox);
         g_signal_connect(G_OBJECT(mailbox),
                          "progress-notify", G_CALLBACK(pop3_progress_notify),
                          mailbox);
@@ -537,15 +521,16 @@ config_mailbox_init(const gchar * prefix)
 	mbnode = balsa_mailbox_node_new_from_mailbox(mailbox);
 	if (strcmp(INBOX_NAME, key) == 0)
 	    special = &balsa_app.inbox;
-	else if (strcmp(OUTBOX_NAME, key) == 0)
+	else if (strcmp(OUTBOX_NAME, key) == 0) {
 	    special = &balsa_app.outbox;
-	else if (strcmp(SENTBOX_NAME, key) == 0)
+            mailbox->no_reassemble = TRUE;
+        } else if (strcmp(SENTBOX_NAME, key) == 0)
 	    special = &balsa_app.sentbox;
 	else if (strcmp(DRAFTS_NAME, key) == 0)
 	    special = &balsa_app.draftbox;
 	else if (strcmp(TRASH_NAME, key) == 0) {
 	    special = &balsa_app.trash;
-            libbalsa_filters_set_trash(balsa_app.trash);
+            libbalsa_filters_set_trash(mailbox);
 	}
 
 	if (special) {
@@ -572,7 +557,7 @@ config_folder_init(const gchar * prefix)
     g_return_val_if_fail(prefix != NULL, FALSE);
 
     if( (folder = balsa_mailbox_node_new_from_config(prefix)) ) {
-	g_signal_connect_swapped(folder->server, "set-host",
+	g_signal_connect_swapped(folder->server, "config-changed",
                                  G_CALLBACK(config_folder_update),
 				 folder);
 	balsa_mblist_mailbox_node_append(NULL, folder);
@@ -884,6 +869,9 @@ config_global_load(void)
     /* Spelling options ... */
     libbalsa_conf_push_group("Spelling");
 
+#if HAVE_GTKSPELL
+    balsa_app.spell_check = libbalsa_conf_get_bool("SpellCheck=false");
+#else                           /* HAVE_GTKSPELL */
     balsa_app.module = d_get_gint("PspellModule", DEFAULT_PSPELL_MODULE);
     balsa_app.suggestion_mode =
 	d_get_gint("PspellSuggestMode", DEFAULT_PSPELL_SUGGEST_MODE);
@@ -893,6 +881,7 @@ config_global_load(void)
 	d_get_gint("SpellCheckSignature", DEFAULT_CHECK_SIG);
     balsa_app.check_quoted =
 	d_get_gint("SpellCheckQuoted", DEFAULT_CHECK_QUOTED);
+#endif                          /* HAVE_GTKSPELL */
 
     libbalsa_conf_pop_group();
 
@@ -1262,11 +1251,16 @@ config_save(void)
     /* Spelling options ... */
     libbalsa_conf_push_group("Spelling");
 
+#if HAVE_GTKSPELL
+    libbalsa_conf_set_bool("SpellCheck", balsa_app.spell_check);
+    balsa_app.spell_check = libbalsa_conf_get_bool("SpellCheck=false");
+#else                           /* HAVE_GTKSPELL */
     libbalsa_conf_set_int("PspellModule", balsa_app.module);
     libbalsa_conf_set_int("PspellSuggestMode", balsa_app.suggestion_mode);
     libbalsa_conf_set_int("PspellIgnoreSize", balsa_app.ignore_size);
     libbalsa_conf_set_int("SpellCheckSignature", balsa_app.check_sig);
     libbalsa_conf_set_int("SpellCheckQuoted", balsa_app.check_quoted);
+#endif                          /* HAVE_GTKSPELL */
 
     libbalsa_conf_pop_group();
 
@@ -1467,7 +1461,7 @@ config_address_book_load(const gchar * key, const gchar * value,
     return FALSE;
 }
 
-static void
+void
 config_address_books_load(void)
 {
     gchar *default_address_book_prefix;
@@ -1809,8 +1803,9 @@ config_filter_load(const gchar * key, const gchar * value, gpointer data)
     char *endptr;
     guint *save = data;
     LibBalsaFilter *fil;
+    long int dummy;
 
-    strtol(value, &endptr, 10);
+    dummy = strtol(value, &endptr, 10);
     if (*endptr) {              /* Bad format. */
         libbalsa_conf_remove_group(key);
         return FALSE;
@@ -1830,12 +1825,12 @@ config_filter_load(const gchar * key, const gchar * value, gpointer data)
                 /* Some 2.0.x versions had a new action code which
                  * changed the value of FILTER_TRASH. */
                 fil->action = FILTER_TRASH;
-            ++save;
+            ++*save;
         }
     }
     if (!fil->condition) {
         g_idle_add((GSourceFunc) config_warning_idle,
-                   _("Filter with no condition was ignored"));
+                   _("Filter with no condition was omitted"));
         libbalsa_filter_free(fil, GINT_TO_POINTER(FALSE));
     } else {
         FILTER_SETFLAG(fil, FILTER_VALID);
@@ -1886,7 +1881,7 @@ config_filters_save(void)
     libbalsa_conf_sync();
     /* This loop takes care of cleaning up old filter sections */
     while (TRUE) {
-	i=snprintf(tmp,tmp_len,"%d/",nb++);
+	i=snprintf(tmp,tmp_len,"%d",nb++);
 	if (libbalsa_conf_has_group(buffer)) {
 	    libbalsa_conf_remove_group(buffer);
 	}
@@ -1945,11 +1940,13 @@ load_mru(GList **mru, const gchar * group)
     int count, i;
     char tmpkey[32];
     
+    tmpkey[sizeof tmpkey - 1] = '\0';
+
     libbalsa_conf_push_group(group);
     count=d_get_gint("MRUCount", 0);
     for(i=0;i<count;i++) {
         gchar *val;
-	sprintf(tmpkey, "MRU%d", i+1);
+	snprintf(tmpkey, sizeof tmpkey - 1, "MRU%d", i + 1);
         if( (val = libbalsa_conf_get_string(tmpkey)) != NULL )
             (*mru)=g_list_append((*mru), val);
     }
@@ -1963,9 +1960,11 @@ save_mru(GList * mru, const gchar * group)
     char tmpkey[32];
     GList *ltmp;
 
+    tmpkey[sizeof tmpkey - 1] = '\0';
+
     libbalsa_conf_push_group(group);
     for (ltmp = mru, i = 0; ltmp; ltmp = ltmp->next, i++) {
-        sprintf(tmpkey, "MRU%d", i + 1);
+	snprintf(tmpkey, sizeof tmpkey - 1, "MRU%d", i + 1);
         libbalsa_conf_set_string(tmpkey, (gchar *) (ltmp->data));
     }
 
