@@ -61,10 +61,14 @@ extern "C" {
         BalsaSendmsgAddress reply_to_info;
 #endif
 	GtkWidget *attachments[4];
+        gchar *in_reply_to;
 	GtkWidget *text;
+#if !HAVE_GTKSPELL
 	GtkWidget *spell_checker;
+#endif                          /* HAVE_GTKSPELL */
 	GtkWidget *notebook;
-	LibBalsaMessage *orig_message;
+	LibBalsaMessage *parent_message; /* to which we're replying     */
+	LibBalsaMessage *draft_message;  /* where the message was saved */
 	SendType type;
 	/* language selection related data */
 	gchar *charset;
@@ -84,7 +88,9 @@ extern "C" {
 	gulong delete_sig_id;
         gulong changed_sig_id;
         gulong delete_range_sig_id;
+        gulong insert_text_sig_id;
         guint wrap_timeout_id;
+        guint autosave_timeout_id;
         gulong identities_changed_id;
 	gboolean modified;
 	gboolean flow;          /* send format=flowed */ 
@@ -104,6 +110,9 @@ extern "C" {
         GtkTextBuffer *buffer2;       /* Undo buffer. */
         GtkWidget *undo_widget;
         GtkWidget *redo_widget;
+
+        /* To update cursor after text is inserted. */
+        GtkTextMark *insert_mark;
     };
 
     BalsaSendmsg *sendmsg_window_new(GtkWidget *, LibBalsaMessage *,
