@@ -771,7 +771,7 @@ static regex_t quoted_rex;
 static gboolean quoted_rex_compiled = FALSE;
 
 void
-balsa_spell_check_start(BalsaSpellCheck * spell_check)
+balsa_spell_check_start(BalsaSpellCheck * spell_check, GtkWindow *parent_wnd)
 {
     PspellCanHaveError *spell_error;
     gchar *string;
@@ -807,8 +807,9 @@ balsa_spell_check_start(BalsaSpellCheck * spell_check)
 
     if (pspell_error_number(spell_error) != 0) {
 	/* quit without breaking things */
-	balsa_information(LIBBALSA_INFORMATION_WARNING,
-			  pspell_error_message(spell_error));
+	balsa_information_parented(parent_wnd,
+                                   LIBBALSA_INFORMATION_ERROR,
+                                   pspell_error_message(spell_error));
 
 	/* Generate a response signal. */
 	gtk_dialog_response(GTK_DIALOG(spell_check), 0);
@@ -975,12 +976,12 @@ balsa_spell_check_learn(BalsaSpellCheck * spell_check,
     /* If result is 0, the learn operation failed */
     if (!result) {
 	if (pspell_manager_error_number(spell_check->spell_manager) != 0) {
-	    balsa_information(BALSA_INFORMATION_SHOW_DIALOG,
+	    balsa_information(LIBBALSA_INFORMATION_ERROR,
 			      "BalsaSpellCheck: Learn operation failed;\n%s\n",
 			      pspell_manager_error_message
 			      (spell_check->spell_manager));
 	} else {
-	    balsa_information(BALSA_INFORMATION_SHOW_DIALOG,
+	    balsa_information(LIBBALSA_INFORMATION_ERROR,
 			      "BalsaSpellCheck: Learn operation failed.\n");
 	}
     }
@@ -1277,7 +1278,7 @@ static gboolean
 check_pspell_errors(PspellManager * manager)
 {
     if (pspell_manager_error_number(manager) != 0) {
-	balsa_information(BALSA_INFORMATION_SHOW_LIST,
+	balsa_information(LIBBALSA_INFORMATION_WARNING,
 			  "BalsaSpellCheck: Pspell Error: %s\n",
 			  pspell_manager_error_message(manager));
 	return TRUE;
